{ $Id: PDOSQL.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOSQL;

{$I directives.inc}

interface

uses PDOClasses, sysUtils;

{$IFDEF ENABLE_MYSQL}
    {$DEFINE COMMON_SQL}
{$ELSE}
{$IFDEF ENABLE_SQLITE}
    {$DEFINE COMMON_SQL}
{$ELSE}
{$IFDEF ENABLE_POSTGRESQL}
    {$DEFINE COMMON_SQL}
{$ENDIF}  //end pgsql
{$ENDIF}  //end sqlite
{$ENDIF}  //end mysql

function assemble_vanilla         (Select: PDO_selectRecord):   AnsiString; overload;
function assemble_vanilla         (Select: PDO_enhancedSelect): Ansistring; overload;
{$IFDEF COMMON_SQL}
function assemble_select_mysql    (Select: PDO_selectRecord):   AnsiString; overload;
function assemble_select_mysql    (Select: PDO_enhancedSelect): AnsiString; overload;
{$ENDIF}
{$IFDEF ENABLE_POSTGRESQL}
function assemble_select_pgsql    (Select: PDO_selectRecord):   AnsiString; overload;
function assemble_select_pgsql    (Select: PDO_enhancedSelect): AnsiString; overload;
{$ENDIF}
{$IFDEF ENABLE_SQLITE}
function assemble_select_sqlite   (Select: PDO_selectRecord):   AnsiString; overload;
function assemble_select_sqlite   (Select: PDO_enhancedSelect): AnsiString; overload;
{$ENDIF}
{$IFDEF ENABLE_DBLIB}
function assemble_select_mssql    (Select: PDO_selectRecord): AnsiString;
{$ENDIF}
{$IFDEF ENABLE_DB2}
function assemble_select_db2      (Select: PDO_selectRecord): AnsiString;
{$ENDIF}
{$IFDEF ENABLE_ORACLE}
function assemble_select_oracle   (Select: PDO_selectRecord): AnsiString;
{$ENDIF}
{$IFDEF ENABLE_FIREBIRD}
function assemble_select_firebird (Select: PDO_selectRecord):   AnsiString; overload;
function assemble_select_firebird (Select: PDO_enhancedSelect): AnsiString; overload;
{$ENDIF}

implementation

function assemble_vanilla (Select: PDO_selectRecord): AnsiString;
var
    condition: ansistring;
    orderby:   ansistring;
begin
    condition := '';
    orderby   := '';

    if (Select.sqlConditions <> '') then condition := 'WHERE '    + select.sqlConditions;
    if (Select.sqlOrderBy <> '')    then orderby   := 'ORDER BY ' + select.sqlOrderBy;

    Result := format ('SELECT %s FROM %s %s %s', [
                        Select.sqlColumns,
                        Select.sqlTables,
                        condition,
                        orderby
               ]);
end;

function assemble_vanilla (Select: PDO_enhancedSelect): Ansistring;
var
    str_condition: ansistring;
    str_orderby:   ansistring;
    str_distinct:  ansistring;
    str_groupby:   ansistring;
    str_having:    ansistring;
    str_from:      ansistring;
begin
    str_distinct  := 'ALL';
    str_condition := '';
    str_orderby   := '';
    str_groupby   := '';
    str_having    := '';
    str_from      := '';

    if Select.DISTINCT = DISTINCT then str_distinct  := 'DISTINCT';
    if Select.FROM  <> ''         then str_from      := 'FROM '     + Select.FROM;
    if Select.WHERE <> ''         then str_condition := 'WHERE '    + Select.WHERE;
    if Select.GROUP <> ''         then str_groupby   := 'GROUP BY ' + Select.GROUP;
    if Select.HAVING <> ''        then str_having    := 'HAVING '   + Select.HAVING;
    if Select.ORDER <> ''         then str_orderby   := 'ORDER BY ' + Select.ORDER;

    Result := format ('SELECT %s %s %s %s %s %s %s', [
                        str_distinct,
                        Select.COLUMNS,
                        str_from,
                        str_condition,
                        str_groupby,
                        str_having,
                        str_orderby
               ]);

end;

{$IFDEF ENABLE_POSTGRESQL}
{PostgreSQL is identical to MySQL in basic SELECT statement syntax}
function assemble_select_pgsql (Select: PDO_selectRecord): AnsiString;
begin
    Result := assemble_select_mysql(Select);
end;

function assemble_select_pgsql (Select: PDO_enhancedSelect): AnsiString;
begin
    Result := assemble_select_mysql(Select);
end;
{$ENDIF}

{$IFDEF ENABLE_SQLITE}
{SQLite is identical to MySQL in basic SELECT statement syntax}
function assemble_select_sqlite (Select: PDO_selectRecord): AnsiString;
begin
    Result := assemble_select_mysql(Select);
end;

function assemble_select_sqlite (Select: PDO_enhancedSelect): AnsiString;
begin
    Result := assemble_select_mysql(Select);
end;
{$ENDIF}

{$IFDEF COMMON_SQL}
function assemble_select_mysql (Select: PDO_selectRecord): AnsiString;
begin
    Result := assemble_vanilla (Select);

    if (Select.sqlOffset = 0) and (Select.sqlLimit > 0) then
        Result := Result + format(' LIMIT %d', [Select.sqlLimit]);

    if (Select.sqlOffset > 0) and (Select.sqlLimit > 0) then
        Result := Result +  format (' LIMIT %d OFFSET %d', [Select.sqlLimit, Select.sqlOffset]);
end;

function assemble_select_mysql (Select: PDO_enhancedSelect): AnsiString;
begin
    Result := assemble_vanilla (Select);

    if (Select.OFFSET = 0) and (Select.LIMIT > 0) then
        Result := Result + format(' LIMIT %d', [Select.LIMIT]);

    if (Select.OFFSET > 0) and (Select.LIMIT > 0) then
        Result := Result +  format (' LIMIT %d OFFSET %d', [Select.LIMIT, Select.OFFSET]);
end;
{$ENDIF}

{$IFDEF ENABLE_DB2}
function assemble_select_db2    (Select: PDO_selectRecord): AnsiString;
begin
    if (Select.sqlOffset > 0) and (Select.sqlLimit > 0) then
        begin
            Result := format ('SELECT * ' +
                              'FROM (SELECT ROW_NUMBER() OVER (ORDER_BY %s) AS RN, %s FROM %s WHERE %s) ' +
                              'WHERE RN > %d AND RN <= %d', [
                                    Select.sqlOrderBy,
                                    Select.sqlColumns,
                                    Select.sqlTables,
                                    Select.sqlConditions,
                                    Select.sqlOffset,
                                    (Select.sqlOffset + Select.sqlLimit)
                     ]);
            exit;
        end;


    Result := assemble_vanilla (Select);
    if (Select.sqlOffset = 0) and (Select.sqlLimit > 0) then
        Result := Result + format(' FETCH FIRST %d ROWS ONLY', [Select.sqlLimit]);
end;
{$ENDIF}

{$IFDEF ENABLE_ORACLE}
function assemble_select_oracle (Select: PDO_selectRecord): AnsiString;
begin
    if (Select.sqlOffset = 0) and (Select.sqlLimit = 0) then
        begin
            Result := assemble_vanilla (Select);
            exit;
        end;

    Result := format ('SELECT * ' +
                      'FROM (SELECT ROW_NUMBER() OVER (ORDER_BY %s) AS RN, %s FROM %s WHERE %s) WHERE ', [
                            Select.sqlOrderBy,
                            Select.sqlColumns,
                            Select.sqlTables,
                            Select.sqlConditions
                ]);

    if (Select.sqlOffset = 0) and (Select.sqlLimit > 0) then
        Result := Result + format ('RN <= %d', [select.sqlLimit + select.sqlOffset]);

    if (Select.sqlOffset > 0) and (Select.sqlLimit > 0) then
        Result := Result + format ('RN > %d AND RN <= %d', [select.sqlOffset ,(select.sqlLimit + select.sqlOffset)]);
end;
{$ENDIF}

{$IFDEF ENABLE_FIREBIRD}
function assemble_select_firebird (Select: PDO_selectRecord): AnsiString;
begin
    Result := assemble_vanilla (Select);
    if (Select.sqlOffset = 0) and (Select.sqlLimit = 0) then exit;

    if (Select.sqlOffset = 0) and (Select.sqlLimit > 0) then
        Result := format ('SELECT FIRST %d', [Select.sqlLimit]) + copy (Result, 7, maxInt);

    if (Select.sqlOffset > 0) and (Select.sqlLimit > 0) then
        Result := format ('SELECT FIRST %d SKIP %d', [Select.sqlLimit, Select.sqlOffset]) + copy (Result, 7, maxInt);

end;

function assemble_select_firebird (Select: PDO_enhancedSelect): AnsiString;
begin
    Result := assemble_vanilla (Select);
    if (Select.OFFSET = 0) and (Select.LIMIT = 0) then exit;

    if (Select.OFFSET = 0) and (Select.LIMIT > 0) then
        Result := format ('SELECT FIRST %d', [Select.LIMIT]) + copy (Result, 7, maxInt);

    if (Select.OFFSET > 0) and (Select.LIMIT > 0) then
        Result := format ('SELECT FIRST %d SKIP %d', [Select.LIMIT, Select.OFFSET]) + copy (Result, 7, maxInt);
end;
{$ENDIF}

{$IFDEF ENABLE_DBLIB}
function assemble_select_mssql  (Select: PDO_selectRecord): AnsiString;
var
    condition:  ansistring;
    orderby:    ansistring;
    test:       ansistring;
    mirror:     ansistring;
    firstOrder: ansistring;
begin
    Result := assemble_vanilla (Select);
    if (Select.sqlOffset = 0) and (Select.sqlLimit = 0) then exit;
                  
    condition := '';
    orderby   := '';

    if (Select.sqlConditions <> '') then condition := 'WHERE '    + select.sqlConditions;
    if (Select.sqlOrderBy <> '')    then orderby   := 'ORDER BY ' + select.sqlOrderBy;

    if (Select.sqlOffset = 0) and (Select.sqlLimit > 0) then
        Result := format ('SELECT TOP %d %s FROM %s %s %s', [
            Select.sqlLimit,
            Select.sqlColumns,
            Select.sqlTables,
            condition,
            orderby
        ]);

     if (Select.sqlOffset > 0) and (Select.sqlLimit > 0) then
        Begin
            firstOrder := select.sqlOrderBy;
            test := uppercase(firstOrder);
            if (ansipos('ASC', test) < 0) and (ansipos('DESC', test) < 0) then
                firstOrder := firstOrder + ' ASC';

                mirror := StringReplace (firstOrder, 'ASC', 'NEW$DESC', [rfReplaceAll, rfIgnoreCase]);
                mirror := StringReplace (mirror, 'DESC', 'ASC', [rfReplaceAll, rfIgnoreCase]);
                mirror := StringReplace (mirror, 'NEW$DESC', 'DESC', [rfReplaceAll, rfIgnoreCase]);

                Result := format ('SELECT * FROM ' +
                                    '(SELECT TOP %d FROM ' +
                                        '(SELECT TOP %d %s FROM %s %s ORDER BY %s) AS L2 ' +
                                     'ORDER BY %s) AS L1 ' +
                                   'ORDER BY %s', [
                                        Select.sqlLimit,
                                        (Select.sqlLimit + Select.sqlOffset),
                                        Select.sqlColumns,
                                        Select.sqlTables,
                                        condition,
                                        orderby,
                                        mirror,
                                        orderby
                            ]);
        end
end;
{$ENDIF}





end.
