{ $Id: PDOFirebirdDriver.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOFirebirdDriver;

interface

{$I ../../directives.inc}

uses
	SysUtils, classes, PDOClasses, PDOInterfaces, PDOConstantsFirebird, PDOLogging, PDOMessages;

type
  TFirebirdBaseDriver = class (TInterfacedObject, IPDOProtocol)
  private
    FSilentMode: Byte;            {0 = silent, 1 = warning, 2 = throw exception}
    FCaseMode: Byte;              {0 = lower, 2 = natural, 3 = upper}
    FOracleNullMode: Byte;        {0 = natural, 1 = null}
    FBufferQueryMode: Boolean;    {false, true}
    FCompressionMode: Boolean;    {false, true}
    FMaximumBLOBSize: LongWord;   {Blob size >= 65535 and < 4294967296}
    FMultiQueryMode: Boolean;     {false, true}

    FLastErrorMessage: Ansistring;
    FLastErrorCode: LongInt;
    FLastErrorSQL: Ansistring;

    Listener: TZAbstractLoggingListener;
  public
    constructor create;

    function GetSilentMode: Byte;
    function GetCaseMode: Byte;
    function GetOracleNull: Byte;
    function GetBufferQuery: Boolean;
    function GetMaximumBLOBSize: Int64;
    function GetSQLDialect: TSQLDialect;
    function GetCompression: Boolean;
    function GetMultipleQuery: Boolean;

    procedure SetSilentMode (mode: Byte);
    procedure SetCaseMode (mode: Byte);
    procedure SetOracleNull (mode: Byte);
    procedure setBufferQuery (mode: Boolean); virtual;
    procedure setCompression (mode: Boolean); virtual;
    procedure setMaximumBLOBSize (BLOBSize: Int64); virtual;
    procedure setMultipleQuery (mode: Boolean); virtual;

    function GetProtocol: AnsiString; virtual; abstract;
    function GetDescription: AnsiString; virtual; abstract;

    function GetClientVersion: AnsiString; virtual;
    function GetClientInfo: AnsiString;  virtual;
    procedure initialize; virtual; abstract;
    procedure preconnect(Info: TStringList; listenerHandle: TZAbstractLoggingListener); virtual;
    procedure log (const Category: TZLoggingCategory; const msg: ansistring;
            const error_code: integer = 0; const error_msg: ansistring = '');


    {MOVED FROM IPDOPROTOCOL TO BASE DRIVERS}
    function GetLastErrorCode: Integer;
    function GetLastErrorMessage: AnsiString;
    function GetSQLState: AnsiString;
    function GetLastInsertID  (connection: TISC_DB_HANDLE): Int64; virtual;
    function GetServerVersion (connection: TISC_DB_HANDLE): AnsiString; virtual; abstract;
    function GetServerInfo    (connection: TISC_DB_HANDLE): AnsiString;  virtual; abstract;


    {original Zeos Functions}
    function isc_attach_database (status_vector: PISC_STATUS; db_name_length: Short; db_name: PChar; db_handle: PISC_DB_HANDLE; parm_buffer_length: Short; parm_buffer: PChar): ISC_STATUS; virtual; abstract;
    function isc_detach_database(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_drop_database(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE): ISC_STATUS;  virtual; abstract;
    function isc_database_info(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; item_list_buffer_length: Short; item_list_buffer: PChar; result_buffer_length: Short; result_buffer: PChar): ISC_STATUS;  virtual; abstract;
    function isc_array_gen_sdl(status_vector: PISC_STATUS; isc_array_desc: PISC_ARRAY_DESC; isc_arg3: PShort; isc_arg4: PChar; isc_arg5: PShort): ISC_STATUS; virtual; abstract;
    function isc_array_get_slice(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; trans_handle: PISC_TR_HANDLE; array_id: PISC_QUAD; descriptor: PISC_ARRAY_DESC; dest_array: PVoid; slice_length: ISC_LONG): ISC_STATUS;  virtual; abstract;
    function isc_array_lookup_bounds(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; trans_handle: PISC_TR_HANDLE; table_name, column_name: PChar; descriptor: PISC_ARRAY_DESC): ISC_STATUS;  virtual; abstract;
    function isc_array_lookup_desc(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; trans_handle: PISC_TR_HANDLE; table_name, column_name: PChar; descriptor: PISC_ARRAY_DESC): ISC_STATUS;  virtual; abstract;
    function isc_array_set_desc(status_vector: PISC_STATUS; table_name: PChar; column_name: PChar; sql_dtype, sql_length, sql_dimensions: PShort; descriptor: PISC_ARRAY_DESC): ISC_STATUS; virtual; abstract;
    function isc_array_put_slice(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; trans_handle: PISC_TR_HANDLE; array_id: PISC_QUAD; descriptor: PISC_ARRAY_DESC; source_array: PVoid; slice_length: PISC_LONG): ISC_STATUS; virtual; abstract;
    function isc_free(isc_arg1: PChar): ISC_LONG;  virtual; abstract;
    function isc_sqlcode(status_vector: PISC_STATUS): ISC_LONG;  virtual; abstract;
    procedure isc_sql_interprete(sqlcode: Short; buffer: PChar; buffer_length: Short);  virtual; abstract;
    function isc_interprete(buffer: PChar; status_vector: PPISC_STATUS): ISC_STATUS;  virtual; abstract;
    function isc_start_transaction(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE; db_handle_count: Short; db_handle: PISC_DB_HANDLE; tpb_length: Word; tpb_address: PChar): ISC_STATUS;  virtual; abstract;
    function isc_start_multiple(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE; db_handle_count: Short; teb_vector_address: PISC_TEB): ISC_STATUS;  virtual; abstract;
    function isc_rollback_transaction(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_rollback_retaining(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_commit_retaining(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_commit_transaction(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_dsql_allocate_statement(status_vector: PISC_STATUS;  db_handle: PISC_DB_HANDLE; stmt_handle: PISC_STMT_HANDLE): ISC_STATUS;  virtual; abstract;
    function isc_dsql_alloc_statement2(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; stmt_handle: PISC_STMT_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_dsql_describe(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS;  virtual; abstract;
    function isc_dsql_describe_bind(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS;  virtual; abstract;
    function isc_dsql_execute(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE; stmt_handle: PISC_STMT_HANDLE; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS;  virtual; abstract;
    function isc_dsql_execute2(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE; stmt_handle: PISC_STMT_HANDLE; dialect: Word; in_xsqlda, out_xsqlda: PXSQLDA): ISC_STATUS; virtual; abstract;
    function isc_dsql_execute_immediate(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; tran_handle: PISC_TR_HANDLE; length: Word; statement: PChar; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS; virtual; abstract;
    function isc_dsql_fetch(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS;  virtual; abstract;
    function isc_dsql_free_statement(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; options: Word): ISC_STATUS;  virtual; abstract;
    function isc_dsql_prepare(status_vector: PISC_STATUS; tran_handle: PISC_TR_HANDLE; stmt_handle: PISC_STMT_HANDLE; length: Word; statement: PChar; dialect: Word; xsqlda: PXSQLDA): ISC_STATUS; virtual; abstract;
    function isc_dsql_set_cursor_name(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; cursor_name: PChar; _type: Word): ISC_STATUS; virtual; abstract;
    function isc_dsql_sql_info(status_vector: PISC_STATUS; stmt_handle: PISC_STMT_HANDLE; item_length: Short; items: PChar; buffer_length: Short; buffer: PChar): ISC_STATUS;  virtual; abstract;
    function isc_open_blob2(status_vector: PISC_STATUS;  db_handle: PISC_DB_HANDLE; tran_handle: PISC_TR_HANDLE; blob_handle: PISC_BLOB_HANDLE; blob_id: PISC_QUAD; bpb_length: Short;  bpb_buffer: PChar): ISC_STATUS; virtual; abstract;
    function isc_create_blob2(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; tran_handle: PISC_TR_HANDLE; blob_handle: PISC_BLOB_HANDLE; blob_id: PISC_QUAD; bpb_length: Short; bpb_address: PChar): ISC_STATUS; virtual; abstract;
    function isc_blob_info(status_vector: PISC_STATUS; blob_handle: PISC_BLOB_HANDLE; item_list_buffer_length: Short; item_list_buffer: PChar; result_buffer_length: Short; result_buffer: PChar): PISC_STATUS; virtual; abstract;
    function isc_close_blob(status_vector: PISC_STATUS; blob_handle: PISC_BLOB_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_cancel_blob(status_vector: PISC_STATUS;  blob_handle: PISC_BLOB_HANDLE): ISC_STATUS; virtual; abstract;
    function isc_get_segment(status_vector: PISC_STATUS; blob_handle: PISC_BLOB_HANDLE; actual_seg_length: PWord; seg_buffer_length: Word; seg_buffer: PChar): ISC_STATUS; virtual; abstract;
    function isc_put_segment(status_vector: PISC_STATUS; blob_handle: PISC_BLOB_HANDLE; seg_buffer_len: Word; seg_buffer: PChar): ISC_STATUS; virtual; abstract;
    function isc_event_block(event_buffer: PPChar; result_buffer: PPChar; id_count: Word; event_list: array of PChar): ISC_LONG; virtual; abstract;
    procedure isc_event_counts(status_vector: PISC_STATUS; buffer_length: Short; event_buffer: PChar; result_buffer: PChar); virtual; abstract;
    function isc_cancel_events(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; event_id: PISC_LONG): ISC_STATUS; virtual; abstract;
    function isc_que_events(status_vector: PISC_STATUS; db_handle: PISC_DB_HANDLE; event_id: PISC_LONG; length: Short; event_buffer: PChar; event_function: TISC_CALLBACK; event_function_arg: PVoid): ISC_STATUS; virtual; abstract;
    procedure isc_decode_date(ib_date: PISC_QUAD; tm_date: PCTimeStructure);  virtual; abstract;
    procedure isc_encode_date(tm_date: PCTimeStructure; ib_date: PISC_QUAD);   virtual; abstract;
    function isc_vax_integer(buffer: PChar; length: Short): ISC_LONG; virtual; abstract;

    procedure isc_decode_sql_date(ib_date: PISC_DATE; tm_date: PCTimeStructure); virtual; abstract;
    procedure isc_decode_sql_time(ib_time: PISC_TIME; tm_date: PCTimeStructure); virtual; abstract;
    procedure isc_decode_timestamp(ib_timestamp: PISC_TIMESTAMP;  tm_date: PCTimeStructure);  virtual; abstract;
    procedure isc_encode_sql_date(tm_date: PCTimeStructure; ib_date: PISC_DATE); virtual; abstract;
    procedure isc_encode_sql_time(tm_date: PCTimeStructure; ib_time: PISC_TIME); virtual; abstract;
    procedure isc_encode_timestamp(tm_date: PCTimeStructure; ib_timestamp: PISC_TIMESTAMP);  virtual; abstract;

  
    property last_error_msg:        ansistring read FLastErrorMessage write FLastErrorMessage;
    property last_error_code:       longint    read FLastErrorCode write FLastErrorCode;
    property last_error_msg_sql:    ansistring read FLastErrorSQL write FLastErrorSQL;

  end;


implementation

{BASE Driver Object}

constructor TFirebirdBaseDriver.Create;
begin
    self.FSilentMode      := ERRORLEVEL_WARNING;
    self.FCaseMode        := CASEMODE_NATURAL;
    self.FOracleNullMode  := EMPTY_STRING_NATURAL;
    self.FBufferQueryMode := QUERY_FREFETCH_ON;
    self.FCompressionMode := OPT_COMPRESS_OFF;
    self.FMultiQueryMode  := OPT_MULTIQUERY_OFF;
    self.FMaximumBLOBSize := 255;   {all strings initially truncated to 255 characters}

    self.FLastErrorMessage := '';
    self.FLastErrorCode    := 0;
    self.FLastErrorSQL     := '';
end;

function TFirebirdBaseDriver.GetSilentMode: Byte;
begin
    Result := self.FSilentMode;
end;

function TFirebirdBaseDriver.GetCaseMode: Byte;
begin
    Result := self.FCaseMode;
end;

function TFirebirdBaseDriver.GetOracleNull: Byte;
begin
    Result := self.FOracleNullMode;
end;

function TFirebirdBaseDriver.GetBufferQuery: Boolean;
begin
    Result := self.FBufferQueryMode;
end;

procedure TFirebirdBaseDriver.SetSilentMode(mode: Byte);
begin
    self.FSilentMode := mode;
end;

procedure TFirebirdBaseDriver.SetCaseMode (mode: Byte);
begin
    self.FCaseMode := mode;
end;

procedure TFirebirdBaseDriver.SetOracleNull(mode: Byte);
begin
    self.FOracleNullMode := mode;
end;

procedure TFirebirdBaseDriver.SetBufferQuery(mode: Boolean);
begin
    self.FBufferQueryMode := mode;
end;

function TFirebirdBaseDriver.GetMaximumBLOBSize: Int64;
Begin
    Result := self.FMaximumBLOBSize;
End;

function TFirebirdBaseDriver.GetCompression: Boolean;
Begin
    Result := self.FCompressionMode;
End;

procedure TFirebirdBaseDriver.setCompression(mode: Boolean);
Begin
    // FIREBIRD DOES NOT SUPPORT TCP/IP COMPRESSION YET

    //self.FCompressionMode := mode;
End;

function TFirebirdBaseDriver.GetMultipleQuery: Boolean;
begin
    Result := self.FMultiQueryMode;
end;

procedure TFirebirdBaseDriver.setMultipleQuery(mode: Boolean);
begin
    // FIREBIRD DOES NOT SUPPORT MULTIPLE QUERIES YET

    //self.FMultiQueryMode := mode;
end;

procedure TFirebirdBaseDriver.setMaximumBLOBSize (BLOBSize: Int64);
Begin
    { FIREBIRD DOESN'T USE THIS, THERE ARE NO BLOB LIMITATIONS }
end;

function TFirebirdBaseDriver.GetSQLDialect: TSQLDialect;
begin
    Result := SQL_FIREBIRD;
end;

procedure TFirebirdBaseDriver.preconnect (Info: TStringList; listenerHandle: TZAbstractLoggingListener);
begin

    if Info.IndexOfName('datasource') < 0            then Info.Add('datasource=');
    if Info.IndexOfName('database') < 0              then Info.Add('database=');
    if Info.IndexOfName('port') < 0                  then info.Add('port=' + inttostr(FIREBIRD_DEFAULT_PORT));
    if Info.IndexOfName('dialect') < 0               then Info.Add('dialect=');
    if Info.IndexOfName('codepage') < 0              then Info.Add('codepage=');
    if Info.IndexOfName('rolename') < 0              then Info.Add('rolename=');


    //compression and multiquery can not be set yet

    self.initialize;
    self.Listener := listenerHandle;
    
end;

procedure TFirebirdBaseDriver.log (const Category: TZLoggingCategory; const msg: ansistring;
    const error_code: integer = 0; const error_msg: ansistring = '');
var
    silent_mode: byte;
    Event: TZLoggingEvent;
    protocol_description: ansistring;
begin
    if not assigned(self.Listener) then exit;

    silent_mode := self.GetSilentMode;
    protocol_description := self.GetDescription;
    
    if silent_mode = ERRORLEVEL_SILENT then exit;


    Event := TZLoggingEvent.Create(Category, protocol_description, msg, error_code, error_msg);
    try
        self.Listener.LogEvent(Event);
    finally
        Event.Destroy;
    end;

    if (silent_mode = ERRORLEVEL_RAISE_EXCEPTION) AND ((error_code <> 0) or (error_msg <> '')) then
    begin
        if error_code <> 0 then
            raise EZSQLException.CreateWithCode(error_code, error_msg)
        else
            raise EZSQLException.Create(error_msg);
    end;
end;

function TFirebirdBaseDriver.GetLastErrorCode: Integer;
begin
    Result := self.FLastErrorCode;
end;

function TFirebirdBaseDriver.GetLastErrorMessage: Ansistring;
begin
    Result := format ('%s [%s]', [self.FLastErrorMessage, self.FLastErrorSQL]);
end;

function TFirebirdBaseDriver.GetSQLState: Ansistring;
begin
    Result := '00000';
end;

function TFirebirdBaseDriver.GetClientVersion: AnsiString;
begin
    Result := format (SUnsupportedByDriver, ['CLIENT LIBRARY VERSION']);
end;

function TFirebirdBaseDriver.GetClientInfo: AnsiString;
begin
    Result := format (SUnsupportedByDriver, ['CLIENT LIBRARY INFO']);
end;

function TFirebirdBaseDriver.GetLastInsertID  (connection: TISC_DB_HANDLE): Int64;
begin
    Result := -1;
end;


end.

