{ $Id: PDOSQLDA.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOSQLDA;

{$I ../../directives.inc}

interface

uses sysUtils
    , PDOUtilsFirebird
    , PDOConstantsFirebird
    , PDOInterfaces;

type
  { Base class contain core functions to work with sqlda structure
    Can allocate memory for sqlda structure get basic information

    For the functions that accept an index, these will always be in range,
    the procedure that uses it must make sure of that.  Since range checking
    now works, this check_range procedure is getting deleted.
  }
  TSQLDA = class (TObject)
  public
    procedure InitFields(Parameters: boolean);
    procedure AllocateSQLDA; virtual;
    procedure FreeParamtersValues;

    function IsBlob(const Index: Word): boolean;
    function IsNullable(const Index: Word): boolean;

    function GetFieldCount: Integer;
    function GetFieldSqlName(const Index: Word): string;
    function GetFieldOwnerName(const Index: Word): string;
    function GetFieldRelationName(const Index: Word): string;
    function GetFieldAliasName(const Index: Word): string;
    function GetFieldIndex(const Name: String): SmallInt;
    function GetFieldScale(const Index: Word): integer;
    function GetFieldSqlType(const Index: Word): TZSQLType;
    function GetFieldLength(const Index: Word): SmallInt;
    function Matrix: PXSQLDA;

    function GetIbSqlType(const Index: Word): TFirebirdFieldTypes;
    function GetIbSqlSubType(const Index: Word): Smallint;
    function GetIbSqlLen(const Index: Word): Smallint;
  protected
    FXSQLDA: PXSQLDA;
    
    procedure SetFieldType(const Index: Word; Size: Integer; Code: Smallint; Scale: Smallint);
    function  get_structure (index: word): PXSQLVAR;
    procedure IbReAlloc(var P: pointer; const OldSize, NewSize: Integer);
  end;


implementation


{**
   Allocate memory for SQLVar in SQLDA structure for every
   fields by it length.
}
procedure TSQLDA.InitFields(Parameters: boolean);
var
    loop: smallInt;
    struct: PXSQLVAR;
    FirebirdFieldType: TFirebirdFieldTypes;
    datalen: Short;
    fields_to_initialize: smallInt;
begin
    if self.FXSQLDA.sqld > self.FXSQLDA.sqln then
        fields_to_initialize := self.FXSQLDA.sqln
    else
        fields_to_initialize := self.FXSQLDA.sqld;

    for loop := 0 to fields_to_initialize - 1 do
    begin
        struct := self.get_structure(loop);
        FirebirdFieldType := TFirebirdFieldTypes (struct.sqltype and (not 1));
        datalen := 1;
        case FirebirdFieldType of
            SQL_BOOLEAN,
            SQL_TEXT,
            SQL_TYPE_DATE,
            SQL_TYPE_TIME,
            SQL_TIMESTAMP,
            SQL_BLOB,
            SQL_ARRAY,
            SQL_QUAD,
            SQL_SHORT,
            SQL_LONG,
            SQL_INT64,
            SQL_DOUBLE,
            SQL_FLOAT,
            SQL_D_FLOAT: if struct.sqllen > 0 then datalen := struct.sqllen;
            SQL_VARYING: datalen := struct.sqllen + 2;
        end;
        self.IbReAlloc(Pointer(struct.sqldata), 0, datalen);

        if parameters = true then
            begin
                struct.sqltype := struct.sqltype or 1;
                self.IbReAlloc(Pointer(struct.sqlind), 0, sizeof(short));
            end
        else
            begin
                if (struct.sqltype and 1) = 0 then
                    struct.sqlind := nil
                else
                    ReallocMem (struct.sqlind, sizeof(short));
            end;
    end;
end;


{**
   Clear allocated data for SQLDA paramters 
}
procedure TSQLDA.FreeParamtersValues;
var
  loop: smallInt;
  struct: PXSQLVAR;
begin
    for loop := 0 to self.FXSQLDA.sqln - 1 do
    begin
        struct := self.get_structure(loop);
        FreeMem(struct.sqldata);
        FreeMem(struct.sqlind);
        struct.sqldata := nil;
        struct.sqlind  := nil;
    end;
end;

{**
   Return alias name for field
   @param Index the index fields
   @return the alias name
}
function TSQLDA.GetFieldAliasName(const Index: Word): string;
var
    struct: PXSQLVAR;
begin
    struct := self.get_structure(index);
    SetString(Result, struct.aliasname, struct.aliasname_length);
end;

{**
   Return pointer to SQLDA structure
}
function TSQLDA.Matrix: PXSQLDA;
begin
  result := FXSQLDA;
end;

{**
   Get fields count not allocated.
   @return fields count
}
function TSQLDA.GetFieldCount: Integer;
begin
  Result := FXSQLDA.sqld;
end;

{**
   Return field index by it name
   @param Index the index fields
   @return the index field
}
function TSQLDA.GetFieldIndex(const Name: String): SmallInt;
var
    loop: word;
    structure: PXSQLVAR;
begin
    Result := -1;
    for loop := 0 to GetFieldCount - 1 do
    begin
        structure := self.get_structure(loop);
        if structure.aliasname_length = Length(name) then
            if StrLIComp(@structure.aliasname, PChar(Name),
                structure.aliasname_length) = 0 then
            begin
                Result := loop;
                Exit;
            end;
    end;
end;

{**
   Return field length
   @param Index the index fields
   @return the field lenth
}
function TSQLDA.GetFieldLength(const Index: Word): SmallInt;
begin
  case GetIbSqlType(Index) of
    SQL_TEXT: Result := GetIbSqlLen(Index);
    SQL_VARYING: Result := GetIbSqlLen(Index);
    else
      Result := GetIbSqlLen(Index);
  end;
end;

{**
   Return field scale
   @param Index the index fields
   @return the field scale
}
function TSQLDA.GetFieldScale(const Index: Word): integer;
begin
    Result := Abs( self.get_structure(index).sqlscale);
end;

{**
   Convert Interbase sql type to SQLType
   @param Index the index fields
   @return the SQLType
}
function TSQLDA.GetFieldSqlType(const Index: Word): TZSQLType;
var
  SqlScale: Integer;
  SqlSubType: Integer;
begin
  SqlScale := GetFieldScale(Index);
  SqlSubType := GetIbSqlSubType(Index);

  case GetIbSqlType(Index) of
    SQL_VARYING, SQL_TEXT: Result := stString;
    SQL_LONG:
      begin
        if SqlScale = 0 then
          Result := stInteger
        else if Abs(SqlScale) <= 4 then
          Result := stFloat
        else
          Result := stDouble;
      end;
    SQL_SHORT:
      begin
        if SqlScale = 0 then
          Result := stShort
        else Result := stFloat;
      end;
    SQL_FLOAT: Result := stFloat;
    SQL_DOUBLE: Result := stDouble;
    SQL_TIMESTAMP: Result := stTimestamp;
    SQL_TYPE_TIME: Result := stTime;
    SQL_TYPE_DATE: Result := stDate;
    SQL_INT64:
      begin
        if SqlScale = 0 then
          Result := stLong
        else if Abs(SqlScale) <= 4 then
          Result := stDouble
        else
          Result := stBigDecimal;
      end;
    SQL_QUAD, SQL_ARRAY, SQL_BLOB:
      begin
        if SqlSubType = isc_blob_text then
          Result := stAsciiStream
        else Result := stBinaryStream;
      end;
    else Result := stString;
  end;
end;

{**
   Return own name for field
   @param Index the index fields
   @return the own name
}
function TSQLDA.GetFieldOwnerName(const Index: Word): string;
var
    struct: PXSQLVAR;
begin
    struct := self.get_structure(index);
    SetString(Result, struct.OwnName, struct.OwnName_length);
end;

{**
   Return real name for field
   @param Index the index fields
   @return the real name
}
function TSQLDA.GetFieldRelationName(const Index: Word): string;
var
    struct: PXSQLVAR;
begin
    struct := self.get_structure(index);
    SetString(Result, struct.RelName, struct.RelName_length);
end;

{**
   Get Interbase sql fields lenth
   @param Index the index fields
   @return Interbase sql fields lenth
}
function TSQLDA.GetIbSqlLen(const Index: Word): Smallint;
begin
  result := self.get_structure(index).sqllen;
end;

{**
   Return sql name for field
   @param Index the index fields
   @return the sql name
}
function TSQLDA.GetFieldSqlName(const Index: Word): string;
var
    struct: PXSQLVAR;
begin
    struct := self.get_structure(index);
    SetString(Result, struct.sqlname, struct.sqlname_length);
end;

{**
   Get Interbase subsql type
   @param Index the index fields
   @return the Interbase subsql
}
function TSQLDA.GetIbSqlSubType(const Index: Word): Smallint;
begin
    result := self.get_structure(index).sqlsubtype;
end;

{**
   Get Interbase sql type
   @param Index the index fields
   @return the interbase sql type
}
function TSQLDA.GetIbSqlType(const Index: Word): TFirebirdFieldTypes;
begin
    result := TFirebirdFieldTypes (self.get_structure(index).sqltype and not (1));
end;

{**
   Reallocate memory and fill memory by #0
   @param pointer to memory block
   @param old size of memory block
   @param new size of memory block
}
procedure TSQLDA.IbReAlloc(var P: Pointer; const OldSize, NewSize: Integer);
begin
    ReallocMem(P, NewSize);
    if NewSize > OldSize then
        Fillchar((Pchar(P) + OldSize)^, NewSize - OldSize, #0);
end;

procedure TSQLDA.SetFieldType(const Index: Word; Size: Integer; Code: Smallint; Scale: Smallint);
begin

  with self.get_structure(index)^ do
  begin
    sqltype := Code;
    if Scale <= 0 then
      sqlscale := Scale;
    sqllen := Size;
    if (Size > 0) then
      IbReAlloc(pointer(sqldata), 0, Size)
    else
    begin
      FreeMem(sqldata);
      sqldata := nil;
    end;
  end;
end;

{**
   Indicate blob field
   @param Index the index fields
   @return true if blob field overwise false
}
function TSQLDA.IsBlob(const Index: Word): boolean;
var
    FirebirdFieldType: TFirebirdFieldTypes;
begin
    FirebirdFieldType := TFirebirdFieldTypes (self.get_structure(index).sqltype and (not 1));
    result := (FirebirdFieldType = SQL_BLOB);
end;

{**
   Indicate blob field
   @param Index the index fields
   @return true if field nullable overwise false
}
function TSQLDA.IsNullable(const Index: Word): boolean;
begin
  Result := self.get_structure(index).sqltype and 1 = 1
end;

{**
   Reallocate SQLDA to fields count length
   @param Value the count fields
}
procedure TSQLDA.AllocateSQLDA;
var
    sqln_length: LongInt;
    sqld_length: LongInt;
begin
    sqln_length := PDOUtilsFirebird.XSQLDA_LENGTH(FXSQLDA.sqln);
    sqld_length := PDOUtilsFirebird.XSQLDA_LENGTH(FXSQLDA.sqld);

    self.IbReAlloc(pointer(FXSQLDA), sqln_length, sqld_length);
    FXSQLDA.sqln := FXSQLDA.sqld;
end;




function TSQLDA.get_structure (index: word): PXSQLVAR;
var
    loop: smallInt;
begin
    Result := @self.FXSQLDA.sqlvar[0];
    for loop := 1 to index do
        inc (Result);
end;

end.
