{ $Id: PDOSQLDAParams.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOSQLDAParams;

{$I ../../directives.inc}

interface

uses sysUtils
    , Classes
    , PDOSQLDA
    , PDOMessages
    , PDOCompatibility
    , PDOUtilsFirebird
    , PDOConstantsFirebird
    , PDOInterfaces
    , PDOFirebirdDriver;

type

{ Parameters class for sqlda structure.
    It clas can only write data to parameters/fields }
  TSQLDAParams = class (TSQLDA)
  protected
    FProtocol: TFirebirdBaseDriver;
    FHandle: PISC_DB_HANDLE;
    FTransactionHandle: PISC_TR_HANDLE;
  private
    procedure EncodeString(Code: TFirebirdFieldTypes; const Index: Word; const Str: String);
    procedure UpdateDateTime(const Index: Integer; Value: TDateTime);
  public
    constructor Create(
        Protocol: TFirebirdBaseDriver;
        Handle: PISC_DB_HANDLE;
        TransactionHandle: PISC_TR_HANDLE;
        default_num_params: word = 1);
    destructor Destroy; override;

    procedure WriteBlob(const Index: Integer; Stream: TStream);

    procedure UpdateNull(const Index: Integer; Value: boolean);
    procedure UpdateBoolean(const Index: Integer; Value: boolean);
    procedure UpdateByte(const Index: Integer; Value: ShortInt);
    procedure UpdateShort(const Index: Integer; Value: SmallInt);
    procedure UpdateInt(const Index: Integer; Value: Integer);
    procedure UpdateLong(const Index: Integer; Value: Int64);
    procedure UpdateFloat(const Index: Integer; Value: Single);
    procedure UpdateDouble(const Index: Integer; Value: Double);
    procedure UpdateBigDecimal(const Index: Integer; Value: Extended);
    procedure UpdatePChar(const Index: Integer; Value: PChar);
    procedure UpdateString(const Index: Integer; Value: string);
    procedure UpdateBytes(const Index: Integer; Value: TByteDynArray);
    procedure UpdateDate(const Index: Integer; Value: TDateTime);
    procedure UpdateTime(const Index: Integer; Value: TDateTime);
    procedure UpdateTimestamp(const Index: Integer; Value: TDateTime);
    procedure UpdateQuad(const Index: Word; const Value: TISC_QUAD);
  end;

implementation


{ TParamsSQLDA }

{**
   Constructs this object and assignes the main properties.
   param PlainDriver the interbase plain driver
}
constructor TSQLDAParams.Create(
        Protocol: TFirebirdBaseDriver;
        Handle: PISC_DB_HANDLE;
        TransactionHandle: PISC_TR_HANDLE;
        default_num_params: word = 1);
begin
  self.FProtocol          := Protocol;
  self.FHandle            := Handle;
  self.FTransactionHandle := TransactionHandle;

  GetMem(self.FXSQLDA, PDOUtilsFirebird.XSQLDA_LENGTH(default_num_params));
  FillChar(self.FXSQLDA^, PDOUtilsFirebird.XSQLDA_LENGTH(default_num_params), 0);

  self.FXSQLDA.version := SQLDA_VERSION1;
  self.FXSQLDA.sqln := default_num_params;
  self.FXSQLDA.sqld := default_num_params;

end;

{**
   Free allocated memory and free object
}
destructor TSQLDAParams.Destroy;
begin
  FreeParamtersValues;
  FreeMem(FXSQLDA);
  inherited Destroy;
end;

{**
   Encode pascal string to Interbase paramter buffer
   @param Code the Interbase data type
   @param Index the index target filed
   @param Str the source string
}
procedure TSQLDAParams.EncodeString(Code: TFirebirdFieldTypes; const Index: Word; const Str: String);
var
  Len: Cardinal;
begin
  Len := Length(Str);
  
   with self.get_structure(index)^ do
    case Code of
      SQL_TEXT :
        begin
          if sqllen = 0 then
            GetMem(sqldata, Len)
          else
            IbReAlloc(Pointer(sqldata), 0, Len + 1);
          sqllen := Len;
          Move(PChar(str)^, sqldata^, sqllen);
        end;
      SQL_VARYING :
        begin
          if sqllen = 0 then
            GetMem(sqldata, Len + 2)
          else
            IbReAlloc(Pointer(sqldata), 0, Len + 2);
          sqllen := Len + 2;
          PISC_VARYING(sqldata).strlen := Len;
          Move(PChar(str)^, PISC_VARYING(sqldata).str,
              PISC_VARYING(sqldata).strlen);
        end;
    end;
end;

{**
   Set up parameter BigDecimal value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateBigDecimal(const Index: Integer; Value: Extended);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Int64), ord(SQL_INT64) + 1, -4);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes (sqltype and not(1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_LONG   : PInteger(sqldata)^  := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Trunc(Value);
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Trunc(Value);
        SQL_SHORT     : PSmallint(sqldata)^ := Trunc(Value);
        SQL_INT64     : PInt64(sqldata)^ := Trunc(Value);
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, FloatToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, FloatToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Boolean value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateBoolean(const Index: Integer; Value: boolean);
var
  SQLCode: TFirebirdFieldTypes;
begin
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and not(1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := ord(Value) * IBScaleDivisor[sqlscale];
        SQL_LONG   : PInteger(sqldata)^  := ord(Value) * IBScaleDivisor[sqlscale];
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := ord(Value) * IBScaleDivisor[sqlscale];
        SQL_DOUBLE : PDouble(sqldata)^   := ord(Value);
      else
        raise EZIBConvertError.Create(SUnsupportedParameterType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := ord(Value);
        SQL_LONG      : PInteger(sqldata)^ := ord(Value);
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := ord(Value);
        SQL_BOOLEAN   : PSmallint(sqldata)^ := ord(Value);
        SQL_SHORT     : PSmallint(sqldata)^ := ord(Value);
        SQL_INT64     : PInt64(sqldata)^ := ord(Value);
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, IntToStr(ord(Value)));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, IntToStr(ord(Value)));
      else
        raise EZIBConvertError.Create(SUnsupportedParameterType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Byte value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateByte(const Index: Integer; Value: ShortInt);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Smallint), ord(SQL_SHORT) + 1, 0);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Value * IBScaleDivisor[sqlscale];
        SQL_LONG   : PInteger(sqldata)^  := Value * IBScaleDivisor[sqlscale];
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Value * IBScaleDivisor[sqlscale];
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedParameterType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Value;
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN: begin
                       if self.FProtocol.GetProtocol <> 'interbase-7' then
                         raise EZIBConvertError.Create(SUnsupportedDataType);
                       PSmallint(sqldata)^ := Value;
                     end;
        SQL_SHORT     : PSmallint(sqldata)^ := Value;
        SQL_INT64     : PInt64(sqldata)^ := Value;
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, IntToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, IntToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedParameterType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter byte value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateBytes(const Index: Integer; Value: TByteDynArray);
begin

end;

{**
   Set up parameter Date value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateDate(const Index: Integer; Value: TDateTime);
begin
  SetFieldType(Index, sizeof(Integer), ord(SQL_TYPE_DATE) + 1, 0);
  UpdateDateTime(Index, Value);
end;

{**
   Set up parameter DateTime value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateDateTime(const Index: Integer;
  Value: TDateTime);
var
  y, m, d: word;
  hr, min, sec, msec: word;
  SQLCode: TFirebirdFieldTypes;
  TmpDate: TCTimeStructure;
begin
  with self.get_structure(index)^ do
  begin
    DecodeDate(Value, y, m, d);
    DecodeTime(Value, hr, min, sec, msec);
    TmpDate.tm_year := y - 1900;
    TmpDate.tm_mon := m - 1;
    TmpDate.tm_mday := d;
    TmpDate.tm_hour := hr;
    TmpDate.tm_min := min;
    TmpDate.tm_sec := sec;
    TmpDate.tm_wday := 0;
    TmpDate.tm_yday := 0;
    TmpDate.tm_isdst := 0;

    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes (sqltype and (not 1));

    case SQLCode of
      SQL_TYPE_DATE : self.FProtocol.isc_encode_sql_date(@TmpDate, PISC_DATE(sqldata));
      SQL_TYPE_TIME : self.FProtocol.isc_encode_sql_time(@TmpDate, PISC_TIME(sqldata));
      SQL_TIMESTAMP :
        self.FProtocol.isc_encode_date(@TmpDate, PISC_QUAD(sqldata));
      else
        raise EZIBConvertError.Create(SInvalidState);
    end;
    if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Double value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateDouble(const Index: Integer; Value: Double);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(double), ord(SQL_DOUBLE) + 1, 0);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_LONG   : PInteger(sqldata)^  := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Trunc(Value);
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Trunc(Value);
        SQL_SHORT     : PSmallint(sqldata)^ := Trunc(Value);
        SQL_INT64     : PInt64(sqldata)^ := Trunc(Value);
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, FloatToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, FloatToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Float value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateFloat(const Index: Integer; Value: Single);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Single), ord(SQL_FLOAT) + 1, 1);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_LONG   : PInteger(sqldata)^  := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Trunc(Value * IBScaleDivisor[sqlscale]);
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
        SQL_D_FLOAT,
        SQL_FLOAT  : PSingle(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Trunc(Value);
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Trunc(Value);
        SQL_SHORT     : PSmallint(sqldata)^ := Trunc(Value);
        SQL_INT64     : PInt64(sqldata)^ := Trunc(Value);
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, FloatToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, FloatToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter integer value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateInt(const Index: Integer; Value: Integer);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Integer), ord(SQL_LONG) + 1, 0);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Value * IBScaleDivisor[sqlscale];
        SQL_LONG   : PInteger(sqldata)^  := Value * IBScaleDivisor[sqlscale];
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Value * IBScaleDivisor[sqlscale];
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Value;
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Value;
        SQL_SHORT     : PSmallint(sqldata)^ := Value;
        SQL_INT64     : PInt64(sqldata)^ := Value;
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, IntToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, IntToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Long value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateLong(const Index: integer; Value: Int64);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Int64), ord(SQL_INT64) + 1, 0);
  
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Value * IBScaleDivisor[sqlscale];
        SQL_LONG   : PInteger(sqldata)^  := Value * IBScaleDivisor[sqlscale];
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Value * IBScaleDivisor[sqlscale];
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Value;
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Value;
        SQL_SHORT     : PSmallint(sqldata)^ := Value;
        SQL_INT64     : PInt64(sqldata)^ := Value;
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, IntToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, IntToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter null value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateNull(const Index: Integer; Value: boolean);
begin
  with self.get_structure(index)^ do
    if (sqlind <> nil) then
      case Value of
        True  : sqlind^ := -1; //NULL
        False : sqlind^ :=  0; //NOT NULL
      end;
end;

{**
   Set up parameter PChar value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdatePChar(const Index: Integer; Value: PChar);
var
  TempString: string;
begin
  TempString := Value;
  UpdateString(Index, TempString);
end;

{**
   Set up parameter Interbase QUAD value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateQuad(const Index: Word; const Value: TISC_QUAD);
var
    code: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(TISC_QUAD), ord(SQL_QUAD) + 1, 0);

  with self.get_structure(index)^ do
    if not ((sqlind <> nil) and (sqlind^ = -1)) then
    begin
        code := TFirebirdFieldTypes (sqltype and not(1)) ;
      case code of
        SQL_QUAD, SQL_DOUBLE, SQL_INT64, SQL_BLOB, SQL_ARRAY: PISC_QUAD(sqldata)^ := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
    end else
      raise EZIBConvertError.Create(SUnsupportedDataType);
end;

{**
   Set up parameter short value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateShort(const Index: Integer; Value: SmallInt);
var
  SQLCode: TFirebirdFieldTypes;
begin
  SetFieldType(Index, sizeof(Smallint), ord(SQL_SHORT) + 1, 0);
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and not(1));

    if (sqlscale < 0)  then
    begin
      case SQLCode of
        SQL_SHORT  : PSmallInt(sqldata)^ := Value * IBScaleDivisor[sqlscale];
        SQL_LONG   : PInteger(sqldata)^  := Value * IBScaleDivisor[sqlscale];
        SQL_INT64,
        SQL_QUAD   : PInt64(sqldata)^    := Value * IBScaleDivisor[sqlscale];
        SQL_DOUBLE : PDouble(sqldata)^   := Value;
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
    end else
      case SQLCode of
        SQL_DOUBLE    : PDouble(sqldata)^   := Value;
        SQL_LONG      : PInteger(sqldata)^ := Value;
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(sqldata)^ := Value;
        SQL_BOOLEAN   : PSmallint(sqldata)^ := Value;
        SQL_SHORT     : PSmallint(sqldata)^ := Value;
        SQL_INT64     : PInt64(sqldata)^ := Value;
        SQL_TEXT      : EncodeString(SQL_TEXT, Index, IntToStr(Value));
        SQL_VARYING   : EncodeString(SQL_VARYING, Index, IntToStr(Value));
      else
        raise EZIBConvertError.Create(SUnsupportedDataType);
      end;
      if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter String value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateString(const Index: Integer; Value: string);
var
 SQLCode: TFirebirdFieldTypes;
begin
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and (not 1));
    case SQLCode of
      SQL_TEXT      : EncodeString(SQL_TEXT, Index, Value);
      SQL_VARYING   : EncodeString(SQL_VARYING, Index, Value);
    else
      raise EZIBConvertError.Create(SErrorConvertion);
    end;
    if (sqlind <> nil) then sqlind^ := 0; // not null
  end;
end;

{**
   Set up parameter Time value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateTime(const Index: Integer; Value: TDateTime);
begin
  SetFieldType(Index, sizeof(Cardinal), ord(SQL_TYPE_TIME) + 1, 0);
  UpdateDateTime(Index, Value);
end;

{**
   Set up parameter Timestamp value
   @param Index the target parameter index
   @param Value the source value
}
procedure TSQLDAParams.UpdateTimestamp(const Index: Integer; Value: TDateTime);
begin
  SetFieldType(Index, sizeof(TISC_QUAD), ord(SQL_TIMESTAMP) + 1, 0);
  UpdateDateTime(Index, Value);
end;

{**
   Write stream to blob field
   @param Index an index field number
   @param Stream the souse data stream
}
procedure TSQLDAParams.WriteBlob(const Index: Integer; Stream: TStream);
var
  Buffer: PChar;
  BlobId: TISC_QUAD;
  BlobHandle: TISC_BLOB_HANDLE;
  StatusVector: TARRAY_ISC_STATUS;
  BlobSize, CurPos, SegLen: Integer;
begin
  BlobHandle := nil;
  Stream.Seek(0, 0);

  { create blob handle }
  self.FProtocol.isc_create_blob2(@StatusVector, FHandle, FTransactionHandle,
    @BlobHandle, @BlobId, 0, nil);
  PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, StatusVector);

  Stream.Position := 0;
  BlobSize := Stream.Size;
  Buffer := AllocMem(BlobSize);
  Try
    Stream.ReadBuffer(Buffer^, BlobSize);

    { put data to blob }
    CurPos := 0;
    SegLen := DefaultBlobSegmentSize;
    while (CurPos < BlobSize) do
    begin
      if (CurPos + SegLen > BlobSize) then
        SegLen := BlobSize - CurPos;
      if self.FProtocol.isc_put_segment(@StatusVector, @BlobHandle, SegLen,
           PChar(@Buffer[CurPos])) > 0 then
        PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, StatusVector);
      Inc(CurPos, SegLen);
    end;

    { close blob handle }
    self.FProtocol.isc_close_blob(@StatusVector, @BlobHandle);
    PDOUtilsFirebird.CheckFirebirdError(self.FProtocol, StatusVector);

    Stream.Seek(0, 0);
    UpdateQuad(Index, BlobId);
  Finally
    Freemem(Buffer);
  End;
end;


end.
 