{ $Id: PDOSQLDAResult.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOSQLDAResult;

{$I ../../directives.inc}

interface

uses sysUtils
    , Classes
    , PDOSQLDA
    , Variants
    , PDOMessages
    , PDOUtilsFirebird
    , PDOConstantsFirebird
    , PDOFirebirdDriver
    , PDOInterfaces;

type
    PTimestamp = ^TDateTime;

  TSQLDAResult = class (TSQLDA)
  protected
    FHandle: PISC_DB_HANDLE;
    FTransactionHandle: PISC_TR_HANDLE;
    FProtocol: TFirebirdBaseDriver;
  private
    procedure DecodeString2(const Code: TFirebirdFieldTypes; const Index: Word; out Str: ansistring);
    procedure ReadBlobFromString(const Index: Word; var str: string);
    function GetQuad(const Index: Integer): TISC_QUAD;
    function GetDT_timestamp (data: PAnsichar): ansistring;
    function GetDT_time      (data: PAnsichar): ansistring;
    function GetDT_date      (data: PAnsichar): ansistring;
    procedure bind_explicit (const bind_address: pointer; const explicit_type: TFirebirdFieldTypes;
            const value: variant);

  public
    constructor Create(
        Protocol: TFirebirdBaseDriver;
        Handle: PISC_DB_HANDLE;
        TransactionHandle: PISC_TR_HANDLE;
        default_num_columns: word = 1);
    destructor Destroy; override;

    function IsNull(const Index: Integer): Boolean;
    function GetString(const Index: Integer): ansistring;
    function GetTimestamp(const Index: Integer): TDateTime;
    procedure bind_native_format (const Index: Integer; const bind_address: pointer);
    procedure bind_explicit_format (const Index: Integer; const bind_address: pointer;
        const explicit_type: TFirebirdFieldTypes);
    procedure copy_from (const result_structure: TZeroTXSQLVAR);
  end;

implementation


{**
   Constructs this object and assignes the main properties.
   param PlainDriver the interbase plain driver
}
constructor TSQLDAResult.Create(
        Protocol: TFirebirdBaseDriver;
        Handle: PISC_DB_HANDLE;
        TransactionHandle: PISC_TR_HANDLE;
        default_num_columns: word = 1);
begin
  self.FProtocol := Protocol;
  FHandle := Handle;
  FTransactionHandle := TransactionHandle;

  GetMem(FXSQLDA, XSQLDA_LENGTH(default_num_columns));
  FillChar(FXSQLDA^, XSQLDA_LENGTH(default_num_columns), 0);

  FXSQLDA.version := SQLDA_VERSION1;
  FXSQLDA.sqln := default_num_columns;
  FXSQLDA.sqld := default_num_columns;
end;

{**
   Decode Interbase field value to pascal string
   @param Code the Interbase data type
   @param Index field index
   @param the field string
}
procedure TSQLDAResult.DecodeString2(const Code: TFirebirdFieldTypes; const Index: Word; out Str: ansistring);
begin
  
  with self.get_structure(index)^ do
  case Code of
    SQL_TEXT    : Str := TrimRight(BufferToStr(sqldata, sqllen));
    SQL_VARYING : SetString(Str, PISC_VARYING(sqldata).str,
      PISC_VARYING(sqldata).strlen);
  end;
end;


{**
   Return String field value
   @param Index the field index
   @return the field String value
}
function TSQLDAResult.GetString(const Index: Integer): ansistring;
var
  SQLCode: TFirebirdFieldTypes;
begin
  Result := '';

  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and not(1));

    if (sqlscale < 0)  then
      begin
        case SQLCode of
          SQL_SHORT  : Result := FloatToStr(PSmallInt(sqldata)^ / IBScaleDivisor[sqlscale]);
          SQL_LONG   : Result := FloatToStr(PInteger(sqldata)^  / IBScaleDivisor[sqlscale]);
          SQL_INT64,
          SQL_QUAD   : Result := FloatToStr(PInt64(sqldata)^    / IBScaleDivisor[sqlscale]);
          SQL_DOUBLE : Result := FloatToStr(PDouble(sqldata)^);
        else
          raise EZIBConvertError.Create(Format(SErrorConvertionField,
            [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
        end;
      end
    else
      case SQLCode of
        SQL_DOUBLE    : Result := FloatToStr(PDouble(sqldata)^);
        SQL_LONG      : Result := IntToStr(PInteger(sqldata)^);
        SQL_D_FLOAT,
        SQL_FLOAT     : Result := FloatToStr(PSingle(sqldata)^);
        SQL_SHORT     : Result := IntToStr(PSmallint(sqldata)^);
        SQL_INT64     : Result := IntToStr(PInt64(sqldata)^);
        SQL_TEXT      : DecodeString2(SQL_TEXT, Index, Result);
        SQL_VARYING   : DecodeString2(SQL_VARYING, Index, Result);
        SQL_BLOB      : ReadBlobFromString(Index, Result);
        SQL_TIMESTAMP : Result := self.GetDT_timestamp(sqldata);
        SQL_TYPE_DATE : Result := self.GetDT_date(sqldata);
        SQL_TYPE_TIME : Result := self.GetDT_time(sqldata);
        SQL_BOOLEAN   :
                        if Boolean(PSmallint(sqldata)^) = True then
                                Result := 'TRUE'
                        else
                                Result := 'FALSE';
      else
        raise EZIBConvertError.Create(Format(SErrorConvertionField,
          [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
      end;
  end;
end;


{**
   Return Timestamp field value
   @param Index the field index
   @return the field Timestamp value
}
function TSQLDAResult.GetTimestamp(const Index: Integer): TDateTime;
var
  TempDate: TCTimeStructure;
  code: TFirebirdFieldTypes;
begin

  with self.get_structure(index)^ do
  begin
    Result := 0;
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;

    self.FProtocol.isc_decode_date(PISC_QUAD(sqldata), @TempDate);
    code := TFirebirdFieldTypes(sqltype and (not 1)) ;
    case code of
        SQL_TIMESTAMP : Result := SysUtils.EncodeDate(TempDate.tm_year + 1900,
                            TempDate.tm_mon + 1, TempDate.tm_mday) + EncodeTime(TempDate.tm_hour,
                            TempDate.tm_min, TempDate.tm_sec, 0);

        SQL_TYPE_DATE : Result := SysUtils.EncodeDate(Word(TempDate.tm_year + 1900),
                            Word(TempDate.tm_mon + 1), Word(TempDate.tm_mday));

        SQL_TYPE_TIME : Result := SysUtils.EncodeTime(Word(TempDate.tm_hour), Word(TempDate.tm_min),
                            Word(TempDate.tm_sec), 0);
    end;
  end;

end;


function TSQLDAResult.GetDT_timestamp (data: PAnsichar): ansistring;
var
    TempDate: TCTimeStructure;
begin
    { timestamp format: YYYY-MM-DD HH:MM:SS }

    self.FProtocol.isc_decode_date(PISC_QUAD(data), @TempDate);
    Result := format ('%.4d-%.2d-%.2d %.2d:%.2d:%.2d', [
        TempDate.tm_year + 1900,
        TempDate.tm_mon + 1,
        TempDate.tm_mday,
        TempDate.tm_hour,
        TempDate.tm_min,
        TempDate.tm_sec
    ]);
end;

function TSQLDAResult.GetDT_time      (data: PAnsichar): ansistring;
var
    TempDate: TCTimeStructure;
begin
    { time format: HH:MM:SS }

    self.FProtocol.isc_decode_date(PISC_QUAD(data), @TempDate);
    Result := format ('%.2d:%.2d:%.2d', [
        TempDate.tm_hour,
        TempDate.tm_min,
        TempDate.tm_sec
    ]);
end;


function TSQLDAResult.GetDT_date (data: PAnsichar): ansistring;
var
    TempDate: TCTimeStructure;
begin
    { date format: YYYY-MM-DD }

    self.FProtocol.isc_decode_date(PISC_QUAD(data), @TempDate);
    Result := format ('%.4d-%.2d-%.2d %.2d:%.2d:%.2d', [
        TempDate.tm_year + 1900,
        TempDate.tm_mon + 1,
        TempDate.tm_mday
    ]);
end;



{**
   Indicate field null
   @param Index the field index
   @return true if fied value NULL overwise false
}
function TSQLDAResult.IsNull(const Index: Integer): Boolean;
begin

  with self.get_structure(index)^ do
    Result := (sqlind <> nil) and (sqlind^ = ISC_NULL);
end;

{**
   Return Interbase QUAD field value
   @param Index the field index
   @return the field Interbase QUAD value
}
function TSQLDAResult.GetQuad(const Index: Integer): TISC_QUAD;
var
    sqlcode: TFirebirdFieldTypes;
begin

  with self.get_structure(index)^ do
  if not ((sqlind <> nil) and (sqlind^ = -1)) then
    begin
        sqlcode := TFirebirdFieldTypes (sqltype and not(1));
        case sqlcode of
            SQL_QUAD,
            SQL_DOUBLE,
            SQL_INT64,
            SQL_BLOB,
            SQL_ARRAY: result := PISC_QUAD(sqldata)^;
        else
            raise EZIBConvertError.Create(SUnsupportedDataType + ' ' + inttostr((sqltype and not(1))));
        end;
    end
  else
    raise EZIBConvertError.Create('Invalid State.');
end;


destructor TSQLDAResult.Destroy;
begin
  FreeParamtersValues;
  FreeMem(FXSQLDA);
  inherited Destroy;
end;

{**
   Read blob data to string
   @param Index an filed index
   @param Str destination string
}
procedure TSQLDAResult.ReadBlobFromString(const Index: Word; var Str: string);
var
  Size: LongInt;
  Buffer: Pointer;
begin
  PDOUtilsFirebird.ReadBlobBuffer(self.FProtocol, FHandle, FTransactionHandle, GetQuad(Index), Size, Buffer);
  try
    SetLength(Str, Size);
    SetString(Str, PChar(Buffer), Size);
  finally
    FreeMem(Buffer, Size);
  end;
end;

procedure TSQLDAResult.bind_native_format (const Index: Integer; const bind_address: pointer);
var
  SQLCode: TFirebirdFieldTypes;

begin
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and not(1));

    if (sqlscale < 0)  then
      begin
        case SQLCode of
          SQL_SHORT  : PSingle(bind_address)^    := PSmallInt(sqldata)^ / IBScaleDivisor[sqlscale];
          SQL_LONG   : PDouble(bind_address)^    := PInteger(sqldata)^ / IBScaleDivisor[sqlscale];
          SQL_INT64,
          SQL_QUAD   : PExtended(bind_address)^  := PInt64(sqldata)^ / IBScaleDivisor[sqlscale];
          SQL_DOUBLE : PDouble(bind_address)^    := PDouble(sqldata)^;
        else
          raise EZIBConvertError.Create(Format(SErrorConvertionField,
            [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
        end;
      end
    else
      case SQLCode of
        SQL_DOUBLE    : PDouble(bind_address)^   := PDouble(sqldata)^;
        SQL_LONG      : PInteger(bind_address)^  := PInteger(sqldata)^;
        SQL_D_FLOAT,
        SQL_FLOAT     : PSingle(bind_address)^   := PSingle(sqldata)^;
        SQL_SHORT     : PSmallint(bind_address)^ := PSmallint(sqldata)^;
        SQL_INT64     : PInt64(bind_address)^    := PInt64(sqldata)^;
        SQL_BOOLEAN   : Pboolean(bind_address)^  := Boolean(PSmallint(sqldata)^);
        SQL_TIMESTAMP,
        SQL_TYPE_DATE,
        SQL_TYPE_TIME : PTimestamp(bind_address)^ := self.GetTimestamp(Index);
        SQL_TEXT      : DecodeString2(SQL_TEXT, Index, Pstring(bind_address)^);
        SQL_VARYING   : DecodeString2(SQL_VARYING, Index, PString(bind_address)^);
        SQL_BLOB      : ReadBlobFromString(Index, PString(bind_address)^);
        

      else
        raise EZIBConvertError.Create(Format(SErrorConvertionField,
          [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
      end;
  end;
end;


procedure TSQLDAResult.bind_explicit (const bind_address: pointer;
    const explicit_type: TFirebirdFieldTypes; const value: variant);
begin
    case explicit_type of
        SQL_SHORT       : PSmallInt(bind_address)^   := value;
        SQL_LONG        : PInteger(bind_address)^    := value;
        SQL_INT64       : PInt64(bind_address)^      := value;
        SQL_FLOAT       : PSingle(bind_address)^     := value;
        SQL_DOUBLE      : PDouble(bind_address)^     := value;
        SQL_TEXT        : Pstring(bind_address)^     := value;
        SQL_VARYING     : PAnsistring(bind_address)^ := value;
        SQL_TIMESTAMP   : PTimestamp(bind_address)^  := value;
        SQL_BLOB        : PAnsistring(bind_address)^ := value;
    end;
end;

procedure TSQLDAResult.bind_explicit_format (const Index: Integer; const bind_address: pointer;
    const explicit_type: TFirebirdFieldTypes);
var
  SQLCode: TFirebirdFieldTypes;
  tmp_ansi: ansistring;
  tmp_reg:  string;
begin
  with self.get_structure(index)^ do
  begin
    if (sqlind <> nil) and (sqlind^ = -1) then Exit;
    SQLCode := TFirebirdFieldTypes(sqltype and not(1));

    if (sqlscale < 0)  then
      begin
        case SQLCode of
          SQL_SHORT  : self.bind_explicit(bind_address, explicit_type, PSmallInt(sqldata)^ / IBScaleDivisor[sqlscale]);
          SQL_LONG   : self.bind_explicit(bind_address, explicit_type, PInteger(sqldata)^ / IBScaleDivisor[sqlscale]);
          SQL_INT64,
          SQL_QUAD   : self.bind_explicit(bind_address, explicit_type, PInt64(sqldata)^ / IBScaleDivisor[sqlscale]);
          SQL_DOUBLE : self.bind_explicit(bind_address, explicit_type, PDouble(sqldata)^);
        else
          raise EZIBConvertError.Create(Format(SErrorConvertionField,
            [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
        end;
      end
    else
      case SQLCode of
        SQL_DOUBLE    : self.bind_explicit(bind_address, explicit_type, PDouble(sqldata)^);
        SQL_LONG      : self.bind_explicit(bind_address, explicit_type, PInteger(sqldata)^);
        SQL_D_FLOAT,
        SQL_FLOAT     : self.bind_explicit(bind_address, explicit_type, PSingle(sqldata)^);
        SQL_SHORT     : self.bind_explicit(bind_address, explicit_type, PSmallint(sqldata)^);
        SQL_INT64     : self.bind_explicit(bind_address, explicit_type, PInt64(bind_address)^);
        SQL_BOOLEAN   : self.bind_explicit(bind_address, explicit_type, Boolean(PSmallint(sqldata)^));
        SQL_TIMESTAMP,
        SQL_TYPE_DATE,
        SQL_TYPE_TIME : self.bind_explicit(bind_address, explicit_type, self.GetTimestamp(Index));
        SQL_TEXT      : begin
                            DecodeString2(SQL_TEXT, Index, tmp_reg);
                            self.bind_explicit(bind_address, explicit_type, tmp_reg);
                        end;
        SQL_VARYING   : begin
                            DecodeString2(SQL_VARYING, Index, tmp_ansi);
                            self.bind_explicit(bind_address, explicit_type, tmp_ansi);
                        end;
        SQL_BLOB      : begin
                            ReadBlobFromString(Index, tmp_ansi);
                            self.bind_explicit(bind_address, explicit_type, tmp_ansi);
                        end;

      else
        raise EZIBConvertError.Create(Format(SErrorConvertionField,
          [GetFieldAliasName(Index), GetNameSqlType(SQLCode)]));
      end;
  end;
end;

procedure TSQLDAResult.copy_from (const result_structure: TZeroTXSQLVAR);
var
    own_struct: PXSQLVAR;
    master_struct: PXSQLVAR;
    index: integer;
    x: byte;
begin
    master_struct := @result_structure[0];
    own_struct := self.get_structure(0);
    for index := 0 to self.GetFieldCount - 1 do
    begin
        own_struct^.sqltype          := master_struct^.sqltype;
        own_struct^.sqlscale         := master_struct^.sqlscale;
        own_struct^.sqlsubtype       := master_struct^.sqlsubtype;
        own_struct^.sqllen           := master_struct^.sqllen;
        own_struct^.sqlname_length   := master_struct^.sqlname_length;
        own_struct^.relname_length   := master_struct^.relname_length;
        own_struct^.ownname_length   := master_struct^.ownname_length;
        own_struct^.aliasname_length := master_struct^.aliasname_length;
        for x := 0 to 31 do
        begin
            own_struct^.sqlname[x]   := master_struct^.sqlname[x];
            own_struct^.relname[x]   := master_struct^.relname[x];
            own_struct^.ownname[x]   := master_struct^.ownname[x];
            own_struct^.aliasname[x] := master_struct^.aliasname[x];
        end;

        inc (master_struct);
        inc (own_struct);
    end;
end;

end.
