{ $Id: PDOStatementFirebird.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOStatementFirebird;

interface

{$I ../../directives.inc}

uses  Classes
    , sysUtils
    , PDOStatement
    , PDOClasses
    , PDOSysUtils
    , PDOLogging
    , PDOMessages
    , PDOSQLDAParams
    , PDOSQLDAResult
    , PDOResultFirebird
    , PDOInterfaces
    , PDOUtilsFirebird
    , PDOFirebirdDriver
    , PDOConstantsFirebird;

const
    RS_CAPACITY_INCREMENT = 500;

type
    ColumnRecord = record
        Name:       AnsiString;
        Datatype:   TZSQLType;
        Datatype2:  TFirebirdFieldTypes;
        DataLength: LongWord;
        Scale:      Integer;
    end;
    BindTargetRec = record
        bind_target     : pointer;
        override_type   : TPDODataTypes;
        override_set    : boolean;
    end;
    TBindTargets = array of BindTargetRec;

    TDateForm = (DF_TIME, DF_DATE, DF_TIMESTAMP);
    TResultSet = array of TSQLDAResult;

    TPDOFirebirdStatement = class(TInterfacedObject, IPDOStatement)
    private
        TypeOfStatement:       TPDOStatementType;
        DirectSQL:             AnsiString;
        PreparedSQL:           AnsiString;
        alpha_bind_values:     TStringList;
        alpha_bind_columns:    TStringList;
	    Successful_query:      Boolean;
        rebind_required:       Boolean;
        Successful_stmt_prep:  Boolean;
        exec_with_prefetch:    Boolean;
        cursor_parked:         Boolean;
        numberColumns:         Byte;
        bind_markers:          Cardinal;
        FProtocol:             TFirebirdBaseDriver;
        FFirebird:             TISC_DB_HANDLE;
        FPrepStmtHandle:       TISC_STMT_HANDLE;
        FDirectStmtHandle:     TISC_STMT_HANDLE;
        FTrHandle:             TISC_TR_HANDLE;
        FDialect:              word;
        columns_bound:         array of Boolean;
        params_bound:          array of Boolean;
        ColumnsInfo:           array of ColumnRecord;
        ParamsInfo:            array of ColumnRecord;
        bind_targets:          TBindTargets;
        SQLDA_bind:            TSQLDAParams;
        SQLDA_columns:         TSQLDAResult;
        last_errors:           PSErrorRecord;
        FLastUpdateCount:      integer;
        Result_set_size:       longword;
        Result_cursor:         longword;
        Result_set_capacity:   longword;
        SQLDAX:                TResultSet;

        procedure clearBuffer;
        procedure initialize_bindings;
        procedure populate_column_info (stmt: TISC_STMT_HANDLE);
        procedure coreColumnPopulation;
        function  execute_direct: boolean;
        function  execute_prepare: boolean;
        procedure free_prepared_statement;
        procedure free_direct_statement;
        function  convertGeneric (raw: TPDOGenericTime; DateForm: TDateForm): TDateTime;
        procedure genericBind (columnIndex: Cardinal; variableAddress: Pointer);
        procedure genericOverride (columnIndex: Cardinal; TypeOverride: TPDODataTypes);
        function  fetch_core_native (var row: TPDORowNative; stmt_handle: TISC_STMT_HANDLE): boolean;
        function  fetch_core_key    (var row: TPDORowKeyStrings; stmt_handle: TISC_STMT_HANDLE): boolean;
        function  fetch_core        (var SQLDA: TSQLDAResult; stmt_handle: TISC_STMT_HANDLE): boolean;
        function  getFieldType (PDODataType: TPDODataTypes): TFirebirdFieldTypes;
        function  checkRebindingNeeded: Boolean;
        procedure prefetch_results (stmt_handle: TISC_STMT_HANDLE);
        procedure TransferVariables (SQLDA: TSQLDAResult);
        procedure free_prefetched_result_set;
        function  close_private_cursor: boolean;

    public
        procedure setSQL  (SQL: AnsiString; StatementType: TPDOStatementType);

        procedure bindValue  (valueIndex: Cardinal;   PassedValue: Variant); overload;
        procedure bindValue  (valueIndex: AnsiString; PassedValue: Variant); overload;
        procedure bindTimeValue  (valueIndex: Cardinal;   PassedValue: TPDOGenericTime); overload;
        procedure bindTimeValue  (valueIndex: AnsiString; PassedValue: TPDOGenericTime); overload;

        procedure bindColumn (columnIndex: Cardinal;   variableAddress: Pointer); overload;
        procedure bindColumn (columnIndex: Cardinal;   variableAddress: Pointer; TypeOverride: TPDODataTypes); overload;
        procedure bindColumn (columnIndex: AnsiString; variableAddress: Pointer); overload;
        procedure bindColumn (columnIndex: AnsiString; variableAddress: Pointer; TypeOverride: TPDODataTypes); overload;

        function closeCursor: boolean;
        function columnCount: Integer;
        function errorCode: AnsiString;
        function errorInfo: PDO_errorInfoRecord;

        function execute: Boolean; overload;
        function execute (DelimitedBind: AnsiString): Boolean; overload;

        function getRowsAffected: Int64;
        function getNumberRows: Int64;
        function LastInsertId: Int64;
        function nextRowSet: Boolean;

        function fetch_bound: Boolean;
        function fetch (var row: TPDORowKeyStrings): Boolean; overload;
        function fetch (var row: TPDORowNative): Boolean; overload;
        function fetch_all (var rowset: TPDORowSetKeyStrings): Boolean; overload;
        function fetch_all (var rowset: TPDORowSetNative): Boolean; overload;

        constructor create (DriverHandle: TFirebirdBaseDriver; DBaseHandle: TISC_DB_HANDLE;
                TransactionHandle: TISC_TR_HANDLE; Firebird_dialect: word);
        procedure beforeDestruction; Override;
    end;


implementation

constructor TPDOFirebirdStatement.create (
    DriverHandle:      TFirebirdBaseDriver;
    DBaseHandle:       TISC_DB_HANDLE;
    TransactionHandle: TISC_TR_HANDLE;
    Firebird_dialect:  word);
begin
    self.DirectSQL              := '';
    self.PreparedSQL            := '';

    self.FLastUpdateCount       := -1;
    self.numberColumns          := 0;
    self.bind_markers           := 0;
    self.ColumnsInfo            := nil;
    self.bind_targets           := nil;

    self.FProtocol              := DriverHandle;
    self.FFirebird              := DBaseHandle;
    self.FTrHandle              := TransactionHandle;
    FDialect                    := Firebird_dialect;
    self.FPrepStmtHandle        := nil;
    self.FDirectStmtHandle      := nil;

    self.alpha_bind_values      := TStringList.Create;
    self.alpha_bind_columns     := TStringList.Create;
    self.rebind_required        := False;
    self.successful_query       := false;
    self.exec_with_prefetch     := false;

    self.Result_set_size        := 0;
    self.Result_cursor          := 0;
    self.Result_set_capacity    := 0;

    self.SQLDA_bind             := TSQLDAParams.Create (self.FProtocol, @self.FFirebird, @self.FTrHandle);
    self.SQLDA_columns          := TSQLDAResult.Create (self.FProtocol, @self.FFirebird, @self.FTrHandle);

    self.last_errors.last_error_msg     := '';
    self.last_errors.last_error_msg_sql := '';
    self.last_errors.last_error_code    := 0;
end;

procedure TPDOFirebirdStatement.beforeDestruction;
begin
    self.alpha_bind_values.Free;
    self.alpha_bind_columns.Free;
    self.SQLDA_bind.Free;
    self.SQLDA_columns.Free;

    self.free_prepared_statement;
    self.free_direct_statement;
    self.free_prefetched_result_set;
end;


procedure TPDOFirebirdStatement.clearBuffer;
begin
    self.numberColumns := 0;
    self.bind_markers  := 0;
    self.params_bound  := nil;
    self.columns_bound := nil;
    self.ColumnsInfo   := nil;
    self.bind_targets  := nil;
    self.rebind_required := false;
    self.Successful_stmt_prep  := false;

end;

procedure TPDOFirebirdStatement.free_prepared_statement;
begin
    if self.FPrepStmtHandle <> nil then
        PDOUtilsFirebird.FreeStatement( self.FProtocol, self.FPrepStmtHandle);
end;

procedure TPDOFirebirdStatement.free_direct_statement;
begin
    if self.FDirectStmtHandle <> nil then
        PDOUtilsFirebird.FreeStatement( self.FProtocol, self.FDirectStmtHandle);

end;

procedure TPDOFirebirdStatement.free_prefetched_result_set;
var
    rowIndex: longword;
begin
    {as fetched result set is read, the rows that have been read free the memory,
     so we only need to free from the cursor to the Result_set_size, which for a
     fully scanned result set is nothing }

    if self.Result_set_capacity = 0 then exit;

    for rowIndex := self.Result_cursor to self.Result_set_size - 1 do
    begin
        self.SQLDAX[rowIndex].Free;
        self.SQLDAX[rowIndex] := nil;
        self.Result_cursor := rowIndex + 1;
    end;
end;


procedure TPDOFirebirdStatement.setSQL (SQL: AnsiString; StatementType: TPDOStatementType);
var
    PS_SQL: AnsiString;
    firebird_stype: TFirebirdStmtType;
    StatusVector: TARRAY_ISC_STATUS;
label
    early_exit,
    free_and_exit;
begin
    self.clearBuffer;
    self.TypeOfStatement := StatementType;
    self.alpha_bind_values.Clear;
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            self.DirectSQL := SQL;
        end
    else
        begin
            PS_SQL := PDOSysUtils.derive_alpha_markers(SQL, self.alpha_bind_values);
            self.PreparedSQL := PS_SQL;

            firebird_stype := PDOUtilsFirebird.PrepareStatement(
                    self.FProtocol, @self.FFirebird, @self.FTrHandle, self.FDialect,
                    PS_SQL, self.FPrepStmtHandle);

            {prepare function frees bad queries and also logs an error}
            if firebird_stype in [stUnknown, stGetSegment, stPutSegment, stStartTrans] then goto early_exit;

            self.Successful_stmt_prep := true;
            self.FProtocol.log(lcPrepStmt, PS_SQL);

            self.FProtocol.isc_dsql_describe_bind(@StatusVector, @self.FPrepStmtHandle,
                self.FDialect, self.SQLDA_bind.Matrix);
            PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcExecute, PS_SQL);
            if self.last_errors.last_error_msg <> '' then goto free_and_exit;


            { Resize XSQLDA structure if needed }
            if self.SQLDA_bind.Matrix^.sqld > self.SQLDA_bind.Matrix^.sqln then
            begin
                self.SQLDA_bind.AllocateSQLDA;
                self.FProtocol.isc_dsql_describe_bind(@StatusVector, @self.FPrepStmtHandle,
                    self.FDialect, self.SQLDA_bind.Matrix);
                PDOUtilsFirebird.CheckPrepStmtError(last_errors, self.FProtocol, StatusVector, lcExecute, PS_SQL);
                if self.last_errors.last_error_msg <> '' then goto free_and_exit;
            end;

            self.SQLDA_bind.InitFields(true);
            self.bind_markers := self.SQLDA_bind.GetFieldCount;
            self.initialize_bindings;
            self.populate_column_info (self.FPrepStmtHandle);
            if (self.numberColumns > 0) then self.rebind_required := true;
        end;
    exit;

free_and_exit:
    PDOUtilsFirebird.FreeStatement(self.FProtocol, self.FPrepStmtHandle);

early_exit:
     self.Successful_stmt_prep := false;
end;

procedure TPDOFirebirdStatement.initialize_bindings;
var
    loop: Byte;
begin
    setLength(self.params_bound, self.bind_markers);
    setLength(self.ParamsInfo,   self.bind_markers);
    loop := 0;
    while (loop < self.bind_markers) do
    begin
        self.params_bound[loop] := false;

        self.ParamsInfo[loop].Name       := self.SQLDA_bind.GetFieldAliasName(loop);
        self.ParamsInfo[loop].Datatype   := self.SQLDA_bind.GetFieldSqlType(loop);
        self.ParamsInfo[loop].Datatype2  := self.SQLDA_bind.GetIbSqlType(loop);
        self.ParamsInfo[loop].DataLength := self.SQLDA_bind.GetFieldLength(loop);
        self.ParamsInfo[loop].Scale      := self.SQLDA_bind.GetFieldScale(loop);

        loop := loop + 1;
    end;
end;

procedure TPDOFirebirdStatement.populate_column_info  (stmt: TISC_STMT_HANDLE);
var
    StatusVector: TARRAY_ISC_STATUS;
    loop: smallInt;
begin
    self.FProtocol.isc_dsql_describe(@StatusVector, @stmt, self.FDialect, self.SQLDA_columns.Matrix);
    PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcExecute);

    { Resize XSQLDA structure if needed }
    if self.SQLDA_columns.Matrix^.sqld > self.SQLDA_columns.Matrix^.sqln then
    begin
        self.SQLDA_columns.AllocateSQLDA;
        self.FProtocol.isc_dsql_describe(@StatusVector, @stmt, self.FDialect, self.SQLDA_columns.Matrix);
        PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcExecute);
    end;
    {
        We aren't going to use this structure for input or output data, so no need to initialize it
        self.SQLDA_columns.InitFields(false);
    }


    self.numberColumns := self.SQLDA_columns.GetFieldCount;
    setLength(self.columns_bound, self.numberColumns);
    setLength(self.bind_targets,  self.numberColumns);
    self.alpha_bind_columns.Clear;
    self.coreColumnPopulation;
    for loop := 0 to self.numberColumns - 1 do
        self.columns_bound[loop] := false;
end;


procedure TPDOFirebirdStatement.coreColumnPopulation;
var
    casemode, loop: Byte;
    keystring: AnsiString;
begin
    SetLength(self.ColumnsInfo, self.numberColumns);
    casemode := self.FProtocol.GetCaseMode;
    loop := 0;
    while (loop < self.numberColumns) do
    begin
        self.ColumnsInfo[loop].Name       := recase (self.SQLDA_columns.GetFieldSqlName(loop), casemode);
        self.ColumnsInfo[loop].Datatype   := self.SQLDA_columns.GetFieldSqlType(loop);
        self.ColumnsInfo[loop].Datatype2  := self.SQLDA_columns.GetIbSqlType(loop);
        self.ColumnsInfo[loop].DataLength := self.SQLDA_columns.GetFieldLength(loop);
        self.ColumnsInfo[loop].Scale      := self.SQLDA_columns.GetFieldScale(loop);

        keystring := AnsiUpperCase(self.ColumnsInfo[loop].Name);
        if (self.alpha_bind_columns.IndexOfName(keystring) = - 1) then
            begin
                keystring := keystring + '=' + IntToStr(loop);
                self.alpha_bind_columns.Add(keystring);
            end;
        loop := loop + 1;
    end;
end;

function TPDOFirebirdStatement.columnCount: Integer;
Begin
    Result := self.numberColumns;
End;

function TPDOFirebirdStatement.errorCode: AnsiString;
begin
    Result := '00000';
end;

function TPDOFirebirdStatement.errorInfo: PDO_errorInfoRecord;
begin
    Result.SQLState := '00000';
    Result.Error_code := self.last_errors.last_error_code;
    Result.Error_msg  := format ('%s [%s]', [self.last_errors.last_error_msg, self.last_errors.last_error_msg_sql]);
end;

function  TPDOFirebirdStatement.execute_direct: boolean;
var
  StatementType: TFirebirdStmtType;
  StatusVector: TARRAY_ISC_STATUS;
begin
    {This function will always be called right after the statement class is created.  It's not possible to
     re-use a statement, so there is no need to check to see if the handles are free}
    Result := false;

    StatementType := PDOUtilsFirebird.PrepareStatement(self.FProtocol, @self.FFirebird,
                        @self.FTrHandle, self.FDialect, self.DirectSQL, self.FDirectStmtHandle);

    if StatementType in [stUnknown, stGetSegment, stPutSegment, stStartTrans] then exit;

    self.populate_column_info(self.FDirectStmtHandle);

    self.FProtocol.isc_dsql_execute (@StatusVector, @self.FTrHandle, @self.FDirectStmtHandle, self.FDialect, nil);
    PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcExecute, self.DirectSQL);
    Result := (self.last_errors.last_error_msg = '');
    self.successful_query := Result;

    self.FLastUpdateCount := -1;
    case StatementType of
        stUpdate,
        stDelete,
        stInsert: self.FLastUpdateCount := PDOUtilsFirebird.GetAffectedRows(self.FProtocol, self.FDirectStmtHandle, StatementType);
        stSelect:
            begin
                  self.prefetch_results(self.FDirectStmtHandle);
                  
                  if self.exec_with_prefetch then
                    self.FLastUpdateCount := self.Result_set_size
                  else
                    self.FLastUpdateCount := -1;
            end;
    end;
    self.FProtocol.log(lcExecute, self.DirectSQL);
end;

function  TPDOFirebirdStatement.execute_prepare: boolean;
var
    loop: smallint;
    StatusVector: TARRAY_ISC_STATUS;
    input_parameters: PXSQLDA;
begin
    Result := false;
    self.successful_query := false;

    for loop := 0 to self.bind_markers - 1 do
        if not self.params_bound[loop] then
        begin
            self.FProtocol.log(lcExecute, SFailedToBindAllValues, 0, format ('param %d', [loop + 1]));
            exit;
        end;

    if not self.Successful_stmt_prep then
    begin
        self.FProtocol.log(lcExecute, SAttemptExecOnBadPrep);
        exit;
    end;

    if self.bind_markers = 0 then
        input_parameters := nil
    else
        input_parameters := self.SQLDA_bind.Matrix;
    self.FProtocol.isc_dsql_execute (@StatusVector, @self.FTrHandle, @self.FPrepStmtHandle, self.FDialect, input_parameters);
    PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcExecPrepStmt);
    if self.last_errors.last_error_msg = '' then
        begin
            self.FProtocol.log(lcExecPrepStmt, self.PreparedSQL);
            Result := true;
            self.Successful_query := true;
            self.prefetch_results(self.FPrepStmtHandle);
        end;
end;

function TPDOFirebirdStatement.execute: boolean;
begin
    if self.TypeOfStatement = DIRECT_STATEMENT then
        Result := self.execute_direct
    else
        Result := self.execute_prepare;
end;

function TPDOFirebirdStatement.getRowsAffected: Int64;
begin
    Result := self.FLastUpdateCount;
end;

procedure TPDOFirebirdStatement.bindValue  (valueIndex: Cardinal;   PassedValue: Variant);
var
    DType: TZSQLType;
    TC_INT64: Int64;
begin
    if (valueIndex >= self.bind_markers) then
        begin
            self.FProtocol.log(lcPrepStmt, format(SBindVarOutOfRange,[valueIndex]) ,0,' max < ' + IntToStr(self.bind_markers));
            exit;
        end;

    DType := self.SQLDA_bind.GetFieldSqlType(valueIndex);
    case DType of
        stBoolean       : self.SQLDA_bind.UpdateBoolean     (valueIndex, Boolean (PassedValue));
        stByte          : self.SQLDA_bind.UpdateByte        (valueIndex, ShortInt(PassedValue));
        stShort         : self.SQLDA_bind.UpdateShort       (valueIndex, SmallInt(PassedValue));
        stInteger       : self.SQLDA_bind.UpdateInt         (valueIndex, Integer(PassedValue));
        stFloat         : self.SQLDA_bind.UpdateFloat       (valueIndex, single (PassedValue));
        stDouble        : self.SQLDA_bind.UpdateDouble      (valueIndex, double (PassedValue));
        stBigDecimal    : self.SQLDA_bind.UpdateBigDecimal  (valueIndex, extended (PassedValue));
        stString        : self.SQLDA_bind.UpdateString      (valueIndex, ansistring (PassedValue));
        stUnicodeString : self.SQLDA_bind.UpdateString      (valueIndex, widestring (PassedValue));
        stBytes         : self.SQLDA_bind.UpdateBytes       (valueIndex, PDOSysUtils.StrToBytes(PassedValue));
        stLong          : begin
                               {for some reason, can't cast variant to Int64}
                                TC_INT64        := PassedValue;
                                self.SQLDA_bind.UpdateLong        (valueIndex, TC_INT64);
                          end;
        {skipping blobs for now: stAsciiStream, stUnicodeStream, stBinaryStream}
    else
            raise EZIBConvertError.Create(SUnsupportedParameterType);
    end;

    {on bind failure, this line will be skipped due to the else exception in the case statement }
    self.params_bound[valueIndex] := true;

end;

procedure TPDOFirebirdStatement.bindValue  (valueIndex: AnsiString; PassedValue: Variant);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_values.IndexOfName(valueIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [valueIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_values.ValueFromIndex[localIndex]);
    self.bindValue(trueIndex, PassedValue);
end;

procedure TPDOFirebirdStatement.bindTimeValue  (valueIndex: Cardinal;   PassedValue: TPDOGenericTime);
var
    DType: TZSQLType;
begin
    if (valueIndex >= self.bind_markers) then
        begin
            self.FProtocol.log(lcPrepStmt, format(SBindVarOutOfRange,[valueIndex]) ,0,' max < ' + IntToStr(self.bind_markers));
            exit;
        end;

    DType := self.SQLDA_bind.GetFieldSqlType(valueIndex);
    case DType of
        stDate          : self.SQLDA_bind.UpdateDate       (valueIndex, convertGeneric (PassedValue, DF_DATE));
        stTime          : self.SQLDA_bind.UpdateTime       (valueIndex, convertGeneric (PassedValue, DF_TIME));
        stTimestamp     : self.SQLDA_bind.UpdateTimestamp  (valueIndex, convertGeneric (PassedValue, DF_TIMESTAMP));
    else
            raise EZIBConvertError.Create(SUnsupportedParameterType);
    end;

    {on bind failure, this line will be skipped due to the else exception in the case statement }
    self.params_bound[valueIndex] := true;

end;

procedure TPDOFirebirdStatement.bindTimeValue  (valueIndex: AnsiString; PassedValue: TPDOGenericTime);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_values.IndexOfName(valueIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [valueIndex]));
            exit
        end;
    trueIndex := StrToInt(self.alpha_bind_values.ValueFromIndex[localIndex]);
    self.bindTimeValue(trueIndex, PassedValue);
end;

procedure TPDOFirebirdStatement.bindColumn (columnIndex: Cardinal;   variableAddress: Pointer);
begin
    self.genericBind(columnIndex, variableAddress);
end;

procedure TPDOFirebirdStatement.bindColumn (columnIndex: Cardinal;   variableAddress: Pointer; TypeOverride: TPDODataTypes);
begin
    self.genericBind(columnIndex, variableAddress);
    self.genericOverride(columnIndex, TypeOverride);
end;

procedure TPDOFirebirdStatement.bindColumn (columnIndex: AnsiString; variableAddress: Pointer);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_columns.IndexOfName(columnIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [columnIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_columns.ValueFromIndex[localIndex]);
    self.genericBind(trueIndex, variableAddress);
end;

procedure TPDOFirebirdStatement.bindColumn (columnIndex: AnsiString; variableAddress: Pointer; TypeOverride: TPDODataTypes);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_columns.IndexOfName(columnIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [columnIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_columns.ValueFromIndex[localIndex]);
    self.genericBind(trueIndex, variableAddress);
    self.genericOverride(trueIndex, TypeOverride);
end;

function TPDOFirebirdStatement.close_private_cursor: boolean;
begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            self.free_direct_statement;
            result := self.FDirectStmtHandle = nil;
        end
    else
        begin
            PDOUtilsFirebird.closeCursor(self.FProtocol, self.FPrepStmtHandle);
            result := self.FPrepStmtHandle = nil;
        end;
end;

function TPDOFirebirdStatement.closeCursor: boolean;
begin
    result := self.close_private_cursor;
    self.free_prefetched_result_set;
end;

function TPDOFirebirdStatement.execute (DelimitedBind: AnsiString): Boolean;
const
    pipe = '|';
var
    dx, counter: integer;
    txt, ns: AnsiString;
Begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            Result := false;
            self.FProtocol.log(lcPrepStmt, SUsedWrongExecute);
            exit;
        end;
     txt := DelimitedBind + pipe;
     counter := 0;
     while (Length(txt) > 0) do
     begin
        dx := AnsiPos(pipe,txt);
        ns := copy (txt, 0, dx - 1);
        txt := copy (txt, dx + 1, MaxInt);
        self.bindValue(counter,ns);
     counter := counter + 1;
     end;
     Result := self.execute;
end;

function TPDOFirebirdStatement.getNumberRows: Int64;
begin
    if self.exec_with_prefetch then
        Result := self.Result_set_size
    else
        Result := -1;
end;

function TPDOFirebirdStatement.LastInsertId: Int64;
begin
    { TODO : function TPDOFirebirdStatement.LastInsertId: Int64;
             This requires a generator that returns the gen id.  We need
             to standardize on the first return parameter or something}
    Result := 0;
end;

function TPDOFirebirdStatement.nextRowSet: Boolean;
begin
    { MULTIPLE ROWSETS NOT SUPPORTED BY FIREBIRD 2.0  }
    Result := false;
end;

function TPDOFirebirdStatement.fetch_bound: Boolean;
var
    SQLDA: TSQLDAResult;
begin
    Result := false;
    if (self.checkRebindingNeeded) then exit;    {bind failure}

    if not self.fetch_core(SQLDA, self.FPrepStmtHandle) then exit;

    self.TransferVariables (SQLDA);
    Result := true;
end;

function TPDOFirebirdStatement.fetch (var row: TPDORowKeyStrings): Boolean;
begin
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
        Result := self.fetch_core_key(row, self.FPrepStmtHandle)
    else
        Result := self.fetch_core_key(row, self.FDirectStmtHandle);
end;

function TPDOFirebirdStatement.fetch (var row: TPDORowNative): Boolean;
begin
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
        Result := self.fetch_core_native(row, self.FPrepStmtHandle)
    else
        Result := self.fetch_core_native(row, self.FDirectStmtHandle);
end;

function TPDOFirebirdStatement.fetch_all (var rowset: TPDORowSetKeyStrings): Boolean;
var
    SQLDA: TSQLDAResult;
    column: smallInt;
    keystring: ansistring;
    stmt_handle: TISC_STMT_HANDLE;
    rowcount: longword;
begin
    Result := False;
    rowset.ClearRowSet;
    if not self.exec_with_prefetch then exit;

    if (self.TypeOfStatement = PREPARED_STATEMENT) then
        stmt_handle := self.FPrepStmtHandle
    else
        stmt_handle := self.FDirectStmtHandle;

    rowcount := 0;
    rowset.InitializeRowSet(self.Result_set_size - self.Result_cursor);
    while self.fetch_core(SQLDA, stmt_handle) do
    begin
        result := true;
        for column := 0 to self.numberColumns - 1 do
        begin
                keystring := format('%s=%s', [self.ColumnsInfo[column].Name, SQLDA.GetString(column)]);
                rowset.row[rowcount].Add(keystring);
        end;
        rowcount := rowcount + 1;
    end;

end;

function TPDOFirebirdStatement.fetch_all (var rowset: TPDORowSetNative): Boolean;
var
    SQLDA: TSQLDAResult;
    column: smallInt;
    null: Boolean;
    nullmode: Byte;
    coltype: TZSQLType;
    stringRep: AnsiString;
    timestamp: TDateTime;
    stmt_handle: TISC_STMT_HANDLE;
    rowcount: longword;
begin
    Result := False;
    rowset.ClearRowSet;
    if not self.exec_with_prefetch then exit;

    if (self.TypeOfStatement = PREPARED_STATEMENT) then
        stmt_handle := self.FPrepStmtHandle
    else
        stmt_handle := self.FDirectStmtHandle;

    nullmode := self.FProtocol.GetOracleNull;
    rowcount := 0;
    rowset.InitializeRowSet(self.Result_set_size - self.Result_cursor);
    while self.fetch_core(SQLDA, stmt_handle) do
    begin
        result := true;
        rowset.row[rowcount].InitializeRow(self.numberColumns);
        for column := 0 to self.numberColumns - 1 do
        begin
            coltype   := SQLDA.GetFieldSqlType(column);
            timestamp := SQLDA.GetTimestamp(column);
            stringRep := SQLDA.GetString(column);
            null      := SQLDA.IsNull(column);

            if (nullmode = EMPTY_STRING_NULL) and
                (coltype = stString) and
                (stringRep = '') then null := true;

            rowset.row[rowcount].column[column] := TFirebirdResult.Create (coltype,stringRep,null,timestamp);
            rowset.row[rowcount].InsertKey(column,self.ColumnsInfo[column].Name);
        end;
        rowcount := rowcount + 1;
    end;

end;

function  TPDOFirebirdStatement.fetch_core_native (var row: TPDORowNative; stmt_handle: TISC_STMT_HANDLE): boolean;
var
    column: smallint;
    null: Boolean;
    nullmode: Byte;
    coltype: TZSQLType;
    stringRep: AnsiString;
    timestamp: TDateTime;
    SQLDA: TSQLDAResult;
begin
    nullmode := self.FProtocol.GetOracleNull;
    row.Clearcolumns;
    row.InitializeRow(self.numberColumns);
    Result := false;

    if not self.fetch_core(SQLDA, @stmt_handle) then exit;

    for column := 0 to self.numberColumns - 1 do
    begin
        coltype   := SQLDA.GetFieldSqlType(column);
        timestamp := SQLDA.GetTimestamp(column);
        stringRep := SQLDA.GetString(column);
        null      := SQLDA.IsNull(column);

        if (nullmode = EMPTY_STRING_NULL) and
           (coltype = stString) and
           (stringRep = '') then null := true;

        row.column[column] := TFirebirdResult.Create (coltype,stringRep,null,timestamp);
        row.InsertKey(column,self.ColumnsInfo[column].Name);
    end;

    Result := True;
end;

function TPDOFirebirdStatement.fetch_core_key (var row: TPDORowKeyStrings; stmt_handle: TISC_STMT_HANDLE): boolean;
var
    keystring: AnsiString;
    column: smallint;
    stringRep: AnsiString;
    SQLDA: TSQLDAResult;
begin
    Result := false;
    row.Clear;

    if not self.fetch_core(SQLDA, @stmt_handle) then exit;

    for column := 0 to self.numberColumns - 1 do
    begin
        stringRep := SQLDA.GetString(column);
        keystring := format ('%s=%s', [self.ColumnsInfo[column].Name, stringRep]);
        row.Add(keystring);
    end;

    Result := True;
end;


function TPDOFirebirdStatement.convertGeneric (raw: TPDOGenericTime; DateForm: TDateForm): TDateTime;
var
    the_date,
    the_time: TDateTime;
Begin
    the_date := sysUtils.EncodeDate(raw.year, raw.month, raw.day);
    the_time := sysUtils.EncodeTime(raw.hour, raw.minute, raw.second, raw.second_part div 1000);
    case DateForm of
        DF_DATE: Result := the_date;
        DF_TIME: Result := the_time;
    else
        Result := the_date + the_time;
    end;
End;

procedure TPDOFirebirdStatement.genericBind (columnIndex: Cardinal; variableAddress: Pointer);
var
    standard_msg: ansistring;
    error_msg: ansistring;
Begin
    if (columnIndex >= self.numberColumns) then
        begin
            standard_msg := format(SBindColumnOutOfRange,[columnIndex]);
            error_msg    := ' max < ' + IntToStr(self.numberColumns);

            self.FProtocol.log(lcPrepStmt, standard_msg, 0, error_msg);
            exit;
        end;
    self.bind_targets[columnIndex].bind_target   := variableAddress;
    self.bind_targets[columnIndex].override_type := byte_buffer;
    self.bind_targets[columnIndex].override_set  := false;

    self.columns_bound[columnIndex] := true;
    self.rebind_required := true;
End;

procedure TPDOFirebirdStatement.genericOverride (columnIndex: Cardinal; TypeOverride: TPDODataTypes);
Begin
    if (columnIndex >= self.numberColumns) then exit;  {already alerted this problem a line before}
    self.bind_targets[columnIndex].override_type := TypeOverride;
    self.bind_targets[columnIndex].override_set  := true;
End;

function TPDOFirebirdStatement.checkRebindingNeeded: Boolean;
var
    loop: smallInt;
Begin
    Result := False;
    if not self.rebind_required then exit;

   {check if all columns bound}

    for loop := 0 to self.numberColumns - 1 do
        if not self.columns_bound[loop] then
        begin
            self.FProtocol.log(lcPrepStmt, SFailedToBindAllColumns, 0, 'column ' + IntToStr(loop));
            Result := true;
            exit;
        end;

    self.rebind_required := false;
End;


procedure TPDOFirebirdStatement.TransferVariables (SQLDA: TSQLDAResult);
var
    column: smallint;
begin
    for column := 0 to self.numberColumns - 1 do
        if not self.bind_targets[column].override_set then
            SQLDA.bind_native_format(column, self.bind_targets[column].bind_target)
        else
            SQLDA.bind_explicit_format(column, self.bind_targets[column].bind_target,
                self.getFieldType(self.bind_targets[column].override_type));
end;


function TPDOFirebirdStatement.getFieldType (PDODataType: TPDODataTypes): TFirebirdFieldTypes;
Begin
    case PDODataType of
        integer_8:          Result := SQL_SHORT;
        integer_8_signed:   Result := SQL_SHORT;
        integer_16:         Result := SQL_SHORT;
        integer_16_signed:  Result := SQL_SHORT;
        integer_32:         Result := SQL_LONG;
        integer_32_signed:  Result := SQL_LONG;
        integer_64_signed:  Result := SQL_INT64;
        float_32:           Result := SQL_FLOAT;
        float_64:           Result := SQL_DOUBLE;
        string_1:           Result := SQL_TEXT;
        string_unlimited:   Result := SQL_VARYING;
        generic_time:       Result := SQL_TIMESTAMP;
        byte_buffer:        Result := SQL_BLOB;
    else
        Result := SQL_SHORT;
    end;
End;

function  TPDOFirebirdStatement.fetch_core (var SQLDA: TSQLDAResult; stmt_handle: TISC_STMT_HANDLE): boolean;
var
    return_code: ISC_STATUS;
    StatusVector: TARRAY_ISC_STATUS;
begin

    Result := false;
    SQLDA  := nil;

    if self.exec_with_prefetch then
    begin
        if not self.cursor_parked then
            begin
                self.SQLDAX[self.Result_cursor].Free;
                self.SQLDAX[self.Result_cursor] := nil;
                self.Result_cursor := self.Result_cursor + 1;
            end
        else
            self.cursor_parked := false;

        if self.Result_cursor < self.Result_set_size then
        begin
            SQLDA  := self.SQLDAX[self.Result_cursor];
            Result := true;
        end;
        exit;
    end;

    {If we get this far, the result set was not prefetched, and we need to grab it manually}

    if not self.successful_query then
    begin
        self.FProtocol.log(lcOther, SLastQueryFailed);
    	exit;
    end;

    return_code := self.FProtocol.isc_dsql_fetch(@StatusVector, @stmt_handle, self.FDialect, self.SQLDAX[0].Matrix);

    if (return_code = 100) then
    begin
        self.close_private_cursor;
        exit;
    end;

    if (return_code <> 0) then
    begin
        PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcOther,
            format(SQueryFailed, ['[FETCH]']));
        exit;
    end;

    SQLDA := self.SQLDAX[0];
    Result := true;
end;

procedure TPDOFirebirdStatement.prefetch_results (stmt_handle: TISC_STMT_HANDLE);
var
    advance: boolean;
    return_code: ISC_STATUS;
    StatusVector: TARRAY_ISC_STATUS;
begin
    self.exec_with_prefetch := self.FProtocol.GetBufferQuery;
    self.cursor_parked      := true;
    self.Result_cursor      := 0;

    if not self.exec_with_prefetch then
    begin
        {Result_cursor is a longword (unsigned) so we can't set it to -1.  This is
         why cursor_parked exists.  When that's true, it's like the cursor is -1 }

        self.Result_set_size        := 1;
        self.Result_set_capacity    := 1;

        setlength(self.SQLDAX, 1);
        self.SQLDAX[0] := TSQLDAResult.Create (self.FProtocol, @self.FFirebird, @self.FTrHandle, self.numberColumns);
        self.SQLDAX[0].copy_from(self.SQLDA_columns.Matrix^.sqlvar);
        self.SQLDAX[0].InitFields(false);
        exit;
    end;

    if not self.successful_query then exit;

    self.Result_set_size := 0;
    advance := true;
    writeln (self.sqlda_columns.getfieldsqlname(1));

    while advance do
    begin
        if self.Result_set_size = self.Result_set_capacity then
        begin
            self.Result_set_capacity := self.Result_set_capacity + RS_CAPACITY_INCREMENT;
            setlength(self.SQLDAX, self.Result_set_capacity);
        end;

        self.SQLDAX[self.Result_set_size] := TSQLDAResult.Create (self.FProtocol, @self.FFirebird, @self.FTrHandle, self.numberColumns);
        self.SQLDAX[self.Result_set_size].copy_from(self.SQLDA_columns.Matrix^.sqlvar);
        self.SQLDAX[self.Result_set_size].InitFields(false);

        return_code := self.FProtocol.isc_dsql_fetch(@StatusVector, @stmt_handle, self.FDialect,
                                                     self.SQLDAX[self.Result_set_size].Matrix);

        if (return_code = 100) then
        begin
            self.close_private_cursor;
            advance := false;
        end;

        if advance and (return_code <> 0) then
        begin
            PDOUtilsFirebird.CheckPrepStmtError(self.last_errors, self.FProtocol, StatusVector, lcOther,
                format(SQueryFailed, ['[PRE-FETCH]']));
            self.close_private_cursor;
            advance := false;
        end;

        if advance then
            self.Result_set_size := self.Result_set_size + 1;
    end;

end;

end.
