{ $Id: PDOUtilsFirebird.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOUtilsFirebird;

interface

{$I ../../directives.inc}


uses Classes, PDOFirebirdDriver, PDOConstantsFirebird, PDOLogging, SysUtils, PDOSysUtils,
        PDOInterfaces, PDOMessages;

type
  { Paparameter string name and it value}
  TZIbParam = record
    Name: string;
    Number: word;
  end;
  PZIbParam = ^TZIbParam;

  { Interbase Statement Type }
  TFirebirdStmtType = (stUnknown, stSelect, stInsert, stUpdate, stDelete,
    stDDL, stGetSegment, stPutSegment, stExecProc, stStartTrans, stCommit,
    stRollback, stSelectForUpdate, stSetGenerator);

  EZIBConvertError = class(Exception);

  { Interbase blob Information structure contain iformation about blob size in bytes,
    segments count, segment size in bytes and blob type.  Note: blob type can be text an binary }
  TIbBlobInfo = record
    NumSegments: SmallInt;
    MaxSegmentSize: SmallInt;
    BlobType: SmallInt;
    TotalSize: LongInt;
  end;

    PSErrorRecord = record
        last_error_msg     : ansistring;
        last_error_code    : LongInt;
        last_error_msg_sql : ansistring;
    end;


const
  { Default Interbase blob size for readig }
  DefaultBlobSegmentSize = 16 * 1024;

  IBScaleDivisor: array[-15..-1] of Int64 = (1000000000000000,100000000000000,
    10000000000000,1000000000000,100000000000,10000000000,1000000000,100000000,
    10000000,1000000,100000,10000,1000,100,10);

  { count database parameters }
  MAX_DPB_PARAMS = 67;
  { prefix database parameters names it used in paramters scann procedure }
  BPBPrefix = 'isc_dpb_';
  { list database parameters and their apropriate numbers }
  DatabaseParams: array [0..MAX_DPB_PARAMS]of TZIbParam = (
    (Name:'isc_dpb_version1';         Number: isc_dpb_version1),
    (Name:'isc_dpb_cdd_pathname';     Number: isc_dpb_cdd_pathname),
    (Name:'isc_dpb_allocation';       Number: isc_dpb_allocation),
    (Name:'isc_dpb_journal';          Number: isc_dpb_journal),
    (Name:'isc_dpb_page_size';        Number: isc_dpb_page_size),
    (Name:'isc_dpb_num_buffers';      Number: isc_dpb_num_buffers),
    (Name:'isc_dpb_buffer_length';    Number: isc_dpb_buffer_length),
    (Name:'isc_dpb_debug';            Number: isc_dpb_debug),
    (Name:'isc_dpb_garbage_collect';  Number: isc_dpb_garbage_collect),
    (Name:'isc_dpb_verify';           Number: isc_dpb_verify),
    (Name:'isc_dpb_sweep';            Number: isc_dpb_sweep),
    (Name:'isc_dpb_enable_journal';   Number: isc_dpb_enable_journal),
    (Name:'isc_dpb_disable_journal';  Number: isc_dpb_disable_journal),
    (Name:'isc_dpb_dbkey_scope';      Number: isc_dpb_dbkey_scope),
    (Name:'isc_dpb_number_of_users';  Number: isc_dpb_number_of_users),
    (Name:'isc_dpb_trace';            Number: isc_dpb_trace),
    (Name:'isc_dpb_no_garbage_collect'; Number: isc_dpb_no_garbage_collect),
    (Name:'isc_dpb_damaged';          Number: isc_dpb_damaged),
    (Name:'isc_dpb_license';          Number: isc_dpb_license),
    (Name:'isc_dpb_sys_user_name';    Number: isc_dpb_sys_user_name),
    (Name:'isc_dpb_encrypt_key';      Number: isc_dpb_encrypt_key),
    (Name:'isc_dpb_activate_shadow';  Number: isc_dpb_activate_shadow),
    (Name:'isc_dpb_sweep_interval';   Number: isc_dpb_sweep_interval),
    (Name:'isc_dpb_delete_shadow';    Number: isc_dpb_delete_shadow),
    (Name:'isc_dpb_force_write';      Number: isc_dpb_force_write),
    (Name:'isc_dpb_begin_log';        Number: isc_dpb_begin_log),
    (Name:'isc_dpb_quit_log';         Number: isc_dpb_quit_log),
    (Name:'isc_dpb_no_reserve';       Number: isc_dpb_no_reserve),
    (Name:'isc_dpb_username';         Number: isc_dpb_user_name),
    (Name:'isc_dpb_password';         Number: isc_dpb_password),
    (Name:'isc_dpb_password_enc';     Number: isc_dpb_password_enc),
    (Name:'isc_dpb_sys_user_name_enc';  Number: isc_dpb_sys_user_name_enc),
    (Name:'isc_dpb_interp';           Number: isc_dpb_interp),
    (Name:'isc_dpb_online_dump';      Number: isc_dpb_online_dump),
    (Name:'isc_dpb_old_file_size';    Number: isc_dpb_old_file_size),
    (Name:'isc_dpb_old_num_files';    Number: isc_dpb_old_num_files),
    (Name:'isc_dpb_old_file';         Number: isc_dpb_old_file),
    (Name:'isc_dpb_old_start_page';   Number: isc_dpb_old_start_page),
    (Name:'isc_dpb_old_start_seqno';  Number: isc_dpb_old_start_seqno),
    (Name:'isc_dpb_old_start_file';   Number: isc_dpb_old_start_file),
    (Name:'isc_dpb_drop_walfile';     Number: isc_dpb_drop_walfile),
    (Name:'isc_dpb_old_dump_id';      Number: isc_dpb_old_dump_id),
    (Name:'isc_dpb_wal_backup_dir';   Number: isc_dpb_wal_backup_dir),
    (Name:'isc_dpb_wal_chkptlen';     Number: isc_dpb_wal_chkptlen),
    (Name:'isc_dpb_wal_numbufs';      Number: isc_dpb_wal_numbufs),
    (Name:'isc_dpb_wal_bufsize';      Number: isc_dpb_wal_bufsize),
    (Name:'isc_dpb_wal_grp_cmt_wait'; Number: isc_dpb_wal_grp_cmt_wait),
    (Name:'isc_dpb_lc_messages';      Number: isc_dpb_lc_messages),
    (Name:'isc_dpb_lc_ctype';         Number: isc_dpb_lc_ctype),
    (Name:'isc_dpb_cache_manager';    Number: isc_dpb_cache_manager),
    (Name:'isc_dpb_shutdown';         Number: isc_dpb_shutdown),
    (Name:'isc_dpb_online';           Number: isc_dpb_online),
    (Name:'isc_dpb_shutdown_delay';   Number: isc_dpb_shutdown_delay),
    (Name:'isc_dpb_reserved';         Number: isc_dpb_reserved),
    (Name:'isc_dpb_overwrite';        Number: isc_dpb_overwrite),
    (Name:'isc_dpb_sec_attach';       Number: isc_dpb_sec_attach),
    (Name:'isc_dpb_disable_wal';      Number: isc_dpb_disable_wal),
    (Name:'isc_dpb_connect_timeout';  Number: isc_dpb_connect_timeout),
    (Name:'isc_dpb_dummy_packet_interval'; Number: isc_dpb_dummy_packet_interval),
    (Name:'isc_dpb_gbak_attach';      Number: isc_dpb_gbak_attach),
    (Name:'isc_dpb_sql_role_name';    Number: isc_dpb_sql_role_name),
    (Name:'isc_dpb_set_page_buffers'; Number: isc_dpb_set_page_buffers),
    (Name:'isc_dpb_working_directory';  Number: isc_dpb_working_directory),
    (Name:'isc_dpb_sql_dialect';      Number: isc_dpb_SQL_dialect),
    (Name:'isc_dpb_set_db_readonly';  Number: isc_dpb_set_db_readonly),
    (Name:'isc_dpb_set_db_sql_dialect'; Number: isc_dpb_set_db_SQL_dialect),
    (Name:'isc_dpb_gfix_attach';      Number: isc_dpb_gfix_attach),
    (Name:'isc_dpb_gstat_attach';     Number: isc_dpb_gstat_attach)
  );

  { count transaction parameters }
  MAX_TPB_PARAMS = 16;
  { prefix transaction parameters names it used in paramters scann procedure }
  TPBPrefix = 'isc_tpb_';
  { list transaction parameters and their apropriate numbers }
  TransactionParams: array [0..MAX_TPB_PARAMS]of TZIbParam = (
    (Name:'isc_tpb_version1';         Number: isc_tpb_version1),
    (Name:'isc_tpb_version3';         Number: isc_tpb_version3),
    (Name:'isc_tpb_consistency';      Number: isc_tpb_consistency),
    (Name:'isc_tpb_concurrency';      Number: isc_tpb_concurrency),
    (Name:'isc_tpb_exclusive';        Number: isc_tpb_exclusive),
    (Name:'isc_tpb_shared';           Number: isc_tpb_shared),
    (Name:'isc_tpb_protected';        Number: isc_tpb_protected),
    (Name:'isc_tpb_wait';             Number: isc_tpb_wait),
    (Name:'isc_tpb_nowait';           Number: isc_tpb_nowait),
    (Name:'isc_tpb_read';             Number: isc_tpb_read),
    (Name:'isc_tpb_write';            Number: isc_tpb_write),
    (Name:'isc_tpb_ignore_limbo';     Number: isc_tpb_ignore_limbo),
    (Name:'isc_tpb_read_committed';   Number: isc_tpb_read_committed),
    (Name:'isc_tpb_rec_version';      Number: isc_tpb_rec_version),
    (Name:'isc_tpb_no_rec_version';   Number: isc_tpb_no_rec_version),
    (Name:'isc_tpb_lock_read';        Number: isc_tpb_lock_read),
    (Name:'isc_tpb_lock_write';       Number: isc_tpb_lock_write)
    );

procedure CheckFirebirdError(Protocol: TFirebirdBaseDriver; StatusVector: TARRAY_ISC_STATUS;
    LogCategory: TZLoggingCategory = lcOther; LogMessage: ansistring = '');
procedure CheckPrepStmtError(var errors: PSErrorRecord; Protocol: TFirebirdBaseDriver;  StatusVector: TARRAY_ISC_STATUS;
        LogCategory: TZLoggingCategory = lcOther; LogMessage: ansistring = '');
function GenerateDPB(Info: TStrings; var FDPBLength, Dialect: Word): PChar;
function GenerateTPB(Params: TStrings; var Handle: TISC_DB_HANDLE): PISC_TEB;
function GetFirebirdDatabaseParamNumber(const Value: ansistring): word;
procedure FreeStatement(Protocol: TFirebirdBaseDriver; var StatementHandle: TISC_STMT_HANDLE);
procedure closeCursor(Protocol: TFirebirdBaseDriver; var StatementHandle: TISC_STMT_HANDLE);
function GetStatementType(Protocol: TFirebirdBaseDriver; StmtHandle: TISC_STMT_HANDLE): TFirebirdStmtType;
function PrepareStatement(Protocol: TFirebirdBaseDriver; Handle: PISC_DB_HANDLE; TrHandle:
    PISC_TR_HANDLE; Dialect: Word; SQL: ansistring; var StmtHandle: TISC_STMT_HANDLE):  TFirebirdStmtType;
function GetAffectedRows(Protocol: TFirebirdBaseDriver; StmtHandle: TISC_STMT_HANDLE;
            StatementType: TFirebirdStmtType): Integer;

function XSQLDA_LENGTH(Value: word): LongInt;
function BufferToStr(Buffer: PChar; Length: LongInt): string;
function GetNameSqlType(Value: TFirebirdFieldTypes): ansistring;
procedure ReadBlobBuffer(Protocol: TFirebirdBaseDriver;
  Handle: PISC_DB_HANDLE; TransactionHandle: PISC_TR_HANDLE;
  BlobId: TISC_QUAD; var Size: Integer; var Buffer: Pointer);
procedure GetBlobInfo(Protocol: TFirebirdBaseDriver;
  BlobHandle: TISC_BLOB_HANDLE; var BlobInfo: TIbBlobInfo);

implementation


{**
   Get affected rows.
   <i>Note:<i> it function may call after statement execution
   @param PlainDriver a interbase plain driver
   @param StmtHandle a statement handle
   @param StatementType a statement type
   @return affected rows
}
function GetAffectedRows(Protocol: TFirebirdBaseDriver; StmtHandle: TISC_STMT_HANDLE;
            StatementType: TFirebirdStmtType): Integer;
var
  ReqInfo: Char;
  OutBuffer: array[0..255] of Char;
  StatusVector: TARRAY_ISC_STATUS;
begin
  Result := -1;
  ReqInfo := Char(isc_info_sql_records);

  if Protocol.isc_dsql_sql_info(@StatusVector, @StmtHandle, 1,
    @ReqInfo, SizeOf(OutBuffer), OutBuffer) > 0 then Exit;

  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);
  if OutBuffer[0] = Char(isc_info_sql_records) then
  begin
    case StatementType of
      stUpdate: Result := Protocol.isc_vax_integer(@OutBuffer[6], 4);
      stDelete: Result := Protocol.isc_vax_integer(@OutBuffer[13], 4);
      stInsert: Result := Protocol.isc_vax_integer(@OutBuffer[27], 4);
      else Result := -1;
    end;
  end;
end;


{**
   Return interbase statement type by statement handle
   @param PlainDriver a interbase plain driver
   @param StmtHandle a statement handle
   @return interbase statement type
}
function GetStatementType(Protocol: TFirebirdBaseDriver; StmtHandle: TISC_STMT_HANDLE): TFirebirdStmtType;
var
  TypeItem: Char;
  StatusVector: TARRAY_ISC_STATUS;
  StatementLength: integer;
  StatementBuffer: array [0..7] of Char;
begin
  Result := stUnknown;
  TypeItem := Char(isc_info_sql_stmt_type);

  { Get information about a prepared DSQL statement. }
  Protocol.isc_dsql_sql_info(@StatusVector, @StmtHandle, 1,  @TypeItem, SizeOf(StatementBuffer), StatementBuffer);
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);

  if StatementBuffer[0] = Char(isc_info_sql_stmt_type) then
  begin
    StatementLength := Protocol.isc_vax_integer( @StatementBuffer[1], 2);
    Result := TFirebirdStmtType(Protocol.isc_vax_integer( @StatementBuffer[3], StatementLength));
  end;
end;


{**
   Prepare statement and create statement handle.
   @param PlainDriver a interbase plain driver
   @param Handle a interbase connection handle
   @param TrHandle a transaction handle
   @param Dialect a interbase sql dialect number
   @param Sql a sql query
   @param StmtHandle a statement handle
   @param SqlData a interbase sql result data
   @return sql statement type
}
function PrepareStatement(Protocol: TFirebirdBaseDriver; Handle: PISC_DB_HANDLE; TrHandle:
    PISC_TR_HANDLE; Dialect: Word; SQL: ansistring; var StmtHandle: TISC_STMT_HANDLE):  TFirebirdStmtType;
var
  StatusVector: TARRAY_ISC_STATUS;
begin
  { Allocate an sql statement }
  Protocol.isc_dsql_alloc_statement2(@StatusVector, Handle, @StmtHandle);
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector, lcExecute, SQL);

  { Prepare an sql statement }
  Protocol.isc_dsql_prepare(@StatusVector, TrHandle, @StmtHandle,  0, PChar(SQL), Dialect, nil);
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector, lcExecute, SQL);

  { Set Statement Type }
  Result := GetStatementType(Protocol, StmtHandle);

  if Result in [stUnknown, stGetSegment, stPutSegment, stStartTrans] then
  begin
    PDOUtilsFirebird.FreeStatement(Protocol, StmtHandle);
    Protocol.log(lcPrepStmt, SFailedtoInitPrepStmt, 0, format (SStatementIsNotAllowed, [SQL]));
  end;
end;


{**
   Free allocated statement and SQLDA for input and output parameters
   @param  the interbase plain driver
   @param  the interbse statement handle
}
procedure FreeStatement(Protocol: TFirebirdBaseDriver; var StatementHandle: TISC_STMT_HANDLE);
var
  StatusVector: TARRAY_ISC_STATUS;
begin
  if StatementHandle <> nil then
  begin
    Protocol.isc_dsql_free_statement(@StatusVector, @StatementHandle, DSQL_drop);
    PDOUtilsFirebird.CheckFirebirdError (Protocol, StatusVector);
  end;
end;

{**
   Closes the cursor on an active statement
   @param  the interbase plain driver
   @param  the interbse statement handle
}
procedure closeCursor(Protocol: TFirebirdBaseDriver; var StatementHandle: TISC_STMT_HANDLE);
var
  StatusVector: TARRAY_ISC_STATUS;
begin
  if StatementHandle <> nil then
  begin
    Protocol.isc_dsql_free_statement(@StatusVector, @StatementHandle, DSQL_close);
    PDOUtilsFirebird.CheckFirebirdError (Protocol, StatusVector);
  end;
end;



{**
  Checks for possible sql errors.
  @param PlainDriver a MySQL plain driver.
  @param Handle a MySQL connection handle.
  @param LogCategory a logging category.
  @param LogMessage a logging message.
}
procedure CheckFirebirdError(Protocol: TFirebirdBaseDriver;  StatusVector: TARRAY_ISC_STATUS;
        LogCategory: TZLoggingCategory = lcOther; LogMessage: ansistring = '');
var
  Msg: array[0..1024] of Char;
  PStatusVector: PISC_STATUS;
  ErrorMessage, ErrorSqlMessage: ansistring;
  ErrorCode: LongInt;
  conjunction: ansistring;
begin
    ErrorMessage    :='';
    ErrorSqlMessage := '';
    ErrorCode       := 0;

    if (StatusVector[0] = 1) and (StatusVector[1] > 0) then
    begin
        PStatusVector := @StatusVector;
        while Protocol.isc_interprete(Msg, @PStatusVector) > 0 do
            ErrorMessage := ErrorMessage + ' ' + StrPas(Msg);

        ErrorCode := Protocol.isc_sqlcode(@StatusVector);
        Protocol.isc_sql_interprete(ErrorCode, Msg, 1024);

        ErrorSqlMessage := StrPas(Msg);

        if ErrorMessage <> '' then
        begin
            if LogMessage <> '' then conjunction := ': ' else conjunction := '';

            Protocol.log(LogCategory, ErrorMessage, ErrorCode, ErrorSqlMessage + conjunction + LogMessage);
        end;
    end;

    Protocol.last_error_msg     := ErrorMessage;
    Protocol.last_error_code    := ErrorCode;
    Protocol.last_error_msg_sql := ErrorSqlMessage;
end;



procedure CheckPrepStmtError(var errors: PSErrorRecord; Protocol: TFirebirdBaseDriver;  StatusVector: TARRAY_ISC_STATUS;
        LogCategory: TZLoggingCategory = lcOther; LogMessage: ansistring = '');
var
  Msg: array[0..1024] of Char;
  PStatusVector: PISC_STATUS;
  conjunction: ansistring;
begin
    errors.last_error_msg     :='';
    errors.last_error_msg_sql := '';
    errors.last_error_code    := 0;

    if (StatusVector[0] = 1) and (StatusVector[1] > 0) then
    begin
        PStatusVector := @StatusVector;
        while Protocol.isc_interprete(Msg, @PStatusVector) > 0 do
            errors.last_error_msg := errors.last_error_msg + ' ' + StrPas(Msg);

        errors.last_error_code := Protocol.isc_sqlcode(@StatusVector);
        Protocol.isc_sql_interprete(errors.last_error_code, Msg, 1024);

        errors.last_error_msg_sql := StrPas(Msg);

        if errors.last_error_msg  <> '' then
        begin
            if LogMessage <> '' then conjunction := ': ' else conjunction := '';

            Protocol.log(LogCategory, errors.last_error_msg , errors.last_error_code, errors.last_error_msg_sql + conjunction + LogMessage);
        end;
    end;
end;




{**
  Generate database connection string by connection information
  @param DPB - a database connection string
  @param Dialect - a sql dialect number
  @param Info - a list connection interbase parameters
  @return a generated string length
}
function GenerateDPB(Info: TStrings; var FDPBLength, Dialect: Word): PChar;
var
  I, Pos, PValue: Integer;
  ParamNo: Word;
  DPB, Buffer, ParamName, ParamValue: String;
begin
  FDPBLength := 1;
  DPB := Char(isc_dpb_version1);

  for I := 0 to Info.Count-1 do
  begin
    Buffer := Info.Strings[I];
    Pos := PDOSysUtils.FirstDelimiter(' ='#9#10#13, Buffer);
    ParamName := Copy(Buffer, 1, Pos-1);
    Delete(Buffer, 1, Pos);
    ParamValue := Buffer;
    ParamNo := GetFirebirdDatabaseParamNumber(ParamName);

    case ParamNo of
      0: Continue;
      isc_dpb_set_db_SQL_dialect:
        Dialect := StrToIntDef(ParamValue, 0);
      isc_dpb_user_name, isc_dpb_password, isc_dpb_password_enc,
      isc_dpb_sys_user_name, isc_dpb_license, isc_dpb_encrypt_key,
      isc_dpb_lc_messages, isc_dpb_lc_ctype, isc_dpb_sql_role_name:
        begin
          DPB := DPB + Char(ParamNo) + Char(Length(ParamValue)) + ParamValue;
          Inc(FDPBLength, 2 + Length(ParamValue));
        end;
      isc_dpb_num_buffers, isc_dpb_dbkey_scope, isc_dpb_force_write,
      isc_dpb_no_reserve, isc_dpb_damaged, isc_dpb_verify:
        begin
          DPB := DPB + Char(ParamNo) + #1 + Char(StrToInt(ParamValue));
          Inc(FDPBLength, 3);
        end;
      isc_dpb_sweep:
        begin
          DPB := DPB + Char(ParamNo) + #1 + Char(isc_dpb_records);
          Inc(FDPBLength, 3);
        end;
      isc_dpb_sweep_interval:
        begin
          PValue := StrToInt(ParamValue);
          DPB := DPB + Char(ParamNo) + #4 + PChar(@PValue)[0] + PChar(@PValue)[1] +
            PChar(@PValue)[2] + PChar(@PValue)[3];
          Inc(FDPBLength, 6);
        end;
      isc_dpb_activate_shadow, isc_dpb_delete_shadow, isc_dpb_begin_log,
      isc_dpb_quit_log:
        begin
          DPB := DPB + Char(ParamNo) + #1 + #0;
          Inc(FDPBLength, 3);
        end;
    end;
  end;

  Result := StrAlloc(FDPBLength + 1);
  StrPCopy(Result, DPB);
end;

{
**
   Generate transaction structuer by connection information
   @param Params - a transaction parameters list
   @param Dialect - a database connection handle
   @return a transaction ISC structure
}
function GenerateTPB(Params: TStrings; var Handle: TISC_DB_HANDLE): PISC_TEB;
var
  I: Integer;
  TPBLength,ParamNo: Word;
  TempStr, ParamValue: String;
  TPB: PChar;
  IsolationLevel: Boolean;
begin
  TPBLength := 0;
  IsolationLevel := False;
  TempStr := '';

  { Prepare transaction parameters string }
  for I := 0 to Params.Count - 1 do
  begin
    ParamValue := Params.Strings[I];
    ParamNo := PDOUtilsFirebird.GetFirebirdDatabaseParamNumber(ParamValue);

    case ParamNo of
      0: Continue;
      isc_tpb_lock_read, isc_tpb_lock_write:
        begin
          TempStr := TempStr + Char(ParamNo) + Char(Length(ParamValue)) + ParamValue;
          Inc(TPBLength, Length(ParamValue) + 2);
        end;
      else
        begin
          TempStr := TempStr + Char(ParamNo);
          Inc(TPBLength, 1);
        end;
    end;

    { Check what was set use transaction isolation level }
    if not IsolationLevel then
      case ParamNo of
        isc_tpb_concurrency, isc_tpb_consistency,
        isc_tpb_read_committed:
          IsolationLevel := True
        else
          IsolationLevel := False;
      end;
  end;

  { Allocate transaction parameters PChar buffer
    if temporally parameters string is empty the set null pointer for
    default database transaction}
  if (TPBLength > 0) and (IsolationLevel) then
  begin
    TPB := StrAlloc(TPBLength + 1);
    TPB := StrPCopy(TPB, TempStr);
  end else
    TPB := nil;

  { Allocate transaction structure }
  Result := AllocMem(Sizeof(TISC_TEB));
  with Result^ do
  begin
    db_handle := @Handle;
    tpb_length := TPBLength;
    tpb_address := TPB;
  end;
end;


{**
  Return interbase connection parameter number by it name
  @param Value - a connection parameter name
  @return - connection parameter number
}
function GetFirebirdDatabaseParamNumber(const Value: ansistring): Word;
var
 I: Integer;
 ParamName: string;
begin
  ParamName := LowerCase(Value);
  Result := 0;
  if System.Pos(BPBPrefix, ParamName) = 1 then
    for I := 1 to MAX_DPB_PARAMS do
    begin
      if ParamName = DatabaseParams[I].Name then
      begin
        Result := DatabaseParams[I].Number;
        Break;
      end;
    end;
end;


function XSQLDA_LENGTH(Value: word): LongInt;
begin
    Result := SizeOf(TXSQLDA);
    if value > 1 then
        Result := Result + ((Value - 1) * SizeOf(TXSQLVAR));
end;

{ Convert string buffer into pascal string }
function BufferToStr(Buffer: PChar; Length: LongInt): string;
begin
  Result := '';
  if Assigned(Buffer) then
    SetString(Result, Buffer, Length);
end;

{**
   Return Interbase SqlType by it number
   @param Value the SqlType number 
}
function GetNameSqlType(Value: TFirebirdFieldTypes): ansistring;
begin
  case Value of
    SQL_VARYING: Result := 'SQL_VARYING';
    SQL_TEXT: Result := 'SQL_TEXT';
    SQL_DOUBLE: Result := 'SQL_DOUBLE';
    SQL_FLOAT: Result := 'SQL_FLOAT';
    SQL_LONG: Result := 'SQL_LONG';
    SQL_SHORT: Result := 'SQL_SHORT';
    SQL_TIMESTAMP: Result := 'SQL_TIMESTAMP';
    SQL_BLOB: Result := 'SQL_BLOB';
    SQL_D_FLOAT: Result := 'SQL_D_FLOAT';
    SQL_ARRAY: Result := 'SQL_ARRAY';
    SQL_QUAD: Result := 'SQL_QUAD';
    SQL_TYPE_TIME: Result := 'SQL_TYPE_TIME';
    SQL_TYPE_DATE: Result := 'SQL_TYPE_DATE';
    SQL_INT64: Result := 'SQL_INT64';
    SQL_BOOLEAN: Result := 'SQL_BOOLEAN';
  else
    Result := 'Unknown';
  end
end;



{**
   Read blob field data to stream by it ISC_QUAD value
   Note: DefaultBlobSegmentSize constant used for limit segment size reading
   @param Handle the database connection handle
   @param TransactionHandle the transaction handle
   @param BlobId the ISC_QUAD structure
   @param Size the result buffer size
   @param Buffer the pointer to result buffer

   Note: Buffer must be nill. Function self allocate memory for data
    and return it size
}
procedure ReadBlobBuffer(Protocol: TFirebirdBaseDriver;
  Handle: PISC_DB_HANDLE; TransactionHandle: PISC_TR_HANDLE;
  BlobId: TISC_QUAD; var Size: Integer; var Buffer: Pointer);
var
  TempBuffer: PChar;
  BlobInfo: TIbBlobInfo;
  BlobSize, CurPos: LongInt;
  BytesRead, SegmentLenght: UShort;
  BlobHandle: TISC_BLOB_HANDLE;
  StatusVector: TARRAY_ISC_STATUS;
begin
  BlobHandle := nil;
  CurPos := 0;
  SegmentLenght := UShort(DefaultBlobSegmentSize);

  { open blob }
  Protocol.isc_open_blob2(@StatusVector, Handle,
         TransactionHandle, @BlobHandle, @BlobId, 0 , nil);
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);

  { get blob info }
  GetBlobInfo(Protocol, BlobHandle, BlobInfo);
  BlobSize := BlobInfo.TotalSize;
  Size := BlobSize;

  { Allocates a blob buffer }
  Buffer := AllocMem(BlobSize);
  TempBuffer := Buffer;

  { Copies data to blob buffer }
  while CurPos < BlobSize do
  begin
    if (CurPos + SegmentLenght > BlobSize) then
      SegmentLenght := BlobSize - CurPos;
    if not(Protocol.isc_get_segment(@StatusVector, @BlobHandle,
           @BytesRead, SegmentLenght, TempBuffer) = 0) or
          (StatusVector[1] <> isc_segment) then
        PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);
    Inc(CurPos, BytesRead);
    Inc(TempBuffer, BytesRead);
    BytesRead := 0;
  end;

  { close blob handle }
  Protocol.isc_close_blob(@StatusVector, @BlobHandle);
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);
end;



{**
   Read blob information by it handle such as blob segment size, segments count,
   blob size and type.
   @param PlainDriver
   @param BlobInfo the blob information structure
}
procedure GetBlobInfo(Protocol: TFirebirdBaseDriver;
  BlobHandle: TISC_BLOB_HANDLE; var BlobInfo: TIbBlobInfo);
var
  Items: array[0..3] of Char;
  Results: array[0..99] of Char;
  I, ItemLength: Integer;
  Item: Integer;
  StatusVector: TARRAY_ISC_STATUS;
begin
  I := 0;
  Items[0] := Char(isc_info_blob_num_segments);
  Items[1] := Char(isc_info_blob_max_segment);
  Items[2] := Char(isc_info_blob_total_length);
  Items[3] := Char(isc_info_blob_type);

  if integer(Protocol.isc_blob_info(@StatusVector, @BlobHandle, 4, @items[0],
    SizeOf(Results), @Results[0])) > 0 then
  PDOUtilsFirebird.CheckFirebirdError(Protocol, StatusVector);

  while (I < SizeOf(Results)) and (Results[I] <> Char(isc_info_end)) do
  begin
    Item := Integer(Results[I]); Inc(I);
    ItemLength := Protocol.isc_vax_integer(@results[I], 2); Inc(I, 2);
    case Item of
      isc_info_blob_num_segments:
        BlobInfo.NumSegments := Protocol.isc_vax_integer(@Results[I], ItemLength);
      isc_info_blob_max_segment:
        BlobInfo.MaxSegmentSize := Protocol.isc_vax_integer(@Results[I], ItemLength);
      isc_info_blob_total_length:
        BlobInfo.TotalSize := Protocol.isc_vax_integer(@Results[I], ItemLength);
      isc_info_blob_type:
        BlobInfo.BlobType := Protocol.isc_vax_integer(@Results[I], ItemLength);
    end;
    Inc(i, ItemLength);
  end;
end;

end.
