{ $Id: PDOMySQLDriver.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOMySQLDriver;

interface

{$I ../../directives.inc}

uses
	SysUtils, classes, PDOClasses, PDOInterfaces, PDOConstantsMySQL, PDOLogging;

type
  TZMySQLBaseDriver = class (TInterfacedObject, IPDOProtocol)
  private
    FSilentMode: Byte;            {0 = silent, 1 = warning, 2 = throw exception}
    FCaseMode: Byte;              {0 = lower, 2 = natural, 3 = upper}
    FOracleNullMode: Byte;        {0 = natural, 1 = null}
    FBufferQueryMode: Boolean;    {false, true}
    FCompressionMode: Boolean;    {false, true}
    FMaximumBLOBSize: LongWord;   {Blob size >= 65535 and < 4294967296}
    FMultiQueryMode: Boolean;     {false, true}

    Listener: TZAbstractLoggingListener;
  public
    constructor create;

    function GetSilentMode: Byte;
    function GetCaseMode: Byte;
    function GetOracleNull: Byte;
    function GetBufferQuery: Boolean;
    function GetMaximumBLOBSize: Int64;
    function GetSQLDialect: TSQLDialect;
    function GetCompression: Boolean;
    function GetMultipleQuery: Boolean;

    procedure SetSilentMode (mode: Byte);
    procedure SetCaseMode (mode: Byte);
    procedure SetOracleNull (mode: Byte);
    procedure setBufferQuery (mode: Boolean);
    procedure setCompression (mode: Boolean);
    procedure setMaximumBLOBSize (BLOBSize: Int64); virtual;
    procedure setMultipleQuery (mode: Boolean);

    function GetProtocol: AnsiString; virtual; abstract;
    function GetDescription: AnsiString; virtual; abstract;

    function GetClientVersion: AnsiString; virtual; abstract;
    function GetClientInfo: AnsiString;  virtual; abstract;
    procedure initialize; virtual; abstract;
    procedure preconnect(Info: TStringList; listenerHandle: TZAbstractLoggingListener);
    procedure log (const Category: TZLoggingCategory; const msg: ansistring;
            const error_code: integer = 0; const error_msg: ansistring = '');

    {MOVED FROM IPDOPROTOCOL TO BASE DRIVERS}
    function getSQLState         (connection: PZMySQLConnect): AnsiString; virtual; abstract;
    function getLastErrorCode    (connection: PZMySQLConnect): Integer;    virtual; abstract;
    function getLastErrorMessage (connection: PZMySQLConnect): AnsiString; virtual; abstract;
    function GetServerVersion    (connection: PZMySQLConnect): AnsiString; virtual; abstract;
    function GetServerInfo       (connection: PZMySQLConnect): AnsiString; virtual; abstract;
    function getLastInsertID     (connection: PZMySQLConnect): Int64;      virtual; abstract;

    function GetLastPreparedError(Handle: PZMySqlPrepStmt): AnsiString; virtual; abstract;
    function GetLastPreparedErrorCode(Handle: PZMySqlPrepStmt): Integer; virtual; abstract;

    {MYSQL NATIVE PROTOCOL}
    function SetAutocommit (Handle: PZMySQLConnect; mode: Boolean): Boolean; virtual; abstract;
    function Commit (Handle: PZMySQLConnect): Boolean;    virtual; abstract;
    function Rollback (Handle: PZMySQLConnect): Boolean;  virtual; abstract;
    function InitializePrepStmt (Handle: PZMySQLConnect): PZMySqlPrepStmt;    virtual; abstract;
    function ClosePrepStmt (PrepStmtHandle: PZMySqlPrepStmt): PZMySqlPrepStmt;    virtual; abstract;
    function PrepareStmt (PrepStmtHandle: PZMySqlPrepStmt; const Query: PChar; Length: Integer): Integer;  virtual; abstract;
    function GetNumRows(Res: PZMySQLResult): Int64; virtual; abstract;

    function GetPreparedSQLState (Handle: PZMySqlPrepStmt): PChar; virtual; abstract;
    function GetPreparedNumRows (Handle: PZMySqlPrepStmt): Int64; virtual; abstract;
    function GetPreparedAffectedRows (Handle: PZMySqlPrepStmt): Int64; virtual; abstract;
    function GetPreparedInsertID  (Handle: PZMySqlPrepStmt): Int64; virtual; abstract;
    function StorePreparedResult  (Handle: PZMySqlPrepStmt): Integer; virtual; abstract;
    function GetPreparedMetaData  (Handle: PZMySqlPrepStmt): PZMySQLResult; virtual; abstract;
    function GetPreparedFieldCount(Handle: PZMySqlPrepStmt): Integer; virtual; abstract;
    function CheckAnotherRowset   (Handle: PZMySQLConnect): Boolean; virtual; abstract;
    function RetrieveNextRowset   (Handle: PZMySQLConnect): Integer; virtual; abstract;
    function BindParameters(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte; virtual; abstract;
    function ExecuteStmt(Handle: PZMySqlPrepStmt): Integer; virtual; abstract;
    function GetPreparedBindMarkers (Handle: PZMySqlPrepStmt): Cardinal; virtual; abstract;
    function FetchBoundResults (Handle: PZMySqlPrepStmt): Integer; virtual; abstract;
    function BindResult(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte; virtual; abstract;
    function SetServerOption (Handle: PZMySQLConnect; option: TMysqlSetOption): Integer; virtual; abstract;

    {original Zeos functions}
    function StoreResult(Handle: PZMySQLConnect): PZMySQLResult; virtual; abstract;
    function UseResult(Handle: PZMySQLConnect): PZMySQLResult; virtual; abstract;
    procedure FreeResult(Res: PZMySQLResult); virtual; abstract;
    function GetAffectedRows(Handle: PZMySQLConnect): Int64; virtual; abstract;

    function FetchRow(Res: PZMySQLResult): PZMySQLRow; virtual; abstract;
    function FetchLengths(Res: PZMySQLResult): PLongInt; virtual; abstract;
    function FetchField(Res: PZMySQLResult): PZMySQLField; virtual; abstract;

    procedure SeekData(Res: PZMySQLResult; Offset: Cardinal); virtual; abstract;
    function SeekRow(Res: PZMySQLResult; Row: PZMySQLRowOffset): PZMySQLRowOffset; virtual; abstract;
    function SeekField(Res: PZMySQLResult; Offset: Cardinal): Cardinal; virtual; abstract;

    function GetFieldType(Field: PZMySQLField): TMysqlFieldTypes; virtual; abstract;
    function GetFieldFlags(Field: PZMySQLField): Integer; virtual; abstract;
    function GetStatus(Handle: PZMySQLConnect): TMySQLStatus; virtual; abstract;
    function GetRowCount(Res: PZMySQLResult): Int64; virtual; abstract;
    function GetFieldCount(Res: PZMySQLResult): Integer; virtual; abstract;
    function GetFieldName(Field: PZMySQLField): PChar; virtual; abstract;
    function GetFieldTable(Field: PZMySQLField): PChar;  virtual; abstract;
    function GetFieldLength(Field: PZMySQLField): Integer; virtual; abstract;
    function GetFieldMaxLength(Field: PZMySQLField): Integer; virtual; abstract;
    function GetFieldDecimals(Field: PZMySQLField): Integer; virtual; abstract;
    function GetFieldData(Row: PZMySQLRow; Offset: Cardinal): PChar; virtual; abstract;

    function Connect(Handle: PZMySQLConnect; const Host, User, Password: PChar):
            PZMySQLConnect; virtual; abstract;
    function RealConnect(Handle: PZMySQLConnect;
            const Host, User, Password, Db: PChar; Port: Cardinal;
            UnixSocket: PChar; ClientFlag: Cardinal): PZMySQLConnect; virtual; abstract;
    procedure Close(Handle: PZMySQLConnect); virtual; abstract;

    function ExecQuery(Handle: PZMySQLConnect; const Query: PChar): Integer; virtual; abstract;
    function ExecRealQuery(Handle: PZMySQLConnect; const Query: PChar; Length: Integer): Integer; virtual; abstract;
    function Init(var Handle: PZMySQLConnect): PZMySQLConnect;  virtual; abstract;
    procedure Dispose(var Handle: PZMySQLConnect); virtual; abstract;

    function SetOptions(Handle: PZMySQLConnect; Option: TMySQLOption;
            const Arg: PChar): Integer;  virtual; abstract;

  end;


implementation

{BASE Driver Object}

constructor TZMySQLBaseDriver.Create;
begin
    self.FSilentMode      := ERRORLEVEL_WARNING;
    self.FCaseMode        := CASEMODE_NATURAL;
    self.FOracleNullMode  := EMPTY_STRING_NATURAL;
    self.FBufferQueryMode := QUERY_FREFETCH_ON;
    self.FCompressionMode := OPT_COMPRESS_ON;
    self.FMultiQueryMode  := OPT_MULTIQUERY_OFF;
    self.FMaximumBLOBSize := 255;   {all strings initially truncated to 255 characters}
end;

function TZMySQLBaseDriver.GetSilentMode: Byte;
begin
    Result := self.FSilentMode;
end;

function TZMySQLBaseDriver.GetCaseMode: Byte;
begin
    Result := self.FCaseMode;
end;

function TZMySQLBaseDriver.GetOracleNull: Byte;
begin
    Result := self.FOracleNullMode;
end;

function TZMySQLBaseDriver.GetBufferQuery: Boolean;
begin
    Result := self.FBufferQueryMode;
end;

procedure TZMySQLBaseDriver.SetSilentMode(mode: Byte);
begin
    self.FSilentMode := mode;
end;

procedure TZMySQLBaseDriver.SetCaseMode (mode: Byte);
begin
    self.FCaseMode := mode;
end;

procedure TZMySQLBaseDriver.SetOracleNull(mode: Byte);
begin
    self.FOracleNullMode := mode;
end;

procedure TZMySQLBaseDriver.SetBufferQuery(mode: Boolean);
begin
    self.FBufferQueryMode := mode;
end;

function TZMySQLBaseDriver.GetMaximumBLOBSize: Int64;
Begin
    Result := self.FMaximumBLOBSize;
End;

function TZMySQLBaseDriver.GetCompression: Boolean;
Begin
    Result := self.FCompressionMode;
End;

procedure TZMySQLBaseDriver.setCompression(mode: Boolean);
Begin
    self.FCompressionMode := mode;
End;

function TZMySQLBaseDriver.GetMultipleQuery: Boolean;
begin
    Result := self.FMultiQueryMode;
end;

procedure TZMySQLBaseDriver.setMultipleQuery(mode: Boolean);
begin
    self.FMultiQueryMode := mode;
end;

procedure TZMySQLBaseDriver.setMaximumBLOBSize (BLOBSize: Int64);
Begin
    { For mysql, the maximum size for blobs is needed, I think for the pre-fetch
      mechanism of the library.  I believe that if mysql fetches the rows one by one, then
      it will size the mysql_res structure accordingly.  However, this is not the
      default so maxblobsize is necessary }

    If (BlOBSize < 255) then
        self.FMaximumBLOBSize := 255
    else
        if (BLOBSize > 4294967295) then
            self.FMaximumBLOBSize := 4294967295
        else
            self.FMaximumBLOBSize := BLOBSize;
end;

function TZMySQLBaseDriver.GetSQLDialect: TSQLDialect;
begin
    Result := SQL_MYSQL;
end;

procedure TZMySQLBaseDriver.preconnect (Info: TStringList; listenerHandle: TZAbstractLoggingListener);
var
    optCompress, optMultiQuery: Boolean;
    optflags: LongInt;
begin
    if Info.IndexOfName('host') < 0         then Info.Add('host=');
    if Info.IndexOfName('dbname') < 0       then Info.Add('dbname=');
    if Info.IndexOfName('port') < 0         then info.Add('port=' + inttostr(MYSQL_PORT));
    if Info.IndexOfName('unix_socket') < 0  then Info.Add('unix_socket=');

    optflags := strtointdef(Info.Values['options'],0);
    optCompress := false;
    optMultiQuery := false;
    if optflags AND 1 > 0 then optCompress := true;
    if optflags AND 2 > 0 then optMultiQuery := true;

    self.setCompression(optCompress);
    self.setMultipleQuery(optMultiQuery);
    self.initialize;
    self.Listener := listenerHandle;
end;

procedure TZMySQLBaseDriver.log (const Category: TZLoggingCategory; const msg: ansistring;
    const error_code: integer = 0; const error_msg: ansistring = '');
var
    silent_mode: byte;
    Event: TZLoggingEvent;
    protocol_description: ansistring;
begin
    if not assigned(self.Listener) then exit;

    silent_mode := self.GetSilentMode;
    protocol_description := self.GetDescription;
    
    if silent_mode = ERRORLEVEL_SILENT then exit;


    Event := TZLoggingEvent.Create(Category, protocol_description, msg, error_code, error_msg);
    try
        self.Listener.LogEvent(Event);
    finally
        Event.Destroy;
    end;

    if (silent_mode = ERRORLEVEL_RAISE_EXCEPTION) AND ((error_code <> 0) or (error_msg <> '')) then
    begin
        if error_code <> 0 then
            raise EZSQLException.CreateWithCode(error_code, error_msg)
        else
            raise EZSQLException.Create(error_msg);
    end;
end;


end.

