{ $Id: PDOResultMySQL.pas 23 2007-01-19 22:06:28Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}

unit PDOResultMySQL;

interface

{$I ../../directives.inc}

uses PDOStatement, Classes, PDOCompatibility, SysUtils, PDOUtilsMySQL,
     PDOSysUtils, PDOInterfaces, PDOResult;

type
    TMysqlResult = class (TResult)
        public
            function asDate:          TDateTime; override;
            function asTime:          TDateTime; override;
            function asTimestamp:     TDateTime; override;
        end;

implementation

function TMysqlResult.asTimestamp: TDateTime;
var
    myhour, mymin, mysec, myyear, mymonth, myday: Integer;
    rsLen: byte;
Begin
    // expecting YYYY-MM-DD HH:MM:SS
    //        or YYYYMMDDHHMMSS
    //        or YYYY-MM-DD

    rsLen := length(self.rawString);
    if rsLen = 19 then
      begin
        myhour := strToInt(copy(self.rawString,12,2));
        mymin  := strToInt(copy(self.rawString,15,2));
        mysec  := strToInt(copy(self.rawString,18,2));
        myyear  := strToInt(copy(self.rawString,1,4));
        mymonth := strToInt(copy(self.rawString,6,2));
        myday   := strToInt(copy(self.rawString,9,2));
        try
            Result := encodeTime(myhour, mymin, mysec,0) + encodeDate(myyear, mymonth, myday);
        except
            Result := 0;
        end;
      end
    else if rsLen = 10 then
        Result := self.asDate
    else if rsLen = 14 then
        Result := MySQLTimestampToDateTime(self.rawString)
    else
        Result := encodeDate(1970, 1, 1) + encodeTime(0,0,0,0);
End;

function TMysqlResult.asTime: TDateTime;
var
    myhour, mymin, mysec: Integer;
    rsLen: byte;
Begin
    // expecting YYYY-MM-DD HH:MM:SS
    //        or YYYYMMDDHHMMSS
    //        or YYYY-MM-DD (defaults to 00:00:00)

    rsLen := length(self.rawString);
    if rsLen = 19 then
      begin
        myhour := strToInt(copy(self.rawString,12,2));
        mymin  := strToInt(copy(self.rawString,15,2));
        mysec  := strToInt(copy(self.rawString,18,2));
        try
            Result := encodeTime(myhour, mymin, mysec,0);
        except
            Result := 0;
        end;
      end
    else if rsLen = 14 then
        Result := frac(MySQLTimestampToDateTime(self.rawString))
    else
        Result :=  encodeTime(0,0,0,0);
End;

function TMysqlResult.asDate: TDateTime;
var
    myyear, mymonth, myday: Integer;
    rsLen: byte;
Begin
    // expecting YYYY-MM-DD HH:MM:SS
    //        or YYYYMMDDHHMMSS
    //        or YYYY-MM-DD

    rsLen := length(self.rawString);
    if (rsLen = 19) or (rsLen = 10) then
      begin
        myyear  := strToInt(copy(self.rawString,1,4));
        mymonth := strToInt(copy(self.rawString,6,2));
        myday   := strToInt(copy(self.rawString,9,2));
        if not sysUtils.TryEncodeDate(myYear, myMonth, myDay, Result) then
            Result := encodeDate(1900, 1, 1);
      end
    else if rsLen = 14 then
        Result := trunc(MySQLTimestampToDateTime(self.rawString))
    else
        Result := encodeDate(1970, 1, 1);
End;


end.
