{ $Id: PDOStatementMySQL.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}


unit PDOStatementMySQL;

interface

{$I ../../directives.inc}

uses PDOStatement, PDOClasses, Types, Classes, Variants,
     SysUtils, PDOMySQLDriver, PDOUtilsMySQL, PDOLogging, PDOSysUtils,
     PDOMessages, PDOConstantsMySQL, PDOInterfaces, PDOResultMySQL;

type
    ColumnRecord = record
        Name:       AnsiString;
        Datatype:   TZSQLType;
        Datatype2:  TMysqlFieldTypes;
        DataLength: LongWord;
        Precision:  Integer;
        Scale:      Integer;
        Flags:      Word;
    end;

    PDOBindRecord2 = record
        buffer:    array of Byte; {PChar;}
        length:    LongWord;
        is_null:   Byte;
    end;

    TPDOMySqlStatement = class(TInterfacedObject, IPDOStatement)
    private
        TypeOfStatement:       TPDOStatementType;
        DirectSQL:             AnsiString;
        PreparedSQL:           AnsiString;
        FQueryHandle:          PZMySQLResult;
        FPrepStmtHandle:       PZMySqlPrepStmt;
        FProtocol:             TZMySQLBaseDriver;
        FMySQL:                PZMySQLConnect;
        FRowHandle:            PZMySQLRow;
        numberColumns:         Byte;
        ColumnsInfo:           array of ColumnRecord;
        alpha_bind_values:     TStringList;
        alpha_bind_columns:    TStringList;
        bind_markers:          Cardinal;
        bind_structures:       array of MYSQL_BIND;
        column_structures:     array of MYSQL_BIND;
        bind_locations:        array of PDOBindRecord2;
        column_locations:      array of PDOBindRecord2;
        column_addresses:      array of Pointer;
        columns_bound:         array of Boolean;
        params_bound:          array of Boolean;
        Successful_stmt_store: Boolean;
        Successful_stmt_prep:  Boolean;
	    Successful_query:      Boolean;
        rebind_required:       Boolean;

        procedure clearBuffer;
        procedure prefetchIfNecessary;
        procedure populateColumnInfo;
        procedure TransferVariables;
        procedure transferBytes (bufferStart: PChar; nativeString: AnsiString);
        procedure coreColumnPopulation (ResultHandle: PZMySQLResult);
        procedure genericBind (columnIndex: Cardinal; variableAddress: Pointer);
        procedure genericOverride (columnIndex: Cardinal; TypeOverride: TPDODataTypes);
        procedure UpdateBindStructure (index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes); virtual;
        procedure UpdateColumnStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes); virtual;
        procedure SetStructureSize; virtual;
        procedure SetStructureSizeColumns; virtual;
        function PointToBindStructures: Pointer; virtual;
        function PointToColumnStructures: Pointer; virtual;
        procedure ExtractColMetaData (index: Byte; var signed: Boolean; var field_type:TMysqlFieldTypes); virtual;
        function getFieldType (testVariant: Variant): TMysqlFieldTypes; overload;
        function getFieldType (PDODataType: TPDODataTypes): TMysqlFieldTypes; overload;
        function checkRebindingNeeded: Boolean;
        function getSignature (testVariant: Variant): Byte; overload;
        function getSignature (PDODataType: TPDODataTypes): Byte; overload;
        function convertGeneric (raw: TPDOGenericTime): MYSQL_TIME;
        function convertMySQL_TIME (raw: MYSQL_TIME):   TPDOGenericTime;
        function oracle(raw: PChar; rawlength: PLongInt; nullmode: Byte; var null: Boolean): AnsiString;
        function getFieldSize (field_type: TMysqlFieldTypes; field_size: LongWord): LongWord;

    public
        procedure setSQL  (SQL: AnsiString; StatementType: TPDOStatementType);

        procedure bindValue  (valueIndex: Cardinal;   PassedValue: Variant); overload;
        procedure bindValue  (valueIndex: AnsiString; PassedValue: Variant); overload;
        procedure bindTimeValue  (valueIndex: Cardinal;   PassedValue: TPDOGenericTime); overload;
        procedure bindTimeValue  (valueIndex: AnsiString; PassedValue: TPDOGenericTime); overload;

        procedure bindColumn (columnIndex: Cardinal;   variableAddress: Pointer); overload;
        procedure bindColumn (columnIndex: Cardinal;   variableAddress: Pointer; TypeOverride: TPDODataTypes); overload;
        procedure bindColumn (columnIndex: AnsiString; variableAddress: Pointer); overload;
        procedure bindColumn (columnIndex: AnsiString; variableAddress: Pointer; TypeOverride: TPDODataTypes); overload;

        function closeCursor: boolean;
        function columnCount: Integer;
        function errorCode: AnsiString;
        function errorInfo: PDO_errorInfoRecord;

        function execute: Boolean; overload;
        function execute (DelimitedBind: AnsiString): Boolean; overload;

        function getRowsAffected: Int64;
        function getNumberRows: Int64;
        function LastInsertId: Int64;
        function nextRowSet: Boolean;

        function fetch_bound: Boolean;
        function fetch (var row: TPDORowKeyStrings): Boolean; overload;
        function fetch (var row: TPDORowNative): Boolean; overload;
        function fetch_all (var rowset: TPDORowSetKeyStrings): Boolean; overload;
        function fetch_all (var rowset: TPDORowSetNative): Boolean; overload;

        constructor create (DriverHandle: TZMySQLBaseDriver; DBaseHandle: PZMySQLConnect);
        procedure beforeDestruction; Override;
    end;

    TPDOMySqlStatement50 = class(TPDOMySqlStatement, IPDOStatement)
    private
        bind_structures:       array of MYSQL_BIND2;
        column_structures:     array of MYSQL_BIND2;

        procedure UpdateBindStructure (index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes); override;
        procedure UpdateColumnStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes); override;
        procedure SetStructureSize; override;
        procedure SetStructureSizeColumns; override;
        function PointToBindStructures: Pointer; override;
        function PointToColumnStructures: Pointer; override;
        procedure ExtractColMetaData (index: Byte; var signed: Boolean; var field_type:TMysqlFieldTypes); override;
    end;


implementation

function TPDOMySqlStatement.getFieldSize (field_type: TMysqlFieldTypes; field_size: LongWord): LongWord;
var
    MaxBlobSize: LongWord;
    SmallBLOB: Word;
Begin
    MaxBlobSize := self.FProtocol.GetMaximumBLOBSize;
    if (MaxBlobSize > 65535) then
        SmallBLOB := 65535
    else
        SmallBLOB := MaxBlobSize;

    case field_type of
        MYSQL_TYPE_TINY:        Result := 1;
        MYSQL_TYPE_SHORT:       Result := 2;
        MYSQL_TYPE_LONG:        Result := 3;
        MYSQL_TYPE_LONGLONG:    Result := 8;
        MYSQL_TYPE_FLOAT:       Result := 4;
        MYSQL_TYPE_DOUBLE:      Result := 8;
        MYSQL_TYPE_TIMESTAMP:   Result := sizeOf(MYSQL_TIME);
        MYSQL_TYPE_DATE:        Result := sizeOf(MYSQL_TIME);
        MYSQL_TYPE_TIME:        Result := sizeOf(MYSQL_TIME);
        MYSQL_TYPE_DATETIME:    Result := sizeOf(MYSQL_TIME);
        MYSQL_TYPE_TINY_BLOB:   Result := 255;
        MYSQL_TYPE_MEDIUM_BLOB: Result := SmallBLOB;
        MYSQL_TYPE_LONG_BLOB:   Result := MaxBlobSize;
        MYSQL_TYPE_BLOB:        Result := MaxBlobSize;
        MYSQL_TYPE_STRING:      Result := MaxBlobSize;
        MYSQL_TYPE_VAR_STRING:  Result := field_size; {mysql 5.0.3 moved limit to 2^16}
    else
        Result := 255;  {unknown ??}
    end;
End;

procedure TPDOMySqlStatement.genericBind (columnIndex: Cardinal; variableAddress: Pointer);
var
    bufferLength: LongWord;
    signature: Byte;
    standard_msg: ansistring;
    error_msg: ansistring;
Begin
    if (columnIndex >= self.numberColumns) then
        begin
            standard_msg := format(SBindColumnOutOfRange,[columnIndex]);
            error_msg    := ' max < ' + IntToStr(self.numberColumns);

            self.FProtocol.log(lcPrepStmt, standard_msg, 0, error_msg);
            exit;
        end;
    bufferLength := Length(self.column_locations[columnIndex].buffer);
    signature := self.ColumnsInfo[columnIndex].Flags and UNSIGNED_FLAG;
    self.UpdateColumnStructure(columnIndex,signature,bufferLength,self.ColumnsInfo[columnIndex].Datatype2);
    self.column_addresses[columnIndex] := variableAddress;
    self.columns_bound[columnIndex] := true;
    self.rebind_required := true;
End;

procedure TPDOMySqlStatement.genericOverride (columnIndex: Cardinal; TypeOverride: TPDODataTypes);
var
    new_field_type: TMysqlFieldTypes;
    new_length: LongWord;
    signature: Byte;
Begin
    if (columnIndex >= self.numberColumns) then exit;  {already alerted this problem a line before}
    new_field_type := self.getFieldType(TypeOverride);
    new_length := self.getFieldSize(new_field_type, 255);
    SetLength(self.column_locations[columnIndex].buffer, new_length);
    signature := self.getSignature(TypeOverride);
    self.UpdateColumnStructure(columnIndex,signature,new_length,new_field_type);
End;

procedure TPDOMySqlStatement.bindColumn (columnIndex: Cardinal;   variableAddress: Pointer; TypeOverride: TPDODataTypes);
Begin
    self.genericBind(columnIndex, variableAddress);
    self.genericOverride(columnIndex, TypeOverride);
End;

procedure TPDOMySqlStatement.bindColumn (columnIndex: Cardinal;   variableAddress: Pointer);
Begin
    self.genericBind(columnIndex, variableAddress);
End;


procedure TPDOMySqlStatement.bindColumn (columnIndex: AnsiString; variableAddress: Pointer);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_columns.IndexOfName(columnIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [columnIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_columns.ValueFromIndex[localIndex]);
    self.genericBind(trueIndex, variableAddress);
end;

procedure TPDOMySqlStatement.bindColumn (columnIndex: AnsiString; variableAddress: Pointer; TypeOverride: TPDODataTypes);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_columns.IndexOfName(columnIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [columnIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_columns.ValueFromIndex[localIndex]);
    self.genericBind(trueIndex, variableAddress);
    self.genericOverride(trueIndex, TypeOverride);
end;


function TPDOMySqlStatement.convertMySQL_TIME (raw: MYSQL_TIME):   TPDOGenericTime;
Begin
    with raw do begin
        Result.year        := year;
        Result.month       := month;
        Result.day         := day;
        Result.hour        := hour;
        Result.minute      := minute;
        Result.second      := second;
        Result.neg         := neg;
        Result.second_part := second_part;
    End;
End;

function TPDOMySqlStatement.convertGeneric (raw: TPDOGenericTime):   MYSQL_TIME;
Begin
    with raw do begin
        Result.year        := year;
        Result.month       := month;
        Result.day         := day;
        Result.hour        := hour;
        Result.minute      := minute;
        Result.second      := second;
        Result.neg         := neg;
        Result.second_part := second_part;
    End;
End;

function TPDOMySqlStatement.getSignature(testVariant: variant): Byte;
var
    basicType: Integer;
Begin
    basicType := VarType(testVariant) and VarTypeMask;
    if (basicType in [varByte,varWord,varLongWord,varBoolean]) then
        Result := 1
    else
        Result := 0;
End;

function TPDOMySqlStatement.getSignature(PDODataType: TPDODataTypes): Byte;
Begin
    if (PDODataType in [integer_8,integer_16,integer_32]) then
        Result := 1
    else
        Result := 0;
End;


function TPDOMySqlStatement.getFieldType (PDODataType: TPDODataTypes): TMysqlFieldTypes;
Begin
    case PDODataType of
        integer_8:          Result := MYSQL_TYPE_TINY;
        integer_8_signed:   Result := MYSQL_TYPE_TINY;
        integer_16:         Result := MYSQL_TYPE_SHORT;
        integer_16_signed:  Result := MYSQL_TYPE_SHORT;
        integer_32:         Result := MYSQL_TYPE_LONG;
        integer_32_signed:  Result := MYSQL_TYPE_LONG;
        integer_64_signed:  Result := MYSQL_TYPE_LONGLONG;
        float_32:           Result := MYSQL_TYPE_FLOAT;
        float_64:           Result := MYSQL_TYPE_DOUBLE;
        string_1:           Result := MYSQL_TYPE_VAR_STRING;
        string_unlimited:   Result := MYSQL_TYPE_STRING;
        generic_time:       Result := MYSQL_TYPE_DATETIME;
        byte_buffer:        Result := MYSQL_TYPE_BLOB;
    else
        Result := MYSQL_TYPE_TINY;
    end;
End;

function TPDOMySqlStatement.getFieldType (testVariant: Variant): TMysqlFieldTypes;
var
    basicType  : Integer;
Begin
    basicType := VarType(testVariant) and VarTypeMask;
    case basicType of
        varBoolean:   Result := MYSQL_TYPE_TINY;
        varByte:      Result := MYSQL_TYPE_TINY;
        varShortInt:  Result := MYSQL_TYPE_TINY;
        varWord:      Result := MYSQL_TYPE_SHORT;
        varSmallint:  Result := MYSQL_TYPE_SHORT;
        varLongWord:  Result := MYSQL_TYPE_LONG;
        varInteger:   Result := MYSQL_TYPE_LONG;
        varInt64:     Result := MYSQL_TYPE_LONGLONG;
        varSingle:    Result := MYSQL_TYPE_FLOAT;
        varDouble:    Result := MYSQL_TYPE_DOUBLE;
        varString:    Result := MYSQL_TYPE_STRING;
     else
       // Result := MYSQL_TYPE_TINY;
       // varDate:      Result := MYSQL_TYPE_TIMESTAMP;
        raise EZSQLException.Create(SUnsupportedDataType);
     end;
End;


function TPDOMySqlStatement.oracle (raw: PChar; rawlength: PLongInt; nullmode: Byte; var null: Boolean): AnsiString;
Begin
    {Here all databases agree that a null is null.  If raw = nil, we are null}

    Result := '';
    null := true;
    if not assigned(raw) then exit;


    {here all databases agree that a filled string is not null}

    if rawlength^ > 0 then
    begin
        SetString(Result,raw,rawlength^);
        null := false;
        exit;
    end;


    {Finally, oracle stores and retrieves empty strings as so to emulate this,
     set empty strings to null if nullmode = EMPTY_STRING_NULL }

    if nullmode = EMPTY_STRING_NATURAL then
        null := false;
End;

function TPDOMySqlStatement.fetch(var row: TPDORowNative): Boolean;
var
    loop: byte;
    PLengths: PLongInt;
    null: Boolean;
    nullmode: Byte;
    buffer: PChar;
    stringRep: AnsiString;
    coltype: TZSQLType;
    tmp_storage: array of AnsiString;
begin
    nullmode := self.FProtocol.GetOracleNull;
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
      begin
        SetLength(tmp_storage, self.numberColumns);
        loop := 0;
        while (loop < self.numberColumns) do
        begin
            self.bindColumn(loop,@tmp_storage[loop], string_unlimited);
            loop := loop + 1;
        end;
        if not self.fetch_bound then
            Result := false
        else
          begin
            Result := true;
            row.InitializeRow(self.numberColumns);
            loop := 0;
            while (loop < self.numberColumns) do
            begin
                coltype := self.ColumnsInfo[loop].Datatype;
                null := (self.column_locations[loop].is_null = 1);
                if (nullmode = EMPTY_STRING_NULL) and
                   (coltype = stString) and
                   (tmp_storage[loop] = '') then null := true;

                row.column[loop] := TMysqlResult.Create (coltype,tmp_storage[loop],null);
                row.InsertKey(loop,self.ColumnsInfo[loop].Name);
                loop := loop + 1;
            end;
          end;
        exit;
      end;
    if not self.successful_query then
    begin
    	Result := false;
        self.FProtocol.log(lcOther, SLastQueryFailed);
    	exit;
    end;
    Result := True;
    row.Clearcolumns;
    self.FRowHandle := self.FProtocol.FetchRow(self.FQueryHandle);
    if (self.FRowHandle = nil) then
      Begin
        self.FProtocol.FreeResult(self.FQueryHandle);
        Result := False;
      End
    else
      Begin
        row.InitializeRow(self.numberColumns);
        PLengths := self.FProtocol.FetchLengths(self.FQueryHandle);
        loop := 0;
        while (loop < self.numberColumns) do
          Begin
            Buffer := FProtocol.GetFieldData(FRowHandle, loop);
            stringRep := self.oracle(buffer,PLengths,nullmode,null);
            coltype := self.ColumnsInfo[loop].Datatype;
            row.column[loop] := TMysqlResult.Create (coltype,stringRep,null);
            row.InsertKey(loop,self.ColumnsInfo[loop].Name);
            Inc(PLengths);
            loop := loop + 1;
          End;
      End;
end;

function TPDOMySqlStatement.fetch (var row: TPDORowKeyStrings): Boolean;
var
    keystring: AnsiString;
    loop: byte;
    PLengths: PLongInt;
    buffer: PChar;
    tmp_storage: array of AnsiString;
    resultString: ansistring;
Begin
    row.Clear;
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
      begin
        SetLength(tmp_storage, self.numberColumns);
        loop := 0;
        while (loop < self.numberColumns) do begin
            self.bindColumn(loop,@tmp_storage[loop], string_unlimited);
            loop := loop + 1;
        end;
        if not self.fetch_bound then
            Result := false
        else
          begin
            Result := true;
            loop := 0;
            while (loop < self.numberColumns) do Begin
                keystring :=  self.ColumnsInfo[loop].Name + '=' +
                    tmp_storage[loop];
                row.Add(keystring);
                loop := loop + 1;
            end;
          end;
        exit;
      end;
    if not self.successful_query then
    begin
    	Result := false;
        self.FProtocol.log(lcOther, SLastQueryFailed);
    	exit;
    end;
    Result := True;
    self.FRowHandle := self.FProtocol.FetchRow(self.FQueryHandle);
    if (self.FRowHandle = nil) then
      Begin
        self.FProtocol.FreeResult(self.FQueryHandle);
        Result := False;
      End
    else
      Begin
        PLengths := self.FProtocol.FetchLengths(self.FQueryHandle);
        loop := 0;
        while (loop < self.numberColumns) do
            Begin
                Buffer := FProtocol.GetFieldData(FRowHandle, loop);
                SetString(resultString,buffer,PLengths^);
                keystring := format ('%s=%s', [self.ColumnsInfo[loop].Name,resultString]);

                row.Add(keystring);
                Inc(PLengths);
                loop := loop + 1;
          End;
      End;
End;

function TPDOMySqlStatement.fetch_all (var rowset: TPDORowSetKeyStrings): Boolean;
Var
    PLengths: PLongInt;
    loop: byte;
    rowcount: Int64;
    buffer: PChar;
    keystring: AnsiString;
    tmp_storage: array of AnsiString;
    resultString: ansistring;
Begin
    Result := False;
    rowset.ClearRowSet;
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
      begin
        SetLength(tmp_storage, self.numberColumns);
        loop := 0;
        while (loop < self.numberColumns) do begin
            self.bindColumn(loop,@tmp_storage[loop], string_unlimited);
            loop := loop + 1;
        end;
        rowset.InitializeRowSet(self.getNumberRows);
        rowcount := 0;
        while (self.fetch_bound) do begin
            Result := true;
            loop := 0;
            while (loop < self.numberColumns) do Begin
                keystring :=  self.ColumnsInfo[loop].Name + '=' +
                    tmp_storage[loop];
                rowset.row[rowcount].Add(keystring);
                loop := loop + 1;
            end;
            rowcount := rowcount + 1;
          end;
        exit;
      end;
    if not self.successful_query then
    begin
    	Result := false;
        self.FProtocol.log(lcOther, SLastQueryFailed);
    	exit;
    end;
    rowcount := self.FProtocol.GetNumRows(self.FQueryHandle);
    rowset.InitializeRowSet(rowcount);
    rowcount := 0;

    repeat
        self.FRowHandle := self.FProtocol.FetchRow(self.FQueryHandle);
        if (self.FRowHandle = nil) then
            self.FProtocol.FreeResult(self.FQueryHandle)
        else
          Begin
            Result := True;
            PLengths := self.FProtocol.FetchLengths(self.FQueryHandle);
            loop := 0;
            while (loop < self.numberColumns) do
              Begin
                Buffer := FProtocol.GetFieldData(FRowHandle, loop);
                SetString(resultString,buffer,PLengths^);
                keystring := format ('%s=%s', [self.ColumnsInfo[loop].Name,resultString]);
                rowset.row[rowcount].Add(keystring);
                Inc(PLengths);
                loop := loop + 1;
              End;
            rowcount := rowcount + 1;
          End;
    until self.FRowHandle = nil;
End;

function TPDOMySqlStatement.fetch_all (var rowset: TPDORowSetNative): Boolean;
Var
    nullmode: Byte;
    PLengths: PLongInt;
    loop: byte;
    rowcount: Int64;
    buffer: PChar;
    null: Boolean;
    stringRep: AnsiString;
    coltype: TZSQLType;
    tmp_storage: array of AnsiString;
Begin
    Result := False;
    rowset.ClearRowSet;
    nullmode := self.FProtocol.GetOracleNull;
    if (self.TypeOfStatement = PREPARED_STATEMENT) then
      begin
        SetLength(tmp_storage, self.numberColumns);
        loop := 0;
        while (loop < self.numberColumns) do begin
            self.bindColumn(loop,@tmp_storage[loop], string_unlimited);
            loop := loop + 1;
        end;
        rowset.InitializeRowSet(self.getNumberRows);
        rowcount := 0;
        while (self.fetch_bound) do begin
            Result := true;
            rowset.row[rowcount].InitializeRow(self.numberColumns);
            loop := 0;
            while (loop < self.numberColumns) do begin
                coltype := self.ColumnsInfo[loop].Datatype;
                null := (self.column_locations[loop].is_null = 1);
                if (nullmode = EMPTY_STRING_NULL) and
                   (coltype = stString) and
                   (tmp_storage[loop] = '') then null := true;

                rowset.row[rowcount].column[loop] := TMysqlResult.Create (coltype,tmp_storage[loop],null);
                rowset.row[rowcount].InsertKey(loop,self.ColumnsInfo[loop].Name);
                loop := loop + 1;
            end;
            rowcount := rowcount + 1;
          end;
        exit;
      end;
    if not self.successful_query then
    begin
    	Result := false;
        self.FProtocol.log(lcOther, SLastQueryFailed);
    	exit;
    end;
    rowcount := self.FProtocol.GetNumRows(self.FQueryHandle);
    rowset.InitializeRowSet(rowcount);
    rowcount := 0;

    repeat
        self.FRowHandle := self.FProtocol.FetchRow(self.FQueryHandle);
        if (self.FRowHandle = nil) then
            self.FProtocol.FreeResult(self.FQueryHandle)
        else
          Begin
            Result := True;
            rowset.row[rowcount].InitializeRow(self.numberColumns);
            PLengths := self.FProtocol.FetchLengths(self.FQueryHandle);
            loop := 0;
            while (loop < self.numberColumns) do
              Begin
                Buffer := FProtocol.GetFieldData(FRowHandle, loop);
                stringRep := self.oracle(buffer,PLengths,nullmode,null);
                coltype := self.ColumnsInfo[loop].Datatype;
                rowset.row[rowcount].column[loop] := TMysqlResult.Create (coltype,stringRep,null);
                rowset.row[rowcount].InsertKey(loop,self.ColumnsInfo[loop].Name);
                Inc(PLengths);
                loop := loop + 1;
              End;
            rowcount := rowcount + 1;
          End;
    until self.FRowHandle = nil;
End;

function TPDOMySqlStatement.nextRowSet: Boolean;
Begin
    Result := False;
    if (self.FProtocol.CheckAnotherRowset(self.FMySQL)) then
        begin
            self.clearBuffer;
            if (self.FProtocol.RetrieveNextRowset(self.FMySQL) = 0) then
            begin
                Result := True;
                self.prefetchIfNecessary;
            end;
        end;
End;

function TPDOMySqlStatement.columnCount: Integer;
Begin
    Result := self.numberColumns;
End;

procedure TPDOMySqlStatement.prefetchIfNecessary;
var
    indicator: Integer;
Begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            if (self.FProtocol.GetBufferQuery) then
                self.FQueryHandle := self.FProtocol.StoreResult(self.FMySQL)
            else
                self.FQueryHandle := self.FProtocol.UseResult(self.FMySQL);
        end
    else
        begin
            indicator := 0;
            if (self.FProtocol.GetBufferQuery) then
                indicator := self.FProtocol.StorePreparedResult(self.FPrepStmtHandle);
            self.Successful_stmt_store := (indicator = 0);
           {  You don't need to "Use Result" for a prepared statement }
        end;
End;

function TPDOMySqlStatement.LastInsertId: Int64;
begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        Result := self.FProtocol.getLastInsertID(self.FMySQL)
    else
        Result := self.FProtocol.GetPreparedInsertID(self.FPrepStmtHandle);
end;

function TPDOMySqlStatement.getRowsAffected: Int64;
begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        Result := self.FProtocol.GetAffectedRows(self.FMySQL)
    else
        Result := self.FProtocol.GetPreparedAffectedRows(self.FPrepStmtHandle);
end;

function TPDOMySqlStatement.getNumberRows: Int64;
begin
    if self.FProtocol.GetBufferQuery then
      begin
        if (self.TypeOfStatement = DIRECT_STATEMENT) then
            Result := self.FProtocol.GetNumRows(self.FQueryHandle)
        else
            Result := self.FProtocol.GetPreparedNumRows(self.FPrepStmtHandle);
      end
    else
        Result := -1;   {if result set is not buffered, then this number is inaccurate, so we set to -1}
end;

function TPDOMySqlStatement.errorInfo: PDO_errorInfoRecord;
Begin
     if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            Result.Error_code := self.FProtocol.GetLastErrorCode(self.FMySQL);
            Result.Error_msg  := self.FProtocol.GetLastErrorMessage(self.FMySQL);
            Result.SQLState   := self.FProtocol.GetSQLState(self.FMySQL);
        end
     else
        begin
            Result.Error_code := self.FProtocol.GetLastPreparedErrorCode(self.FPrepStmtHandle);
            Result.Error_msg  := self.FProtocol.GetLastPreparedError(self.FPrepStmtHandle);
            Result.SQLState   := self.FProtocol.GetPreparedSQLState(self.FPrepStmtHandle);
        end;
End;

function TPDOMySqlStatement.errorCode: AnsiString;
Begin
     if (self.TypeOfStatement = DIRECT_STATEMENT) then
        Result := self.FProtocol.GetSQLState(self.FMySQL)
     else
        Result := self.FProtocol.GetPreparedSQLState(self.FPrepStmtHandle);
End;

function TPDOMySqlStatement.execute(DelimitedBind: AnsiString): Boolean;
const
    pipe = '|';
var
    dx, counter: integer;
    txt, ns: AnsiString;
Begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            Result := false;
            self.FProtocol.log(lcPrepStmt, SUsedWrongExecute);
            exit;
        end;
     txt := DelimitedBind + pipe;
     counter := 0;
     while (Length(txt) > 0) do
     begin
        dx := AnsiPos(pipe,txt);
        ns := copy (txt, 0, dx - 1);
        txt := copy (txt, dx + 1, MaxInt);
        self.bindValue(counter,ns);
     counter := counter + 1;
     end;
     Result := self.execute;
End;

function TPDOMySqlStatement.execute: Boolean;
var
    testResult: Byte;
    loop: Cardinal;
begin
    Result := false;
    self.successful_query := false;
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        begin
            testResult := self.FProtocol.ExecRealQuery(self.FMySQL, PChar(self.DirectSQL), length(self.DirectSQL));
            if (testResult <> 0) then
                begin
                    checkMySQLError (self.FProtocol, self.FMySQL, lcExecute, self.DirectSQL);

                    exit;
                end;
            self.FProtocol.log(lcPrepstmt, self.DirectSQL);
            self.prefetchIfNecessary;
            self.populateColumnInfo;   
            Result := True;
	    self.successful_query := true;
        end
    else
        begin
            loop := 0;
            while (loop < self.bind_markers) do
              begin
                if (self.params_bound[loop] = false) then
                  begin
                    self.FProtocol.log(lcExecute, SFailedToBindAllValues, 0, 'param ' + IntToStr(loop + 1));
                    exit;
                  end;
                loop := loop + 1;
              end;

            if not self.Successful_stmt_prep then
                begin
                    self.FProtocol.log(lcExecute, SAttemptExecOnBadPrep);
                    exit;
                end;
            testResult := self.FProtocol.BindParameters(self.FPrepStmtHandle, self.PointToBindStructures);
            if (testResult  <> 0) then
                begin
                    checkMySQLPrepStmtError (self.FProtocol, self.FPrepStmtHandle, lcPrepStmt, SBindingFailure);
                    exit;
                End;

            testResult := self.FProtocol.ExecuteStmt(self.FPrepStmtHandle);
            if (testResult <> 0) then
                begin
                    checkMySQLPrepStmtError(self.FProtocol,self.FPrepStmtHandle, lcPrepStmt, SPreparedStmtExecFailure);
                    exit;
                End;
            self.FProtocol.log(lcExecPrepStmt, self.PreparedSQL);
            { if (self.checkRebindingNeeded) then exit;  binding not required until fetch} {bind failure}
            self.prefetchIfNecessary;
            Result := True;
            self.successful_query := true;
        end;
end;

procedure TPDOMySqlStatement.setSQL (SQL: AnsiString; StatementType: TPDOStatementType);
var
    PS_SQL: AnsiString;
    testResult: Integer;
begin
    self.clearBuffer;
    self.TypeOfStatement := StatementType;
    self.alpha_bind_values.Clear;
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
        self.DirectSQL := SQL
    else
        begin
            PS_SQL := PDOSysUtils.derive_alpha_markers(SQL, self.alpha_bind_values);
            self.PreparedSQL := PS_SQL;
            self.FPrepStmtHandle := self.FProtocol.InitializePrepStmt(self.FMySQL);
            if (self.FPrepStmtHandle = nil) then
              begin
                CheckMySQLPrepStmtError(self.FProtocol, self.FPrepStmtHandle, lcPrepStmt, SFailedtoInitPrepStmt);
                exit;
              end;
            testResult := self.FProtocol.PrepareStmt(self.FPrepStmtHandle, PChar(PS_SQL),length(PS_SQL));
            if (testResult <> 0) then
              begin
                Successful_stmt_prep := false;
                CheckMySQLPrepStmtError(self.FProtocol, self.FPrepStmtHandle, lcPrepStmt, SFailedtoPrepareStmt);
                exit;
              end;
            Successful_stmt_prep := true;
            self.FProtocol.log(lcPrepStmt, PS_SQL);
            self.bind_markers := self.FProtocol.GetPreparedBindMarkers(self.FPrepStmtHandle);
            self.SetStructureSize;
            self.populateColumnInfo;
            if (self.numberColumns > 0) then self.rebind_required := true;
        end;
end;

constructor TPDOMySqlStatement.create (
    DriverHandle: TZMySQLBaseDriver;
    DBaseHandle:  PZMySQLConnect);
begin
    self.FQueryHandle    := nil;
    self.FPrepStmtHandle := nil;
    self.FRowHandle      := nil;
    self.FProtocol       := DriverHandle;
    self.FMySQL          := DBaseHandle;
    self.ColumnsInfo     := nil;
    self.numberColumns   := 0;
    self.bind_markers    := 0;

    self.alpha_bind_values :=  TStringList.Create;
    self.alpha_bind_columns := TStringList.Create;
    self.rebind_required := False;
    self.successful_query := false;
end;

procedure TPDOMySqlStatement.beforeDestruction;
begin
    self.alpha_bind_values.Free;
    self.alpha_bind_columns.Free;
    self.clearBuffer;  {clears statement and query handles mainly}
end;

procedure TPDOMySqlStatement.clearBuffer;
begin
    if self.FRowHandle <> nil then
        self.FProtocol.FreeResult(self.FQueryHandle);
    if self.FPrepStmtHandle <> nil then
        self.FProtocol.ClosePrepStmt(self.FPrepStmtHandle);
    self.FRowHandle := nil;
    self.FPrepStmtHandle := nil;
    self.ColumnsInfo := nil;
    self.numberColumns := 0;
    self.bind_markers  := 0;
    self.params_bound  := nil;
    self.columns_bound := nil;
    self.rebind_required := false;
    self.Successful_stmt_store := false;
    self.Successful_stmt_prep  := false;

end;

function TPDOMySqlStatement.closeCursor: Boolean;
Begin
    Result := (self.FQueryHandle <> nil) or (self.FPrepStmtHandle <> nil);
    self.clearBuffer;
End;

procedure TPDOMySqlStatement.coreColumnPopulation (ResultHandle: PZMySQLResult);
var
    FieldHandle: PZMySQLField;
    casemode, loop: Byte;
    reglength, maxlength: Integer;
    keystring: AnsiString;
begin
    SetLength(self.ColumnsInfo, self.numberColumns);
    casemode := self.FProtocol.GetCaseMode;
    loop := 0;
    while (loop < self.numberColumns) do
    begin
        self.FProtocol.SeekField(ResultHandle, loop);
        FieldHandle := self.FProtocol.FetchField(ResultHandle);
        if (FieldHandle = nil) then break;
        reglength          := self.FProtocol.GetFieldLength(FieldHandle);
        maxlength          := self.FProtocol.GetFieldMaxLength(FieldHandle);
        with self.ColumnsInfo[loop] do
        begin
            Name       := recase(self.FProtocol.GetFieldName(FieldHandle),casemode);
            Flags      := self.FProtocol.GetFieldFlags(FieldHandle);
            Scale      := self.FProtocol.GetFieldDecimals(FieldHandle);
            Datatype   := ConvertMySQLHandleToSQLType (self.FProtocol, FieldHandle, Flags);
            Datatype2  := self.FProtocol.GetFieldType(FieldHandle);
            Precision  := maxlength;
            DataLength := reglength;
            if (reglength > maxlength) then
                Precision := reglength;
        end;
        keystring := AnsiUpperCase(self.ColumnsInfo[loop].Name);
        if (self.alpha_bind_columns.IndexOfName(keystring) = - 1) then
            begin
                keystring := keystring + '=' + IntToStr(loop);
                self.alpha_bind_columns.Add(keystring);
            end;
        loop := loop + 1;
    end;
end;

procedure TPDOMySqlStatement.populateColumnInfo;
var
    loop: Byte;
    ResultHandle: PZMySQLResult;
    bufferSize: LongWord;
begin
    if (self.TypeOfStatement = DIRECT_STATEMENT) then
      begin
        self.numberColumns := self.FProtocol.GetFieldCount(self.FQueryHandle);
        ResultHandle := self.FQueryHandle;
        self.coreColumnPopulation(ResultHandle);
      end
    else
      Begin
        self.numberColumns := self.FProtocol.GetPreparedFieldCount(self.FPrepStmtHandle);
        self.SetStructureSizeColumns;

        ResultHandle := self.FProtocol.GetPreparedMetaData(self.FPrepStmtHandle);
        self.alpha_bind_columns.Clear;
        self.coreColumnPopulation(ResultHandle);
        self.FProtocol.FreeResult(ResultHandle);
        loop := 0;
        while (loop < self.numberColumns) do
          begin
            self.columns_bound[loop] := false;
            BufferSize := self.getFieldSize(self.ColumnsInfo[loop].Datatype2,
                self.ColumnsInfo[loop].DataLength);
            SetLength(self.column_locations[loop].buffer,BufferSize);
            loop := loop + 1;
          end;
      end;
end;

function TPDOMySqlStatement.fetch_bound: Boolean;
var
    testResult: Byte;
begin
    Result := false;
    if (self.checkRebindingNeeded) then exit;    {bind failure}

    testResult := self.FProtocol.FetchBoundResults(self.FPrepStmtHandle);
    if (testResult = 1) then
        begin
            CheckMySQLPrepStmtError(self.FProtocol,self.FPrepStmtHandle,lcExecPrepStmt,SFailedFetchingResult);
            exit;
        end;
    if (testResult = 0) then
    begin
        TransferVariables;
        Result := true;
    end;
end;

procedure TPDOMySqlStatement.TransferVariables;
var
    loop: Byte;
    target: Pointer;
    PBuffer: Pointer;
    Signed: Boolean;
    bufLen: LongWord;
    bufType: TMysqlFieldTypes;
begin
    bufType := MYSQL_TYPE_TINY;  {useless initialization}
    signed  := true;             {ditto}
    for loop := 0 to self.numberColumns - 1 do
      begin
        self.ExtractColMetaData(loop,signed,bufType);
        target :=  self.column_addresses[loop];
        PBuffer := PChar(self.column_locations[loop].buffer);
        bufLen :=  self.column_locations[loop].length;
        case buftype of
          MYSQL_TYPE_TINY:      Begin
                                  if signed then shortInt(target^) := shortInt(PBuffer^)
                                  else Byte(target^) := Byte(PBuffer^);
                                end;
          MYSQL_TYPE_SHORT:     Begin
                                  if signed then SmallInt(target^) := smallInt(PBuffer^)
                                  else Word(target^) := Word(PBuffer^);
                                end;
          MYSQL_TYPE_LONG:      Begin
                                  if signed then LongInt(target^) := LongInt(PBuffer^)
                                  else LongWord(target^) := LongWord(PBuffer^);
                                end;
          MYSQL_TYPE_LONGLONG:    Int64(target^) := Int64(PBuffer^);
          MYSQL_TYPE_FLOAT:       Single(target^) := Single(PBuffer^);
          MYSQL_TYPE_DOUBLE:      Double(target^) := Double(PBuffer^);
          MYSQL_TYPE_STRING:      SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_VAR_STRING:  SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_TINY_BLOB:   SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_MEDIUM_BLOB: SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_BLOB:        SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_LONG_BLOB:   SetString(AnsiString(target^),PChar(PBuffer),buflen);
          MYSQL_TYPE_TIME:        TPDOGenericTime(target^) := convertMySQL_TIME (MYSQL_TIME(PBuffer^));
          MYSQL_TYPE_DATE:        TPDOGenericTime(target^) := convertMySQL_TIME (MYSQL_TIME(PBuffer^));
          MYSQL_TYPE_DATETIME:    TPDOGenericTime(target^) := convertMySQL_TIME (MYSQL_TIME(PBuffer^));
          MYSQL_TYPE_TIMESTAMP:   TPDOGenericTime(target^) := convertMySQL_TIME (MYSQL_TIME(PBuffer^));
       end;
    end;
end;

function TPDOMySqlStatement.checkRebindingNeeded: Boolean;
var
    testResult: Byte;
    loop: smallInt;
Begin
    Result := False;
    if not self.rebind_required then exit;

    {check if all columns bound}

    for loop := 0 to self.numberColumns - 1 do
        if not self.columns_bound[loop] then
        begin
            self.FProtocol.log(lcPrepStmt, SFailedToBindAllColumns, 0, 'column ' + IntToStr(loop));
            Result := true;
            exit;
        end;

    testResult := self.FProtocol.BindResult(self.FPrepStmtHandle, self.PointToColumnStructures);
    if (testResult <> 0) then
        begin
            CheckMySQLPrepStmtError(self.FProtocol,self.FPrepStmtHandle,lcPrepStmt, SFailedToBindResults);
            Result := true;
            exit;
        end;
    self.rebind_required := false;
End;

procedure TPDOMySqlStatement.bindValue  (valueIndex: Cardinal; PassedValue: Variant);
var
    field_size, field_size_twin: LongWord;
    field_type: TMysqlFieldTypes;
    signature:  Byte;
    PBuffer: Pointer;
    TC_INT64: Int64;
    castString: ansiString;
Begin
    if (valueIndex >= self.bind_markers) then
        begin
            self.FProtocol.log(lcPrepStmt, format(SBindVarOutOfRange,[valueIndex]) ,0,' max < ' + IntToStr(self.bind_markers));
            exit;
        end;

    field_type := self.getFieldType(PassedValue);
    field_size := self.getFieldSize(field_type,255);
    if (field_type = MYSQL_TYPE_STRING) then
      begin
        castString := PassedValue;
        field_size := length(castString);
        field_size_twin := field_size + 1;
      end
    else field_size_twin := field_size;

    signature := self.getSignature(PassedValue);

    SetLength(self.bind_locations[valueIndex].buffer, field_size_twin);
    PBuffer := @self.bind_locations[valueIndex].buffer[0];



    case field_type of
        MYSQL_TYPE_TINY:     if ((VarType(PassedValue) and VarTypeMask) = varBoolean) then
                                begin
                                    if PassedValue then
                                        Byte(PBuffer^)       := Byte(1)
                                    else
                                        Byte(PBuffer^)       := Byte(0);
                                end
                             else  Byte(PBuffer^) := Byte(PassedValue);
        MYSQL_TYPE_SHORT:    Word(PBuffer^)       := Word(PassedValue);
        MYSQL_TYPE_LONG:     LongWord(PBuffer^)   := LongWord(PassedValue);
        MYSQL_TYPE_FLOAT:    Single(PBuffer^)     := Single(PassedValue);
        MYSQL_TYPE_DOUBLE:   Double(PBuffer^)     := Double(PassedValue);
        MYSQL_TYPE_STRING:   self.transferBytes(PChar(PBuffer), PassedValue);
        MYSQL_TYPE_LONGLONG: Begin
                                {for some reason, can't cast variant to Int64}
                                TC_INT64        := PassedValue;
                                Int64(PBuffer^) := TC_INT64;
                             end;
    end;
    self.bind_locations[valueIndex].length := field_size;
    self.bind_locations[valueIndex].is_null := 0;
    self.UpdateBindStructure(valueIndex,signature,field_size,field_type);
    self.params_bound[valueIndex] := true;
End;


procedure TPDOMySqlStatement.bindValue (valueIndex: AnsiString; PassedValue: Variant);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_values.IndexOfName(valueIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [valueIndex]));
            exit;
        end;
    trueIndex := StrToInt(self.alpha_bind_values.ValueFromIndex[localIndex]);
    self.bindValue(trueIndex, PassedValue);
End;

procedure TPDOMySqlStatement.bindTimeValue  (valueIndex: Cardinal; PassedValue: TPDOGenericTime);
var
    field_size: LongWord;
    PBuffer: Pointer;
Begin
    if (valueIndex >= self.bind_markers) then
        begin
            self.FProtocol.log(lcPrepStmt, format(SBindVarOutOfRange,[valueIndex]) ,0,' max < ' + IntToStr(self.bind_markers));
            exit;
        end;

    field_size := self.getFieldSize(MYSQL_TYPE_DATETIME,255);
    SetLength(self.bind_locations[valueIndex].buffer, field_size);
    PBuffer  := @self.bind_locations[valueIndex].buffer[0];
    MYSQL_TIME(PBuffer^) := self.convertGeneric(PassedValue);
    self.bind_locations[valueIndex].length := field_size;
    self.bind_locations[valueIndex].is_null := 0;
    self.UpdateBindStructure(valueIndex,0,field_size,MYSQL_TYPE_DATETIME);
    self.params_bound[valueIndex] := true;
End;

procedure TPDOMySqlStatement.bindTimeValue  (valueIndex: AnsiString; PassedValue: TPDOGenericTime);
var
    localIndex: SmallInt;
    trueIndex: Cardinal;
Begin
    localIndex := self.alpha_bind_values.IndexOfName(valueIndex);
    if (localIndex = -1) then
        begin
            raise EZSQLException.Create(Format(SBoundVarStrIndexMissing, [valueIndex]));
            exit
        end;
    trueIndex := StrToInt(self.alpha_bind_values.ValueFromIndex[localIndex]);
    self.bindTimeValue(trueIndex, PassedValue);
End;

{
function TPDOMySqlStatement.signInteger (raw: LongWord; power: Byte): LongInt;
var
    bitmask: Int64;
begin
    bitmask := 1 shl (power - 1);
    if (raw and bitmask > 0) then     //it's a negative number
        Result := raw - (bitmask shl 1)
    else
        Result := raw;
end;
}

procedure TPDOMySqlStatement.transferBytes (bufferStart: PChar; nativeString: AnsiString);
var
    loop: Cardinal;
begin
    for loop := 1 to length(nativeString) do
    begin
        bufferStart^ := nativeString[loop];
        inc(bufferStart);
    end;
    bufferStart^ := chr(0);
end;


procedure TPDOMySqlStatement.UpdateBindStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes);
begin
    with self.bind_structures[index] do begin
        buffer_length := field_size;
        buffer_type   := field_type;
        is_unsigned   := signature;
        buffer        := @self.bind_locations[index].buffer[0];
        length        := @self.bind_locations[index].length;
        is_null       := @self.bind_locations[index].is_null;
    end;
end;

procedure TPDOMySqlStatement.UpdateColumnStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes);
begin
    with self.column_structures[index] do begin
        buffer_type   := field_type;
        buffer_length := field_size;
        is_unsigned   := signature;
        buffer        := @self.column_locations[index].buffer[0];
        length        := @self.column_locations[index].length;
        is_null       := @self.column_locations[index].is_null;
     end;
end;

procedure TPDOMySqlStatement.ExtractColMetaData (index: Byte; var signed: Boolean; var field_type:TMysqlFieldTypes);
begin
    with self.column_structures[index] do begin
        field_type := buffer_type;
        signed  := (is_unsigned = 0);
    end;
end;

procedure TPDOMySqlStatement.SetStructureSize;
var
    loop: Byte;
begin
    SetLength(self.bind_structures,self.bind_markers);
    setLength(self.bind_locations,self.bind_markers);
    setLength(self.params_bound, self.bind_markers);
    loop := 0;
    while (loop < self.bind_markers) do begin
        self.params_bound[loop] := false;
        loop := loop + 1;
    end;
end;

procedure TPDOMySqlStatement.SetStructureSizeColumns;
begin
    setLength(self.column_structures, self.numberColumns);
    setLength(self.column_locations,  self.numberColumns);
    setLength(self.columns_bound,     self.numberColumns);
    setLength(self.column_addresses,  self.numberColumns);
end;

function TPDOMySqlStatement.PointToBindStructures: Pointer;
begin
    Result := @self.bind_structures[0];
end;

function TPDOMySqlStatement.PointToColumnStructures: Pointer;
begin
    Result := @self.column_structures[0]
end;


{IDENTICAL VERSION FOR MYSQL 5.0, needs to have MYSQL_BIND2 in scope}


procedure TPDOMySqlStatement50.UpdateBindStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes);
begin
    with self.bind_structures[index] do begin
        buffer_length := field_size;
        buffer_type   := field_type;
        is_unsigned   := signature;
        buffer        := @self.bind_locations[index].buffer[0];
        length        := @self.bind_locations[index].length;
        is_null       := @self.bind_locations[index].is_null;
    end;
end;

procedure TPDOMySqlStatement50.UpdateColumnStructure(index, signature: Byte; field_size: Cardinal; field_type:TMysqlFieldTypes);
begin
    with self.column_structures[index] do begin
        buffer_type   := field_type;
        buffer_length := field_size;
        is_unsigned   := signature;
        buffer        := @self.column_locations[index].buffer[0];
        length        := @self.column_locations[index].length;
        is_null       := @self.column_locations[index].is_null;
     end;
end;

procedure TPDOMySqlStatement50.ExtractColMetaData (index: Byte; var signed: Boolean; var field_type:TMysqlFieldTypes);
begin
    with self.column_structures[index] do begin
        field_type := buffer_type;
        signed  := (is_unsigned = 0);
    end;
end;

procedure TPDOMySqlStatement50.SetStructureSize;
var
    loop: Byte;
begin
    SetLength(self.bind_structures,self.bind_markers);
    setLength(self.bind_locations,self.bind_markers);
    setLength(self.params_bound, self.bind_markers);
    loop := 0;
    while (loop < self.bind_markers) do begin
        self.params_bound[loop] := false;
        loop := loop + 1;
    end;
end;

procedure TPDOMySqlStatement50.SetStructureSizeColumns;
begin
    setLength(self.column_structures, self.numberColumns);
    setLength(self.column_locations,  self.numberColumns);
    setLength(self.columns_bound,     self.numberColumns);
    setLength(self.column_addresses,  self.numberColumns);
end;

function TPDOMySqlStatement50.PointToBindStructures: Pointer;
begin
    Result := @self.bind_structures[0];
end;

function TPDOMySqlStatement50.PointToColumnStructures: Pointer;
begin
    Result := @self.column_structures[0]
end;
end.
