{ $Id: PDODriverMySQL40.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}



unit PDODriverMySQL40;

interface

{$I ../../../directives.inc}

uses
	SysUtils, PDOClasses, PDOInterfaces, PDOConstantsMySQL, PDOMySQLDriver, PDOLibraryMySQL40, PDOMessages;

type
  TMySQL40Driver = class (TZMySQLBaseDriver, IPDOProtocol)
  private
    function versionConversion (raw: Cardinal): AnsiString;
    
  public

    function GetProtocol: AnsiString; override;
    function GetDescription: AnsiString; override;
    procedure Initialize; override;

    procedure Debug(Debug: PChar);
    function DumpDebugInfo(Handle: PZMySQLConnect): Integer;
    function Init(var Handle: PZMySQLConnect): PZMySQLConnect; override;
    procedure Dispose(var Handle: PZMySQLConnect);  override;

    function Connect(Handle: PZMySQLConnect;
      const Host, User, Password: PChar): PZMySQLConnect; override;
    function RealConnect(Handle: PZMySQLConnect;
      const Host, User, Password, Db: PChar; Port: Cardinal;
      UnixSocket: PChar; ClientFlag: Cardinal): PZMySQLConnect; override;
    procedure Close(Handle: PZMySQLConnect); override;

    function ExecQuery(Handle: PZMySQLConnect; const Query: PChar): Integer; override;
    function ExecRealQuery(Handle: PZMySQLConnect; const Query: PChar;
      Length: Integer): Integer; override;

    function SelectDatabase(Handle: PZMySQLConnect; const Database: PChar): Integer;
    function CreateDatabase(Handle: PZMySQLConnect;
      const Database: PChar): Integer;
    function DropDatabase(Handle: PZMySQLConnect;
      const Database: PChar): Integer;

    function SetOptions(Handle: PZMySQLConnect; Option: TMySQLOption;
      const Arg: PChar): Integer; override;
    function GetEscapeString(StrTo, StrFrom: PChar; Length: Cardinal): Cardinal;

    function GetClientInfo: AnsiString; override;
    function GetHostInfo(Handle: PZMySQLConnect): PChar;
    function GetProtoInfo(Handle: PZMySQLConnect): Cardinal;
    function GetThreadId(Handle: PZMySQLConnect): Cardinal;

    function GetListDatabases(Handle: PZMySQLConnect;
      Wild: PChar): PZMySQLResult;
    function GetListTables(Handle: PZMySQLConnect;
      const Wild: PChar): PZMySQLResult;
    function GetListFields(Handle: PZMySQLConnect;
      const Table, Wild: PChar): PZMySQLResult;
    function GetListProcesses(Handle: PZMySQLConnect): PZMySQLResult;

    function StoreResult(Handle: PZMySQLConnect): PZMySQLResult; override;
    function UseResult(Handle: PZMySQLConnect): PZMySQLResult; override;
    procedure FreeResult(Res: PZMySQLResult);  override;
    function GetAffectedRows(Handle: PZMySQLConnect): Int64; override;

    function FetchRow(Res: PZMySQLResult): PZMySQLRow;  override;
    function FetchLengths(Res: PZMySQLResult): PLongInt;  override;
    function FetchField(Res: PZMySQLResult): PZMySQLField;  override;

    procedure SeekData(Res: PZMySQLResult; Offset: Cardinal);  override;
    function SeekRow(Res: PZMySQLResult; Row: PZMySQLRowOffset): PZMySQLRowOffset;  override;
    function SeekField(Res: PZMySQLResult; Offset: Cardinal): Cardinal;   override;

    function GetFieldType(Field: PZMySQLField): TMysqlFieldTypes;  override;
    function GetFieldFlags(Field: PZMySQLField): Integer;  override;
    function GetStatus(Handle: PZMySQLConnect): TMySQLStatus;  override;
    function GetRowCount(Res: PZMySQLResult): Int64;   override;
    function GetFieldCount(Res: PZMySQLResult): Integer;    override;
    function GetFieldName(Field: PZMySQLField): PChar;   override;
    function GetFieldTable(Field: PZMySQLField): PChar;   override;
    function GetFieldLength(Field: PZMySQLField): Integer;  override;
    function GetFieldMaxLength(Field: PZMySQLField): Integer;   override;
    function GetFieldDecimals(Field: PZMySQLField): Integer;    override;
    function GetFieldData(Row: PZMySQLRow; Offset: Cardinal): PChar;  override;


    function SetAutocommit (Handle: PZMySQLConnect; mode: Boolean): Boolean;  override;
    function Commit (Handle: PZMySQLConnect): Boolean;   override;
    function Rollback (Handle: PZMySQLConnect): Boolean;  override;
    function InitializePrepStmt (Handle: PZMySQLConnect): PZMySqlPrepStmt;  override;
    function ClosePrepStmt (PrepStmtHandle: PZMySqlPrepStmt): PZMySqlPrepStmt;  override;
    function PrepareStmt (PrepStmtHandle: PZMySqlPrepStmt; const Query: PChar; Length: Integer): Integer;  override;
    function GetClientVersion: AnsiString; override;
    function GetNumRows(Res: PZMySQLResult): Int64;  override;
    function GetLastPreparedError(Handle: PZMySqlPrepStmt): AnsiString;  override;
    function GetLastPreparedErrorCode(Handle: PZMySqlPrepStmt): Integer;  override;
    function GetPreparedSQLState (Handle: PZMySqlPrepStmt): PChar;   override;
    function GetPreparedNumRows (Handle: PZMySqlPrepStmt): Int64;  override;
    function GetPreparedAffectedRows (Handle: PZMySqlPrepStmt): Int64;   override;
    function GetPreparedInsertID (Handle: PZMySqlPrepStmt): Int64;   override;
    function StorePreparedResult (Handle: PZMySqlPrepStmt): Integer;   override;
    function GetPreparedMetaData (Handle: PZMySqlPrepStmt): PZMySQLResult;  override;
    function GetPreparedFieldCount(Handle: PZMySqlPrepStmt): Integer;  override;
    function CheckAnotherRowset   (Handle: PZMySQLConnect): Boolean;  override;
    function RetrieveNextRowset   (Handle: PZMySQLConnect): Integer;  override;
    function BindParameters (Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;  override;
    function ExecuteStmt (Handle: PZMySqlPrepStmt): Integer;   override;
    function GetPreparedBindMarkers (Handle: PZMySqlPrepStmt): Cardinal;  override;
    function FetchBoundResults (Handle: PZMySqlPrepStmt): Integer;   override;
    function BindResult (Handle: PZMySqlPrepStmt;  bindArray: PZMysqlBindArray): Byte;  override;
    function SetServerOption (Handle: PZMySQLConnect; option: TMysqlSetOption): Integer; override;

    {MOVED FROM IPDOPROTOCOL TO BASE DRIVERS}
    function GetSQLState         (Handle: PZMySQLConnect): AnsiString; override;
    function GetLastErrorMessage (Handle: PZMySQLConnect): AnsiString; override;
    function GetLastErrorCode    (Handle: PZMySQLConnect): Integer;    override;
    function GetLastInsertID     (Handle: PZMySQLConnect): Int64;      override;
    function GetServerVersion    (Handle: PZMySQLConnect): AnsiString; override;
    function GetServerInfo       (Handle: PZMySQLConnect): AnsiString; override;
  end;
  
  


implementation


function TMySQL40Driver.GetProtocol: AnsiString;
begin
  Result := 'mysql-4.0';
end;

function TMySQL40Driver.GetDescription: AnsiString;
begin
  Result := 'Native driver for MySQL 4.0.28';
end;

procedure TMySQL40Driver.Initialize;
begin
  PDOLibraryMySQL40.LibraryLoader.LoadIfNeeded;
end;

procedure TMySQL40Driver.Close(Handle: PZMySQLConnect);
begin
  PDOLibraryMySQL40.mysql_close(Handle);
 // PDOLibraryMySQL40.mysql_server_end;  {this crashes 4.0}
end;

function TMySQL40Driver.Connect(Handle: PZMySQLConnect; const Host,
  User, Password: PChar): PZMySQLConnect;
begin
  Result := PDOLibraryMySQL40.mysql_connect(Handle, Host, User, Password);
end;

function TMySQL40Driver.CreateDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_create_db(Handle, Database);
end;

procedure TMySQL40Driver.Debug(Debug: PChar);
begin
  PDOLibraryMySQL40.mysql_debug(Debug);
end;

function TMySQL40Driver.DropDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_drop_db(Handle, Database);
end;

function TMySQL40Driver.DumpDebugInfo(Handle: PZMySQLConnect): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_dump_debug_info(Handle);
end;

function TMySQL40Driver.ExecQuery(Handle: PZMySQLConnect;
  const Query: PChar): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_query(Handle, Query);
end;

function TMySQL40Driver.ExecRealQuery(Handle: PZMySQLConnect;
  const Query: PChar; Length: Integer): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_real_query(Handle, Query, Length);
end;

function TMySQL40Driver.FetchField(Res: PZMySQLResult): PZMySQLField;
begin
  Result := PDOLibraryMySQL40.mysql_fetch_field(Res);
end;

function TMySQL40Driver.FetchLengths(Res: PZMySQLResult): PLongInt;
begin
  Result := PDOLibraryMySQL40.mysql_fetch_lengths(Res);
end;

function TMySQL40Driver.FetchRow(Res: PZMySQLResult): PZMySQLRow;
begin
    if assigned(Res) then
        Result := PDOLibraryMySQL40.mysql_fetch_row(Res)
    else
        Result := nil;
end;

procedure TMySQL40Driver.FreeResult(Res: PZMySQLResult);
begin
  PDOLibraryMySQL40.mysql_free_result(Res);
end;

function TMySQL40Driver.GetAffectedRows(Handle: PZMySQLConnect): Int64;
begin
  Result := PDOLibraryMySQL40.mysql_affected_rows(Handle);
end;

function TMySQL40Driver.GetClientInfo: AnsiString;
begin
  Result := AnsiString(PDOLibraryMySQL40.mysql_get_client_info);
end;

function TMySQL40Driver.GetEscapeString(StrTo, StrFrom: PChar;
  Length: Cardinal): Cardinal;
begin
  Result := PDOLibraryMySQL40.mysql_escape_string(StrTo, StrFrom, Length);
end;

function TMySQL40Driver.GetHostInfo(Handle: PZMySQLConnect): PChar;
begin
  Result := PDOLibraryMySQL40.mysql_get_host_info(Handle);
end;

function TMySQL40Driver.GetListDatabases(Handle: PZMySQLConnect;
  Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_list_dbs(Handle, Wild);
end;

function TMySQL40Driver.GetListFields(Handle: PZMySQLConnect;
  const Table, Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_list_fields(Handle, Table, Wild);
end;

function TMySQL40Driver.GetListProcesses(
  Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_list_processes(Handle);
end;

function TMySQL40Driver.GetListTables(Handle: PZMySQLConnect;
  const Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_list_tables(Handle, Wild);
end;

function TMySQL40Driver.GetProtoInfo(Handle: PZMySQLConnect): Cardinal;
begin
  Result := PDOLibraryMySQL40.mysql_get_proto_info(Handle);
end;

function TMySQL40Driver.GetServerInfo(Handle: PZMySQLConnect): AnsiString;
begin
  Result := AnsiString(PDOLibraryMySQL40.mysql_get_server_info(Handle));
end;

function TMySQL40Driver.GetThreadId(Handle: PZMySQLConnect): Cardinal;
begin
  Result := PDOLibraryMySQL40.mysql_thread_id(Handle);
end;

function TMySQL40Driver.Init(var Handle: PZMySQLConnect): PZMySQLConnect;
begin
  Handle := PDOLibraryMySQL40.mysql_init(nil);
  Result := Handle;
end;

procedure TMySQL40Driver.Dispose(var Handle: PZMySQLConnect);
begin
  Handle := nil;
end;

function TMySQL40Driver.RealConnect(Handle: PZMySQLConnect;
  const Host, User, Password, Db: PChar; Port: Cardinal; UnixSocket: PChar;
  ClientFlag: Cardinal): PZMySQLConnect;
begin
  Result := PDOLibraryMySQL40.mysql_real_connect(Handle, Host, User, Password, Db,
    Port, UnixSocket, ClientFlag);
end;

procedure TMySQL40Driver.SeekData(Res: PZMySQLResult; Offset: Cardinal);
begin
  PDOLibraryMySQL40.mysql_data_seek(Res, Offset);
end;

function TMySQL40Driver.SeekField(Res: PZMySQLResult; Offset: Cardinal): Cardinal;
begin
  Result := PDOLibraryMySQL40.mysql_field_seek(Res, Offset);
end;

function TMySQL40Driver.SeekRow(Res: PZMySQLResult;
  Row: PZMySQLRowOffset): PZMySQLRowOffset;
begin
  Result := PDOLibraryMySQL40.mysql_row_seek(Res, Row);
end;

function TMySQL40Driver.SelectDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_select_db(Handle, Database);
end;

function TMySQL40Driver.SetOptions(Handle: PZMySQLConnect;
  Option: TMySQLOption; const Arg: PChar): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_options(Handle,
    TMySqlOption(Option), Arg);
end;

function TMySQL40Driver.StoreResult(Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_store_result(Handle);
end;

function TMySQL40Driver.UseResult(Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL40.mysql_use_result(Handle);
end;

function TMySQL40Driver.GetLastErrorMessage(Handle: PZMySQLConnect): AnsiString;
begin
  Result := PDOLibraryMySQL40.mysql_error(Handle);
end;

function TMySQL40Driver.GetFieldType(Field: PZMySQLField): TMysqlFieldTypes;
begin
  Result := TMysqlFieldTypes(PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.fieldtype);
end;

function TMySQL40Driver.GetFieldFlags(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.flags;
end;

function TMySQL40Driver.GetRowCount(Res: PZMySQLResult): Int64;
begin
  Result := PDOLibraryMySQL40.PMYSQL_RES(Res).row_count;
end;

function TMySQL40Driver.GetStatus(Handle: PZMySQLConnect): TMySQLStatus;
begin
  Result := TMySQLStatus(PDOLibraryMySQL40.PMYSQL(Handle).status);
end;

function TMySQL40Driver.GetFieldCount(Res: PZMySQLResult): Integer;
begin
    if assigned (Res) then
        Result := PDOLibraryMySQL40.PMYSQL_RES(Res).field_count
    else
        Result := 0;
end;

function TMySQL40Driver.GetFieldDecimals(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.decimals;
end;

function TMySQL40Driver.GetFieldLength(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.length;
end;

function TMySQL40Driver.GetFieldMaxLength(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.max_length;
end;

function TMySQL40Driver.GetFieldName(Field: PZMySQLField): PChar;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.name;
end;

function TMySQL40Driver.GetFieldTable(Field: PZMySQLField): PChar;
begin
  Result := PDOLibraryMySQL40.PMYSQL_FIELD(Field)^.table;
end;

function TMySQL40Driver.GetFieldData(Row: PZMySQLRow; Offset: Cardinal): PChar;
begin
  Result := PDOLibraryMySQL40.PMYSQL_ROW(ROW)[Offset];
end;

function TMySQL40Driver.GetLastErrorCode(Handle: PZMySQLConnect): Integer;
begin
  Result := PDOLibraryMySQL40.mysql_errno(PDOLibraryMySQL40.PMYSQL(Handle));
end;

function TMySQL40Driver.SetAutocommit(Handle: PZMySQLConnect; mode: Boolean): Boolean;
var
    query: AnsiString;
    testResult: Integer;
begin
    if (mode = True) then
        query := 'AUTOCOMMIT=1'
    else
        query := 'AUTOCOMMIT=0';
    testResult := PDOLibraryMySQL40.mysql_query(PDOLibraryMySQL40.PMYSQL(Handle),pchar(query));
    Result := (testResult = 0);
end;

function TMySQL40Driver.Commit(Handle: PZMySQLConnect): Boolean;
var
    testResult: Integer;
begin
    testResult := PDOLibraryMySQL40.mysql_query(PDOLibraryMySQL40.PMYSQL(Handle),'COMMIT');
    Result := (testResult = 0);
end;

function TMySQL40Driver.Rollback (Handle: PZMySQLConnect): Boolean;
var
    testResult: Integer;
begin
    testResult := PDOLibraryMySQL40.mysql_query(PDOLibraryMySQL40.PMYSQL(Handle),'ROLLBACK');
    Result := (testResult = 0);
end;

function TMySQL40Driver.getLastInsertID (Handle: PZMySQLConnect): Int64;
begin
    Result := PDOLibraryMySQL40.mysql_insert_id(PDOLibraryMySQL40.PMYSQL(Handle));
end;

function TMySQL40Driver.getSQLState (Handle: PZMySQLConnect): AnsiString;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_sqlstate']);
    Result := '?????';  {mysql 4.0 doesn't support this}
end;

function TMySQL40Driver.InitializePrepStmt (Handle: PZMySQLConnect): PZMySqlPrepStmt;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_init']);
    result := nil;
end;

function TMySQL40Driver.ClosePrepStmt (PrepStmtHandle: PZMySqlPrepStmt): PZMySqlPrepStmt;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_close']);
    result := nil;
end;

function TMySQL40Driver.PrepareStmt (PrepStmtHandle: PZMySqlPrepStmt; const Query: PChar; Length: Integer): Integer;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_prepare']);
    Result := 1;
end;

function TMySQL40Driver.versionConversion (raw: Cardinal): AnsiString;
var
    WorkingStr, major, minor, subversion: AnsiString;
    shorty: Byte;
begin
    WorkingStr := IntToStr(raw);
    shorty := 1;
    if (length(WorkingStr) > 5) then shorty := 2;
    major := copy(WorkingStr,1,shorty);
    minor := copy(WorkingStr,3,2);
    subversion := copy(WorkingStr,5,2);
    if (major[1] = '0') then
        result := major[2]
    else
        result := major;
    if (minor[1] = '0') then
        result := result + '.' + minor[2]
    else
        result := result + '.' + minor;
    if (subversion[1] = '0') then
        result := result + '.' + subversion[2]
    else
        result := result + '.' + subversion;
end;

function TMySQL40Driver.GetClientVersion: AnsiString;
Begin
    Result := versionConversion (PDOLibraryMySQL40.mysql_get_client_version);
End;


function TMySQL40Driver.GetServerVersion (Handle: PZMySQLConnect): AnsiString;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_server_version'] );
    Result := 'N/A - driver unsupported';
End;

function TMySQL40Driver.GetLastPreparedError(Handle: PZMySqlPrepStmt):AnsiString;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_error'] );
    result := '';
end;

function TMySQL40Driver.GetLastPreparedErrorCode(Handle: PZMySqlPrepStmt):Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_errno']);
    result :=  0;
End;

function TMySQL40Driver.GetPreparedSQLState(Handle: PZMySqlPrepStmt): PChar;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_sql_state']);
    result := '?????';
end;

function TMySQL40Driver.GetNumRows(Res: PZMySQLResult): Int64;
Begin
    if (Res = nil) then
        Result := 0
    else
        Result :=  PDOLibraryMySQL40.mysql_num_rows (Res);
End;

function TMySQL40Driver.GetPreparedNumRows(Handle: PZMySqlPrepStmt): Int64;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_num_rows']);
    result := 0;
End;

function TMySQL40Driver.GetPreparedAffectedRows(Handle: PZMySqlPrepStmt): Int64;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_affected_rows']);
    result := 0;
End;

function TMySQL40Driver.GetPreparedInsertID(Handle: PZMySqlPrepStmt): Int64;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_insert_id']);
    result := 0;
End;

function TMySQL40Driver.StorePreparedResult (Handle: PZMySqlPrepStmt): Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_store_result']);
    result := 1;
End;

function TMySQL40Driver.GetPreparedMetaData (Handle: PZMySqlPrepStmt): PZMySQLResult;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_result_metadata']);
    result := nil;
End;

function TMySQL40Driver.GetPreparedFieldCount(Handle: PZMySqlPrepStmt): Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_field_count']);
    result := 0;
End;

function TMySQL40Driver.CheckAnotherRowset(Handle: PZMySQLConnect): Boolean;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_more_results']);
    result := false;
End;

function TMySQL40Driver.RetrieveNextRowset(Handle: PZMySQLConnect): Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_next_rowset']);
    result := 1;
End;

function TMySQL40Driver.BindParameters(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_bind_parameters']);
    result := 1;
End;

function TMySQL40Driver.BindResult(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_bind_result']);
    result := 1;
End;

function TMySQL40Driver.ExecuteStmt(Handle: PZMySqlPrepStmt): Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_execute']);
    result := 1;
End;

function TMySQL40Driver.GetPreparedBindMarkers (Handle: PZMySqlPrepStmt): Cardinal;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_param_count']);
    result := 0;
End;

function TMySQL40Driver.FetchBoundResults(Handle: PZMySqlPrepStmt): Integer;
Begin
    Exception.CreateFmt (SUnsupportedByDriver, ['mysql_stmt_fetch']);
    result := 1;
End;

function TMySQL40Driver.SetServerOption (Handle: PZMySQLConnect; option: TMysqlSetOption): Integer;
begin
    Exception.CreateFmt (SUnsupportedByDriver, ['SetServerOption']);
    result := 1;
end;

  
end.