{ $Id: PDODriverMySQL41.pas 52 2007-01-30 00:10:24Z jmarino $

############################################################################################

	Pascal Data Objects Library
	Copyright  2006 John Marino, http://www.synsport.com
	Project site: http://pdo.sourceforge.net

	Note: This file contains code fragments from Zeoslib
	Copyright  1999-2004 Sergey Seroukhov

############################################################################################

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

############################################################################################}



unit PDODriverMySQL41;

interface

{$I ../../../directives.inc}

uses
	SysUtils, PDOClasses, PDOInterfaces, PDOConstantsMySQL, PDOMySQLDriver, PDOLibraryMySQL41;

type
  TMySQL41Driver = class (TZMySQLBaseDriver, IPDOProtocol)
  private
    function versionConversion (raw: Cardinal): AnsiString;

  public
    function GetProtocol: AnsiString; override;
    function GetDescription: AnsiString; override;
    procedure Initialize; override;

    procedure Debug(Debug: PChar);
    function DumpDebugInfo(Handle: PZMySQLConnect): Integer;
    function Init(var Handle: PZMySQLConnect): PZMySQLConnect; override;
    procedure Dispose(var Handle: PZMySQLConnect);  override;

    function Connect(Handle: PZMySQLConnect;
      const Host, User, Password: PChar): PZMySQLConnect; override;
    function RealConnect(Handle: PZMySQLConnect;
      const Host, User, Password, Db: PChar; Port: Cardinal;
      UnixSocket: PChar; ClientFlag: Cardinal): PZMySQLConnect; override;
    procedure Close(Handle: PZMySQLConnect); override;

    function ExecQuery(Handle: PZMySQLConnect; const Query: PChar): Integer; override;
    function ExecRealQuery(Handle: PZMySQLConnect; const Query: PChar;
      Length: Integer): Integer; override;

    function SelectDatabase(Handle: PZMySQLConnect; const Database: PChar): Integer;
    function CreateDatabase(Handle: PZMySQLConnect;
      const Database: PChar): Integer;
    function DropDatabase(Handle: PZMySQLConnect;
      const Database: PChar): Integer;

    function GetStatInfo(Handle: PZMySQLConnect): PChar;
    function SetOptions(Handle: PZMySQLConnect; Option: TMySQLOption;
      const Arg: PChar): Integer; override;
    function GetEscapeString(StrTo, StrFrom: PChar; Length: Cardinal): Cardinal;

    function GetClientInfo: AnsiString; override;
    function GetHostInfo(Handle: PZMySQLConnect): PChar;
    function GetProtoInfo(Handle: PZMySQLConnect): Cardinal;
    function GetThreadId(Handle: PZMySQLConnect): Cardinal;

    function GetListDatabases(Handle: PZMySQLConnect;
      Wild: PChar): PZMySQLResult;
    function GetListTables(Handle: PZMySQLConnect;
      const Wild: PChar): PZMySQLResult;
    function GetListFields(Handle: PZMySQLConnect;
      const Table, Wild: PChar): PZMySQLResult;
    function GetListProcesses(Handle: PZMySQLConnect): PZMySQLResult;

    function StoreResult(Handle: PZMySQLConnect): PZMySQLResult; override;
    function UseResult(Handle: PZMySQLConnect): PZMySQLResult; override;
    procedure FreeResult(Res: PZMySQLResult);  override;
    function GetAffectedRows(Handle: PZMySQLConnect): Int64; override;

    function FetchRow(Res: PZMySQLResult): PZMySQLRow;  override;
    function FetchLengths(Res: PZMySQLResult): PLongInt;  override;
    function FetchField(Res: PZMySQLResult): PZMySQLField;  override;

    procedure SeekData(Res: PZMySQLResult; Offset: Cardinal);  override;
    function SeekRow(Res: PZMySQLResult; Row: PZMySQLRowOffset): PZMySQLRowOffset;  override;
    function SeekField(Res: PZMySQLResult; Offset: Cardinal): Cardinal;   override;

    function GetFieldType(Field: PZMySQLField): TMysqlFieldTypes;  override;
    function GetFieldFlags(Field: PZMySQLField): Integer;  override;
    function GetStatus(Handle: PZMySQLConnect): TMySQLStatus;  override;
    function GetRowCount(Res: PZMySQLResult): Int64;   override;
    function GetFieldCount(Res: PZMySQLResult): Integer;    override;
    function GetFieldName(Field: PZMySQLField): PChar;   override;
    function GetFieldTable(Field: PZMySQLField): PChar;   override;
    function GetFieldLength(Field: PZMySQLField): Integer;  override;
    function GetFieldMaxLength(Field: PZMySQLField): Integer;   override;
    function GetFieldDecimals(Field: PZMySQLField): Integer;    override;
    function GetFieldData(Row: PZMySQLRow; Offset: Cardinal): PChar;  override;


    function SetAutocommit (Handle: PZMySQLConnect; mode: Boolean): Boolean;  override;
    function Commit (Handle: PZMySQLConnect): Boolean;   override;
    function Rollback (Handle: PZMySQLConnect): Boolean;  override;
    function InitializePrepStmt (Handle: PZMySQLConnect): PZMySqlPrepStmt;  override;
    function ClosePrepStmt (PrepStmtHandle: PZMySqlPrepStmt): PZMySqlPrepStmt;  override;
    function PrepareStmt (PrepStmtHandle: PZMySqlPrepStmt; const Query: PChar; Length: Integer): Integer;  override;
    function GetClientVersion: AnsiString; override;
    function GetNumRows(Res: PZMySQLResult): Int64;  override;
    function GetLastPreparedError(Handle: PZMySqlPrepStmt): AnsiString;  override;
    function GetLastPreparedErrorCode(Handle: PZMySqlPrepStmt): Integer;  override;
    function GetPreparedSQLState (Handle: PZMySqlPrepStmt): PChar;   override;
    function GetPreparedNumRows (Handle: PZMySqlPrepStmt): Int64;  override;
    function GetPreparedAffectedRows (Handle: PZMySqlPrepStmt): Int64;   override;
    function GetPreparedInsertID (Handle: PZMySqlPrepStmt): Int64;   override;
    function StorePreparedResult (Handle: PZMySqlPrepStmt): Integer;   override;
    function GetPreparedMetaData (Handle: PZMySqlPrepStmt): PZMySQLResult;  override;
    function GetPreparedFieldCount(Handle: PZMySqlPrepStmt): Integer;  override;
    function CheckAnotherRowset   (Handle: PZMySQLConnect): Boolean;  override;
    function RetrieveNextRowset   (Handle: PZMySQLConnect): Integer;  override;
    function BindParameters (Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;  override;
    function ExecuteStmt (Handle: PZMySqlPrepStmt): Integer;   override;
    function GetPreparedBindMarkers (Handle: PZMySqlPrepStmt): Cardinal;  override;
    function FetchBoundResults (Handle: PZMySqlPrepStmt): Integer;   override;
    function BindResult (Handle: PZMySqlPrepStmt;  bindArray: PZMysqlBindArray): Byte;  override;
    function SetServerOption (Handle: PZMySQLConnect; option: TMysqlSetOption): Integer; override;

    {MOVED FROM IPDOPROTOCOL TO BASE DRIVERS}
    function GetSQLState         (Handle: PZMySQLConnect): AnsiString; override;
    function GetLastErrorMessage (Handle: PZMySQLConnect): AnsiString; override;
    function GetLastErrorCode    (Handle: PZMySQLConnect): Integer;    override;
    function GetLastInsertID     (Handle: PZMySQLConnect): Int64;      override;
    function GetServerVersion    (Handle: PZMySQLConnect): AnsiString; override;
    function GetServerInfo       (Handle: PZMySQLConnect): AnsiString; override;
  end;
  




implementation


function TMySQL41Driver.GetProtocol: AnsiString;
begin
  Result := 'mysql-4.1';
end;

function TMySQL41Driver.GetDescription: AnsiString;
begin
  Result := 'Native driver for MySQL 4.1.16';
end;

procedure TMySQL41Driver.Initialize;
begin
  PDOLibraryMySQL41.LibraryLoader.LoadIfNeeded;
  {PDOLibraryMySQL41.mysql_server_init};
end;

procedure TMySQL41Driver.Close(Handle: PZMySQLConnect);
begin
  PDOLibraryMySQL41.mysql_close(Handle);
//  PDOLibraryMySQL41.mysql_server_end;
end;

function TMySQL41Driver.Connect(Handle: PZMySQLConnect; const Host,
  User, Password: PChar): PZMySQLConnect;
begin
  Result := PDOLibraryMySQL41.mysql_connect(Handle, Host, User, Password);
end;

function TMySQL41Driver.CreateDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_create_db(Handle, Database);
end;

procedure TMySQL41Driver.Debug(Debug: PChar);
begin
  PDOLibraryMySQL41.mysql_debug(Debug);
end;

function TMySQL41Driver.DropDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_drop_db(Handle, Database);
end;

function TMySQL41Driver.DumpDebugInfo(Handle: PZMySQLConnect): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_dump_debug_info(Handle);
end;

function TMySQL41Driver.ExecQuery(Handle: PZMySQLConnect;
  const Query: PChar): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_query(Handle, Query);
end;

function TMySQL41Driver.ExecRealQuery(Handle: PZMySQLConnect;
  const Query: PChar; Length: Integer): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_real_query(Handle, Query, Length);
end;

function TMySQL41Driver.FetchField(Res: PZMySQLResult): PZMySQLField;
begin
  Result := PDOLibraryMySQL41.mysql_fetch_field(Res);
end;

function TMySQL41Driver.FetchLengths(Res: PZMySQLResult): PLongInt;
begin
  Result := PDOLibraryMySQL41.mysql_fetch_lengths(Res);
end;

function TMySQL41Driver.FetchRow(Res: PZMySQLResult): PZMySQLRow;
begin
    if assigned(Res) then
        Result := PDOLibraryMySQL41.mysql_fetch_row(Res)
    else
        Result := nil;
end;

procedure TMySQL41Driver.FreeResult(Res: PZMySQLResult);
begin
  PDOLibraryMySQL41.mysql_free_result(Res);
end;

function TMySQL41Driver.GetAffectedRows(Handle: PZMySQLConnect): Int64;
begin
  Result := PDOLibraryMySQL41.mysql_affected_rows(Handle);
end;

function TMySQL41Driver.GetClientInfo: AnsiString;
begin
  Result := AnsiString(PDOLibraryMySQL41.mysql_get_client_info);
end;

function TMySQL41Driver.GetEscapeString(StrTo, StrFrom: PChar;
  Length: Cardinal): Cardinal;
begin
  Result := PDOLibraryMySQL41.mysql_escape_string(StrTo, StrFrom, Length);
end;

function TMySQL41Driver.GetHostInfo(Handle: PZMySQLConnect): PChar;
begin
  Result := PDOLibraryMySQL41.mysql_get_host_info(Handle);
end;

function TMySQL41Driver.GetListDatabases(Handle: PZMySQLConnect;
  Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_list_dbs(Handle, Wild);
end;

function TMySQL41Driver.GetListFields(Handle: PZMySQLConnect;
  const Table, Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_list_fields(Handle, Table, Wild);
end;

function TMySQL41Driver.GetListProcesses(
  Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_list_processes(Handle);
end;

function TMySQL41Driver.GetListTables(Handle: PZMySQLConnect;
  const Wild: PChar): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_list_tables(Handle, Wild);
end;

function TMySQL41Driver.GetProtoInfo(Handle: PZMySQLConnect): Cardinal;
begin
  Result := PDOLibraryMySQL41.mysql_get_proto_info(Handle);
end;

function TMySQL41Driver.GetServerInfo(Handle: PZMySQLConnect): AnsiString;
begin
  Result := AnsiString(PDOLibraryMySQL41.mysql_get_server_info(Handle));
end;

function TMySQL41Driver.GetStatInfo(Handle: PZMySQLConnect): PChar;
begin
  Result := PDOLibraryMySQL41.mysql_stat(Handle);
end;

function TMySQL41Driver.GetThreadId(Handle: PZMySQLConnect): Cardinal;
begin
  Result := PDOLibraryMySQL41.mysql_thread_id(Handle);
end;

function TMySQL41Driver.Init(var Handle: PZMySQLConnect): PZMySQLConnect;
begin
  Handle := PDOLibraryMySQL41.mysql_init(nil);
  Result := Handle;
end;

procedure TMySQL41Driver.Dispose(var Handle: PZMySQLConnect);
begin
  Handle := nil;
end;

function TMySQL41Driver.RealConnect(Handle: PZMySQLConnect;
  const Host, User, Password, Db: PChar; Port: Cardinal; UnixSocket: PChar;
  ClientFlag: Cardinal): PZMySQLConnect;
begin
  Result := PDOLibraryMySQL41.mysql_real_connect(Handle, Host, User, Password, Db,
    Port, UnixSocket, ClientFlag);
end;

procedure TMySQL41Driver.SeekData(Res: PZMySQLResult;
  Offset: Cardinal);
begin
  PDOLibraryMySQL41.mysql_data_seek(Res, Offset);
end;

function TMySQL41Driver.SeekField(Res: PZMySQLResult;
  Offset: Cardinal): Cardinal;
begin
  Result := PDOLibraryMySQL41.mysql_field_seek(Res, Offset);
end;

function TMySQL41Driver.SeekRow(Res: PZMySQLResult;
  Row: PZMySQLRowOffset): PZMySQLRowOffset;
begin
  Result := PDOLibraryMySQL41.mysql_row_seek(Res, Row);
end;

function TMySQL41Driver.SelectDatabase(Handle: PZMySQLConnect;
  const Database: PChar): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_select_db(Handle, Database);
end;

function TMySQL41Driver.SetOptions(Handle: PZMySQLConnect;
  Option: TMySQLOption; const Arg: PChar): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_options(Handle,
    TMySqlOption(Option), Arg);
end;

function TMySQL41Driver.StoreResult(
  Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_store_result(Handle);
end;

function TMySQL41Driver.UseResult(Handle: PZMySQLConnect): PZMySQLResult;
begin
  Result := PDOLibraryMySQL41.mysql_use_result(Handle);
end;

function TMySQL41Driver.GetLastErrorMessage(Handle: PZMySQLConnect): AnsiString;
begin
  Result := PDOLibraryMySQL41.mysql_error(Handle);
end;

function TMySQL41Driver.GetFieldType(Field: PZMySQLField): TMysqlFieldTypes;
begin
  Result := TMysqlFieldTypes(PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.fieldtype);
end;

function TMySQL41Driver.GetFieldFlags(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.flags;
end;

function TMySQL41Driver.GetRowCount(Res: PZMySQLResult): Int64;
begin
  Result := PDOLibraryMySQL41.PMYSQL_RES(Res).row_count;
end;

function TMySQL41Driver.GetStatus(Handle: PZMySQLConnect): TMySQLStatus;
begin
  Result := TMySQLStatus(PDOLibraryMySQL41.PMYSQL(Handle).status);
end;

function TMySQL41Driver.GetFieldCount(Res: PZMySQLResult): Integer;
begin
    if assigned(Res) then
        Result := PDOLibraryMySQL41.PMYSQL_RES(Res).field_count
    else
        Result := 0;
end;

function TMySQL41Driver.GetFieldDecimals(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.decimals;
end;

function TMySQL41Driver.GetFieldLength(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.length;
end;

function TMySQL41Driver.GetFieldMaxLength(Field: PZMySQLField): Integer;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.max_length;
end;

function TMySQL41Driver.GetFieldName(Field: PZMySQLField): PChar;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.name;
end;

function TMySQL41Driver.GetFieldTable(Field: PZMySQLField): PChar;
begin
  Result := PDOLibraryMySQL41.PMYSQL_FIELD(Field)^.table;
end;

function TMySQL41Driver.GetFieldData(Row: PZMySQLRow;
  Offset: Cardinal): PChar;
begin
  Result := PDOLibraryMySQL41.PMYSQL_ROW(ROW)[Offset];
end;

function TMySQL41Driver.GetLastErrorCode(Handle: PZMySQLConnect): Integer;
begin
  Result := PDOLibraryMySQL41.mysql_errno(PDOLibraryMySQL41.PMYSQL(Handle));
end;

function TMySQL41Driver.SetAutocommit(Handle: PZMySQLConnect; mode: Boolean): Boolean;
var
    my_bool, my_mode: Byte;
begin
    if (mode = True) then
        my_mode := 1
    else
        my_mode := 0;
    my_bool := PDOLibraryMySQL41.mysql_autocommit(PDOLibraryMySQL41.PMYSQL(Handle), my_mode);
    Result := (my_bool = 0);
end;

function TMySQL41Driver.Commit(Handle: PZMySQLConnect): Boolean;
var
    my_bool: Byte;
begin
    my_bool := PDOLibraryMySQL41.mysql_commit(PDOLibraryMySQL41.PMYSQL(Handle));
    Result := (my_bool = 0);
end;

function TMySQL41Driver.Rollback (Handle: PZMySQLConnect): Boolean;
var
    my_bool: Byte;
begin
    my_bool := PDOLibraryMySQL41.mysql_rollback(PDOLibraryMySQL41.PMYSQL(Handle));
    Result := (my_bool = 0);
end;

function TMySQL41Driver.getLastInsertID (Handle: PZMySQLConnect): Int64;
begin
    Result := PDOLibraryMySQL41.mysql_insert_id(PDOLibraryMySQL41.PMYSQL(Handle));
end;

function TMySQL41Driver.getSQLState (Handle: PZMySQLConnect): AnsiString;
begin
    Result := PDOLibraryMySQL41.mysql_sqlstate (PDOLibraryMySQL41.PMYSQL(Handle));
end;


function TMySQL41Driver.InitializePrepStmt (Handle: PZMySQLConnect): PZMySqlPrepStmt;
begin
    Result := PDOLibraryMySQL41.mysql_stmt_init(PDOLibraryMySQL41.PMYSQL(Handle));
end;

function TMySQL41Driver.ClosePrepStmt (PrepStmtHandle: PZMySqlPrepStmt): PZMySqlPrepStmt;
var
    my_bool: Byte;
begin
    my_bool := PDOLibraryMySQL41.mysql_stmt_close(PDOLibraryMySQL41.PMYSQL_STMT(PrepStmtHandle));
    if (my_bool = 0) then
        Result := nil
    else
        Result := PrepStmtHandle;
end;

function TMySQL41Driver.PrepareStmt (PrepStmtHandle: PZMySqlPrepStmt; const Query: PChar; Length: Integer): Integer;
begin
    Result := PDOLibraryMySQL41.mysql_stmt_prepare(PDOLibraryMySQL41.PMYSQL_STMT(PrepStmtHandle), Query, Length);
end;

function TMySQL41Driver.versionConversion (raw: Cardinal): AnsiString;
var
    WorkingStr, major, minor, subversion: AnsiString;
    shorty: Byte;
begin
    WorkingStr := IntToStr(raw);
    shorty := 1;
    if (length(WorkingStr) > 5) then shorty := 2;
    major := copy(WorkingStr,1,shorty);
    minor := copy(WorkingStr,3,2);
    subversion := copy(WorkingStr,5,2);
    if (major[1] = '0') then
        result := major[2]
    else
        result := major;
    if (minor[1] = '0') then
        result := result + '.' + minor[2]
    else
        result := result + '.' + minor;
    if (subversion[1] = '0') then
        result := result + '.' + subversion[2]
    else
        result := result + '.' + subversion;
end;

function TMySQL41Driver.GetClientVersion: AnsiString;
Begin
    Result := versionConversion (PDOLibraryMySQL41.mysql_get_client_version);
End;


function TMySQL41Driver.GetServerVersion (Handle: PZMySQLConnect): AnsiString;
Begin
    Result := versionConversion (PDOLibraryMySQL41.mysql_get_server_version(PDOLibraryMySQL41.PMYSQL(Handle)));
End;

function TMySQL41Driver.GetLastPreparedError(Handle: PZMySqlPrepStmt):AnsiString;
begin
    Result := PDOLibraryMySQL41.mysql_stmt_error(PDOLibraryMySQL41.PMYSQL_STMT(Handle));
end;

function TMySQL41Driver.GetLastPreparedErrorCode(Handle: PZMySqlPrepStmt):Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_errno(PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedSQLState(Handle: PZMySqlPrepStmt): PChar;
begin
    Result := PDOLibraryMySQL41.mysql_stmt_sqlstate (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
end;

function TMySQL41Driver.GetNumRows(Res: PZMySQLResult): Int64;
Begin
    if (Res = nil) then
        Result := 0
    else
        Result :=  PDOLibraryMySQL41.mysql_num_rows (Res);
End;

function TMySQL41Driver.GetPreparedNumRows(Handle: PZMySqlPrepStmt): Int64;
Begin
    if (Handle = nil) then
        Result := 0
    else
        Result :=  PDOLibraryMySQL41.mysql_stmt_num_rows (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedAffectedRows(Handle: PZMySqlPrepStmt): Int64;
Begin
    Result :=  PDOLibraryMySQL41.mysql_stmt_affected_rows (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedInsertID(Handle: PZMySqlPrepStmt): Int64;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_insert_id (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.StorePreparedResult (Handle: PZMySqlPrepStmt): Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_store_result (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedMetaData (Handle: PZMySqlPrepStmt): PZMySQLResult;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_result_metadata (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedFieldCount(Handle: PZMySqlPrepStmt): Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_field_count(PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.CheckAnotherRowset(Handle: PZMySQLConnect): Boolean;
var
    my_bool: Byte;
Begin
    my_bool :=  PDOLibraryMySQL41.mysql_more_results (PDOLibraryMySQL41.PMYSQL(Handle));
    if (my_bool = 0) then
        Result := False
    else
        Result := True;
End;

function TMySQL41Driver.RetrieveNextRowset(Handle: PZMySQLConnect): Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_next_result (PDOLibraryMySQL41.PMYSQL(Handle));
End;

function TMySQL41Driver.BindParameters(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_bind_param (PDOLibraryMySQL41.PMYSQL_STMT(Handle), PMYSQL_BIND(bindArray));
End;

function TMySQL41Driver.BindResult(Handle: PZMySqlPrepStmt; bindArray: PZMysqlBindArray): Byte;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_bind_result (PDOLibraryMySQL41.PMYSQL_STMT(Handle), PMYSQL_BIND(bindArray));
End;

function TMySQL41Driver.ExecuteStmt(Handle: PZMySqlPrepStmt): Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_execute (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.GetPreparedBindMarkers (Handle: PZMySqlPrepStmt): Cardinal;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_param_count (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.FetchBoundResults(Handle: PZMySqlPrepStmt): Integer;
Begin
    Result := PDOLibraryMySQL41.mysql_stmt_fetch (PDOLibraryMySQL41.PMYSQL_STMT(Handle));
End;

function TMySQL41Driver.SetServerOption (Handle: PZMySQLConnect; option: TMysqlSetOption): Integer;
begin
    Result := PDOLibraryMySQL41.mysql_set_server_option (Handle, option);
end;

end.